package cn.superdesk.uniorder.common.api;

import javax.crypto.Cipher;
import javax.crypto.spec.SecretKeySpec;
import java.util.Base64;

public class AESUtil {
    private static final int KEY_LENGTH = 16;
    private static final String Fill_STR = "0";
    private static final String UTF_8 = "utf-8";
    private static final String ALGORITHM = "AES";

    //算法/模式/补码方式
    private static final String TRANSFORMATION = "AES/ECB/PKCS5Padding";

    /**
     * 加密
     *
     * @param sSrc 原始字符串
     * @param sKey 秘钥
     * @return 加密后的字符串
     * @throws Exception
     */
    public static String Encrypt(String sSrc, String sKey) throws Exception {
        if (sKey == null) {
            System.out.println("Key为空null");
            return null;
        }
        // 判断Key是否为16位
        if (sKey.length() != KEY_LENGTH) {
            System.out.println("Key长度不是16位");
            return null;
        }
        byte[] raw = sKey.getBytes(UTF_8);
        SecretKeySpec skeySpec = new SecretKeySpec(raw, ALGORITHM);
        Cipher cipher = Cipher.getInstance(TRANSFORMATION);//"算法/模式/补码方式"
        cipher.init(Cipher.ENCRYPT_MODE, skeySpec);
        byte[] encrypted = cipher.doFinal(sSrc.getBytes(UTF_8));

        return Base64.getEncoder().encodeToString(encrypted);//此处使用BASE64做转码功能，同时能起到2次加密的作用。
    }


    /**
     * 解密
     *
     * @param sSrc 加密后的字符串
     * @param sKey 秘钥
     * @return 解密后的字符串
     * @throws Exception
     */
    public static String Decrypt(String sSrc, String sKey) throws Exception {
        try {
            // 判断Key是否正确
            if (sKey == null) {
                System.out.println("Key为空null");
                return null;
            }
            // 判断Key是否为16位
            if (sKey.length() != KEY_LENGTH) {
                System.out.println("Key长度不是16位");
                return null;
            }
            byte[] raw = sKey.getBytes(UTF_8);
            SecretKeySpec skeySpec = new SecretKeySpec(raw, ALGORITHM);
            Cipher cipher = Cipher.getInstance(TRANSFORMATION);
            cipher.init(Cipher.DECRYPT_MODE, skeySpec);
            byte[] encrypted1 = org.apache.commons.codec.binary.Base64.decodeBase64(sSrc);//先用base64解密
            try {
                byte[] original = cipher.doFinal(encrypted1);
                String originalString = new String(original, UTF_8);
                return originalString;
            } catch (Exception e) {
                System.out.println(e.toString());
                return null;
            }
        } catch (Exception ex) {
            System.out.println(ex.toString());
            ex.printStackTrace();
            return null;
        }
    }

    /**
     * 生成16位的Key
     *
     * @param sk 原始sk
     * @return 生成16位后的key
     * @throws Exception
     */
    public static String generateKeyWith16FromSK(String sk) throws Exception {
        try {
            // 判断Key是否正确
            if (sk == null) {
                System.out.println("sk为null");
                return null;
            }
            // 判断Key是否为16位
            if (sk.length() > KEY_LENGTH) {
                //大于16位,截取前16位
                return sk.substring(0, KEY_LENGTH);
            } else if (sk.length() == KEY_LENGTH) {
                // 16位,返回
                return sk;
            } else {
                // 小于16位,补位至16位
                return addZeroForNum(sk, KEY_LENGTH);
            }
        } catch (Exception ex) {
            System.out.println(ex.toString());
            return null;
        }
    }

    /**
     * 补位
     *
     * @param str
     * @param strLength
     * @return
     */
    public static String addZeroForNum(String str, int strLength) {
        int strLen = str.length();
        if (strLen < strLength) {
            while (strLen < strLength) {
                StringBuffer sb = new StringBuffer();
                sb.append(str).append(Fill_STR);//右补0
                str = sb.toString();
                strLen = str.length();
            }
        }
        return str;
    }

    public static void main(String[] args) throws Exception {
        /*
         * 此处使用AES-128-ECB加密模式，key需要为16位。
         */
        String sk1 = "5FiVgBXvn61pUMDPbwwZgaEyA8tEgQ==";
        String sk2 = "5FiVgBXvn";
        String sk3 = "5FiVgBXvn61pUMDP";
        String sk4 = null;

        String cKey1 = generateKeyWith16FromSK(sk1);
        String cKey2 = generateKeyWith16FromSK(sk2);
        String cKey3 = generateKeyWith16FromSK(sk3);
        String cKey4 = generateKeyWith16FromSK(sk4);

        System.out.printf("sk -> {%s}  , ckey -> {%s}\n", sk1, cKey1);
        System.out.printf("sk -> {%s}  , ckey -> {%s}\n", sk2, cKey2);
        System.out.printf("sk -> {%s}  , ckey -> {%s}\n", sk3, cKey3);
        System.out.printf("sk -> {%s}  , ckey -> {%s}\n", sk4, cKey4);

        // 需要加密的字串
        String cSrc = "{\n" +
                "    \"fullName\":\"888888公司\",\n" +
                "    \"updateTime\":\"20220921 11:53:04\",\n" +
                "    \"propertyCode\":\"1\",\n" +
                "    \"clientCode\":\"888888\",\n" +
                "    \"customerId\":\"888888\",\n" +
                "    \"status\":\"0\"\n" +
                "}";
        System.out.println(cSrc);
        // 加密
        String enString = AESUtil.Encrypt(cSrc, cKey1);
        System.out.printf("加密后的字串是：{%s}\n", enString);

        // 解密
        String deString = AESUtil.Decrypt(enString, cKey1);
        System.out.printf("解密后的字串是：{%s}\n", deString);

        // 解密失败
        String deString2 = AESUtil.Decrypt(enString, cKey2);
        System.out.printf("解密后的字串是：{%s}\n", deString2);
    }
}
