package cn.superdesk.uniorder.common.api;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.digest.HmacAlgorithms;
import org.apache.commons.codec.digest.HmacUtils;
import org.apache.commons.codec.digest.Md5Crypt;
import org.apache.http.HttpEntity;
import org.apache.http.HttpHeaders;
import org.apache.http.NameValuePair;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.client.methods.HttpRequestBase;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import java.io.ByteArrayOutputStream;
import java.io.UnsupportedEncodingException;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.security.GeneralSecurityException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.security.cert.X509Certificate;
import java.util.Base64;
import java.util.Comparator;
import java.util.List;
import java.util.Objects;

@Slf4j
public class APIUtil {
    private static final String APPKEY_HEADER = "X-AURORA-APPKEY";
    private static final String SIGN_HEADER = "X-AURORA-SIGN";
    private static final String TIMESTAMP_HEADER = "X-AURORA-TIMESTAMP";
    private static final String CONTENT_MD5 = "CONTENT-MD5";

    public static String post(String url, String body, String appKey, String appSecret)
            throws Exception {
        // 获得Http客户端
        CloseableHttpClient httpClient = HttpClientBuilder.create()
                .setSSLContext(getAllTrustedSslContext())
                .setSSLHostnameVerifier(getAllTrustedVerifier()).build();

        // 创建post请求
        HttpPost httpPost = new HttpPost(url);

        String key = AESUtil.generateKeyWith16FromSK(appSecret);
        // 加密body
        String encryptBody = AESUtil.Encrypt(body, key);
        httpPost.setEntity(new StringEntity(encryptBody, Charset.forName("UTF-8")));
        httpPost.setHeader(HttpHeaders.CONTENT_TYPE, "application/json; charset=UTF-8");
        doSign(httpPost, body.getBytes(), appKey, appSecret);

        // 由客户端执行(发送)post请求
        CloseableHttpResponse response = httpClient.execute(httpPost);
        // 从响应模型中获取响应实体
        HttpEntity responseEntity = response.getEntity();
        String source = streamToString(responseEntity);
        log.info("解密前={}",source);
        String decrypt = AESUtil.Decrypt(source, key);
        log.info("解密后={}",decrypt);
        // 释放资源
        if (httpClient != null) {
            httpClient.close();
        }
        if (response != null) {
            response.close();
        }
        return decrypt;
    }

    public static String post(String url, String body) throws Exception {
        // 获得Http客户端
        CloseableHttpClient httpClient = HttpClientBuilder.create()
                .setSSLContext(getAllTrustedSslContext())
                .setSSLHostnameVerifier(getAllTrustedVerifier()).build();
        // 创建post请求
        HttpPost httpPost = new HttpPost(url);

        httpPost.setEntity(new StringEntity(body, StandardCharsets.UTF_8));
        httpPost.setHeader(HttpHeaders.CONTENT_TYPE, "application/json; charset=UTF-8");

        // 由客户端执行(发送)post请求
        CloseableHttpResponse response = httpClient.execute(httpPost);
        // 从响应模型中获取响应实体
        HttpEntity responseEntity = response.getEntity();
        return streamToString(responseEntity);
    }

    private static SSLContext getAllTrustedSslContext() throws GeneralSecurityException {
        TrustManager[] trustAllCerts = new TrustManager[]{new X509TrustManager() {
            @Override
            public X509Certificate[] getAcceptedIssuers() {
                return new X509Certificate[0];
            }

            @Override
            public void checkClientTrusted(X509Certificate[] chain, String authType) {
            }

            @Override
            public void checkServerTrusted(X509Certificate[] chain, String authType) {
            }
        }};

        SSLContext context = SSLContext.getInstance("TLS");
        context.init(null, trustAllCerts, new SecureRandom());
        return context;
    }

    private static HostnameVerifier getAllTrustedVerifier() {
        return (hostname, session) -> true;
    }

    private static void doSign(HttpRequestBase httpRequestBase, byte[] body, String appKey, String appSecret)
            throws NoSuchAlgorithmException, UnsupportedEncodingException, InvalidKeyException {
        String method = httpRequestBase.getMethod().toUpperCase();
        URIBuilder uriBuilder = new URIBuilder(httpRequestBase.getURI());
        String queryString = queryJoin(uriBuilder.getQueryParams());


        long timestamp = System.currentTimeMillis();
        // System.currentTimeMillis();
        // 构造待签字符串
        String stringToSign = null;

        if (httpRequestBase instanceof HttpPost || httpRequestBase instanceof HttpPut) {
            String contentMd5 = null == body || body.length == 0 ? "" :
                    Base64.getEncoder().encodeToString(Md5Crypt.md5Crypt(body).getBytes());

            httpRequestBase.setHeader(CONTENT_MD5, contentMd5);
            stringToSign = method + "\n" + queryString + "\n" + timestamp + "\n" + contentMd5;
        } else {
            stringToSign = method + "\n" + queryString + "\n" + timestamp;
        }

        String signature = Base64.getEncoder().encodeToString(new HmacUtils(HmacAlgorithms.HMAC_SHA_256, appSecret).hmacHex(stringToSign).getBytes());
        httpRequestBase.setHeader(APPKEY_HEADER, appKey);
        httpRequestBase.setHeader(SIGN_HEADER, signature);
        httpRequestBase.setHeader(TIMESTAMP_HEADER, String.valueOf(timestamp));

        log.info("timestamp={}",timestamp);
    }

    private static String queryJoin(List<NameValuePair> query) {
        if (null == query || query.isEmpty()) {
            return "";
        }

        String split = "";
        StringBuilder stringBuilder = new StringBuilder();
        query.sort(Comparator.comparing(NameValuePair::getName));
        for (NameValuePair entry : query) {
            stringBuilder.append(split).append(entry.getName());
            if (!Objects.isNull(entry.getValue())) {
                stringBuilder.append("=").append(entry.getValue());
                split = "&";
            }
        }
        return stringBuilder.toString();
    }

    private static String streamToString(HttpEntity httpEntity) throws Exception {
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        httpEntity.writeTo(outputStream);
        return outputStream.toString();
    }

    public static void main(String[] args) {
        String appKey = "v5btDnbis7m4xkEa";
        String appSecret = "w1X1HeKrWazfxSKD+ZSsoxQyM5pu2g==";
        String body = "{\"supplierCode\":\"1111\"}";
        String url = "http://10.252.21.5:8925/test2/dev/mdm/data_support_supplier_select_s";

        try {
            APIUtil.post(url, body, appKey, appSecret);
        } catch (Exception e) {
            e.printStackTrace();
        }

    }
}
