package cn.superdesk.uniorder.common.enums.accounting.customer;


import java.math.BigDecimal;
import java.util.EnumSet;
import java.util.HashMap;
import java.util.Map;

/**
 * @Description: 对账单税率类型
 */
public enum AccountingTaxTypeEnum {

	YBNS01(BigDecimal.valueOf(0), "YBNS01","一般纳税"),
	YBNS06(BigDecimal.valueOf(6), "YBNS02","一般纳税"),
	YBNS09(BigDecimal.valueOf(9), "YBNS03","一般纳税"),
	YBNS13(BigDecimal.valueOf(13), "YBNS04","一般纳税"),
	YBNS03(BigDecimal.valueOf(3), "JYJS02","一般纳税"),
	YBNS05(BigDecimal.valueOf(4), "JYJS01","一般纳税"),
	;

	private BigDecimal key;
	private String  desc;
	private String  remark;

	AccountingTaxTypeEnum(BigDecimal key, String desc,String  remark) {
		this.key = key;
		this.desc = desc;
		this.remark = remark;
	}

	public BigDecimal getKey() {
		return key;
	}

	public String getDesc() {
		return desc;
	}

	public String getRemark() {
		return remark;
	}

	public static String getDesc(BigDecimal key) {
		for (AccountingTaxTypeEnum c : AccountingTaxTypeEnum.values()) {
			if (c.key.compareTo(key) == 0) {
				return c.getDesc();
			}
		}
		return null;
	}
	public static AccountingTaxTypeEnum getEnum(BigDecimal key) {
		for (AccountingTaxTypeEnum c : AccountingTaxTypeEnum.values()) {
			if (c.key.compareTo(key) == 0) {
				return c;
			}
		}
		return null;
	}

	private static final Map<BigDecimal, AccountingTaxTypeEnum> LOOKUP = new HashMap<>();

	static {
		for (AccountingTaxTypeEnum s : EnumSet.allOf(AccountingTaxTypeEnum.class)) {
			LOOKUP.put(s.getKey(), s);
		}
	}

	public static AccountingTaxTypeEnum get(BigDecimal key) {
		return LOOKUP.get(key);
	}
}
