package cn.superdesk.uniorder.common.enums.common;

import java.util.EnumSet;
import java.util.HashMap;
import java.util.Map;

/**
 * 报账单类型枚举
 */
public enum CommonReportAccountingTypeEnum {

    ZERO(0, "应收报账单（非微信）", CommonReportAccountingDataTypeEnum.ZERO),

    ONE(1, "应收报账单（微信）", CommonReportAccountingDataTypeEnum.ONE),
    TWO(2, "应付报账单", CommonReportAccountingDataTypeEnum.ZERO),

    THREE(3, "实收报账单（非微信）", CommonReportAccountingDataTypeEnum.ZERO),
    FOUR(4, "实收报账单（微信）", CommonReportAccountingDataTypeEnum.ONE),
    FIVE(5, "付款报账单", CommonReportAccountingDataTypeEnum.ZERO),
    ;

    private final Integer key;
    private final String  desc;

    private final CommonReportAccountingDataTypeEnum dataType;

    CommonReportAccountingTypeEnum(Integer key, String desc, CommonReportAccountingDataTypeEnum dataType) {
        this.key = key;
        this.desc = desc;
        this.dataType = dataType;
    }

    public Integer getKey() {
        return key;
    }

    public String getDesc() {
        return desc;
    }

    public CommonReportAccountingDataTypeEnum getDataType() {
        return dataType;
    }

    public static String getDesc(Integer key) {
        for (CommonReportAccountingTypeEnum c : CommonReportAccountingTypeEnum.values()) {
            if (c.key.equals(key)) {
                return c.getDesc();
            }
        }
        return null;
    }

    private static final Map<Integer, CommonReportAccountingTypeEnum> LOOKUP = new HashMap<>();

    static {
        for (CommonReportAccountingTypeEnum s : EnumSet.allOf(CommonReportAccountingTypeEnum.class)) {
            LOOKUP.put(s.key, s);
        }
    }

    public static CommonReportAccountingTypeEnum get(Integer key) {
        return LOOKUP.get(key);
    }

}
