package cn.superdesk.uniorder.common.enums.invoice.customer;

import java.util.EnumSet;
import java.util.HashMap;
import java.util.Map;

/**
 * 付款报账状态枚举
 */
public enum ReceivedStatusEnum {
	ZERO(0, "未回款"),
	ONE(1, "部分回款"),
	TWO(2, "全部回款"),
	THREE(3, "无需回款"),

	;

	private final Integer key;
	private final String desc;

	ReceivedStatusEnum(Integer key, String desc) {
		this.key = key;
		this.desc = desc;
	}

	public Integer getKey() {
		return key;
	}

	public String getDesc() {
		return desc;
	}

	public static String getDesc(Integer key) {
		if (key == null) {
			return ZERO.getDesc();
		}
		for (ReceivedStatusEnum c : ReceivedStatusEnum.values()) {
			if (c.key.equals(key)) {
				return c.getDesc();
			}
		}
		return "未知状态";
	}

	private static final Map<Integer, ReceivedStatusEnum> LOOKUP = new HashMap<>();

	static {
		for (ReceivedStatusEnum s : EnumSet.allOf(ReceivedStatusEnum.class)) {
			LOOKUP.put(s.getKey(), s);
		}
	}

	public static ReceivedStatusEnum get(Integer key) {
		return LOOKUP.get(key);
	}
}
