package cn.superdesk.uniorder.common.enums.invoice.supplier;

import java.util.EnumSet;
import java.util.HashMap;
import java.util.Map;

/**
 * 付款报账状态枚举
 */
public enum PaymentStatusEnum {

	ZERO(0, "待付款"),
	ONE(1, "部分付款"),
	TWO(2, "全部付款"),

	;

	private final Integer key;
	private final String desc;

	PaymentStatusEnum(Integer key, String desc) {
		this.key = key;
		this.desc = desc;
	}

	public Integer getKey() {
		return key;
	}

	public String getDesc() {
		return desc;
	}

	public static String getDesc(Integer key) {
		if (key == null) {
			return ZERO.getDesc();
		}
		for (PaymentStatusEnum c : PaymentStatusEnum.values()) {
			if (c.key.equals(key)) {
				return c.getDesc();
			}
		}
		return "未知状态";
	}

	private static final Map<Integer, PaymentStatusEnum> LOOKUP = new HashMap<>();

	static {
		for (PaymentStatusEnum s : EnumSet.allOf(PaymentStatusEnum.class)) {
			LOOKUP.put(s.getKey(), s);
		}
	}

	public static PaymentStatusEnum get(Integer key) {
		return LOOKUP.get(key);
	}
}
