package cn.superdesk.uniorder.common.rest.response;

import com.fasterxml.jackson.annotation.JsonProperty;

import java.io.Serializable;
import java.util.Map;

/**
 * @Description: 待调整
 * @Author: linjinyu
 * @CreateDate: 2021/10/27 下午3:00
 * @UpdateUser: linjinyu
 * @UpdateDate: 2021/10/27 下午3:00
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
public class R<T> implements Serializable {

	private static final long serialVersionUID = 1L;

	// 请求code
	private String  code    = "200";
	//失败
	public static final int SUCCESS = 200;
	//失败
	public static final int FAIL = 100;
	// 请求状态
	private boolean success = true;
	// 错误消息
	private String  message;
	// 返回数据
	private T       data;

	@JsonProperty(access = JsonProperty.Access.READ_ONLY)
	private Map<String, Object> headData;

	/**
	 * currentPage
	 */
	private int currentPage;
	/**
	 * pageSize
	 */
	private int pageSize;
	/**
	 * totalRows
	 */
	private int count;

	public int getCurrentPage() {
		return currentPage;
	}

	public void setCurrentPage(int currentPage) {
		this.currentPage = currentPage;
	}

	public int getPageSize() {
		return pageSize;
	}

	public void setPageSize(int pageSize) {
		this.pageSize = pageSize;
	}

	public int getCount() {
		return count;
	}

	public void setCount(int count) {
		this.count = count;
	}

	public R(int currentPage, int pageSize, int count, T data) {
		super();
		this.data = data;
		this.currentPage = currentPage;
		this.pageSize = pageSize;
		this.count = count;
	}

	public R(int currentPage, int pageSize, int count, T data, Map<String, Object> headData) {
		super();
		this.data = data;
		this.headData = headData;
		this.currentPage = currentPage;
		this.pageSize = pageSize;
		this.count = count;
	}

	public R() {
		super();
	}

	public R(String code, String message, boolean success) {
		super();
		this.code = code;
		this.success = success;
		this.message = message;
	}

	public R(String code, String message, T data) {
		super();
		this.code = code;
		this.message = message;
		this.data = data;
	}

	public R(T data) {
		super();
		this.data = data;
	}

	public R(String code, String message, T data, boolean success) {
		super();
		this.code = code;
		this.message = message;
		this.data = data;
		this.success = success;
	}

	public static <T> R<T> ok() {
		return restResult(null, SUCCESS, "操作成功",Boolean.TRUE);
	}

	public static <T> R<T> ok(T data) {
		return restResult(data, SUCCESS, "操作成功",Boolean.TRUE);
	}

	public static <T> R<T> ok(String msg) {
		return restResult(null, SUCCESS, msg,Boolean.TRUE);
	}

	public static <T> R<T> ok(String msg, T data) {
		return restResult(data, SUCCESS, msg,Boolean.TRUE);
	}

	public static <T> R<T> fail() {
		return restResult(null, FAIL, "操作失败",Boolean.FALSE);
	}

	public static <T> R<T> fail(String msg) {
		return restResult(null, FAIL, msg,Boolean.FALSE);
	}

	public static <T> R<T> fail(T data) {
		return restResult(data, FAIL, "操作失败",Boolean.FALSE);
	}

	public static <T> R<T> fail(String msg, T data) {
		return restResult(data, FAIL, msg,Boolean.FALSE);
	}

	public static <T> R<T> fail(int code, String msg) {
		return restResult(null, code, msg,Boolean.FALSE);
	}

	private static <T> R<T> restResult(T data, int code, String msg,boolean success) {
		R<T> r = new R<>();
		r.setCode(code);
		r.setData(data);
		r.setMessage(msg);
		r.setSuccess(success);
		return r;
	}

	public String getCode() {
		return code;
	}

	public void setCode(String code) {
		this.code = code;
	}

	public void setCode(Integer code) {
		this.code = String.valueOf(code);
	}

	public boolean isSuccess() {
		return success;
	}

	public void setSuccess(boolean success) {
		this.success = success;
	}

	public String getMessage() {
		return message;
	}

	public void setMessage(String message) {
		this.message = message;
	}

	public T getData() {
		return data;
	}

	public void setData(T data) {
		this.data = data;
	}

	public Map<String, Object> getHeadData() {
		return headData;
	}

	public void setHeadData(Map<String, Object> headData) {
		this.headData = headData;
	}

}
