package cn.superdesk.uniorder.common.utils;

import org.apache.commons.lang3.time.DateFormatUtils;

import java.lang.management.ManagementFactory;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * 时间工具类
 * 
 * @author wufujing
 */
public class DateUtils extends org.apache.commons.lang3.time.DateUtils
{
    public static String YYYY = "yyyy";

    public static String YYYY_MM = "yyyy-MM";
    
    public static String YYYYMM = "yyyyMM";

    public static String YYYY_MM_DD = "yyyy-MM-dd";
    public static String YYYYMMDD = "yyyyMMdd";

    public static String YYYYMMDDHHMMSS = "yyyyMMddHHmmss";

    public static String YYYY_MM_DD_HH_MM_SS = "yyyy-MM-dd HH:mm:ss";
    
    private static String[] parsePatterns = {
            "yyyy-MM-dd", "yyyy-MM-dd HH:mm:ss", "yyyy-MM-dd HH:mm", "yyyy-MM", 
            "yyyy/MM/dd", "yyyy/MM/dd HH:mm:ss", "yyyy/MM/dd HH:mm", "yyyy/MM",
            "yyyyMMdd", "yyyyMMddHHmmss", "yyyyMMddHHmm", "yyyyMM",
            "yyyy.MM.dd", "yyyy.MM.dd HH:mm:ss", "yyyy.MM.dd HH:mm", "yyyy.MM"};

    /**
     * 获取当前Date型日期
     * 
     * @return Date() 当前日期
     */
    public static Date getNowDate()
    {
        return new Date();
    }

    /**
     * 获取当前日期, 默认格式为yyyy-MM-dd
     * 
     * @return String
     */
    public static String getDate()
    {
        return dateTimeNow(YYYY_MM_DD);
    }

    public static final String getTime()
    {
        return dateTimeNow(YYYY_MM_DD_HH_MM_SS);
    }

    public static final String dateTimeNow()
    {
        return dateTimeNow(YYYYMMDDHHMMSS);
    }

    public static final String dateTimeNow(final String format)
    {
        return parseDateToStr(format, new Date());
    }

    public static final String dateTime(final Date date)
    {
        return parseDateToStr(YYYY_MM_DD, date);
    }
    public static final String monthTime(final Date date)
    {
    	return parseDateToStr(YYYY_MM, date);
    }
    public static final String dateTime(final Date date, String pattern)
    {
        return parseDateToStr(pattern, date);
    }

    public static final String parseDateToStr(final String format, final Date date)
    {
        return new SimpleDateFormat(format).format(date);
    }

    public static final Date dateTime(final String format, final String ts)
    {
        try
        {
            return new SimpleDateFormat(format).parse(ts);
        }
        catch (ParseException e)
        {
            throw new RuntimeException(e);
        }
    }

    /**
     * 日期路径 即年/月/日 如2018/08/08
     */
    public static final String datePath()
    {
        Date now = new Date();
        return DateFormatUtils.format(now, "yyyy/MM/dd");
    }

    /**
     * 日期路径 即年/月/日 如20180808
     */
    public static final String dateTime()
    {
        Date now = new Date();
        return DateFormatUtils.format(now, "yyyyMMdd");
    }
    
    /**
     * 日期路径 即年/月/日 如20180808
     */
    public static final String getYear(Date now)
    {
        return DateFormatUtils.format(now, "yyyy");
    }
    
    /**
     * 日期路径 即年/月/日 如20180808
     */
    public static final String getMonth(Date now)
    {
        return DateFormatUtils.format(now, "MM");
    }
    
    /**
     * 日期路径 即年/月/日 如20180808
     */
    public static final String getDay(Date now)
    {
        return DateFormatUtils.format(now, "dd");
    }

    /**
     * 日期型字符串转化为日期 格式
     */
    public static Date parseDate(Object str)
    {
        if (str == null)
        {
            return null;
        }
        try
        {
            return parseDate(str.toString(), parsePatterns);
        }
        catch (ParseException e)
        {
            return null;
        }
    }
    
    /**
     * 获取服务器启动时间
     */
    public static Date getServerStartDate()
    {
        long time = ManagementFactory.getRuntimeMXBean().getStartTime();
        return new Date(time);
    }

    /**
     * 计算两个时间差
     */
    public static String getDatePoor(Date endDate, Date nowDate)
    {
        long nd = 1000 * 24 * 60 * 60;
        long nh = 1000 * 60 * 60;
        long nm = 1000 * 60;
        // long ns = 1000;
        // 获得两个时间的毫秒时间差异
        long diff = endDate.getTime() - nowDate.getTime();
        // 计算差多少天
        long day = diff / nd;
        // 计算差多少小时
        long hour = diff % nd / nh;
        // 计算差多少分钟
        long min = diff % nd % nh / nm;
        // 计算差多少秒//输出结果
        // long sec = diff % nd % nh % nm / ns;
        return day + "天" + hour + "小时" + min + "分钟";
    }
    
    /**
     * 根据日期获取星期
     * 
     * @param strdate
     * @return
     */
    public static String getWeekDayByDate(String strdate) {
        final String dayNames[] = { "星期日", "星期一", "星期二", "星期三", "星期四", "星期五", "星期六" };
        SimpleDateFormat sdfInput = new SimpleDateFormat("yyyy-MM-dd");
        Calendar calendar = Calendar.getInstance();
        Date date = new Date();
        try {
            date = sdfInput.parse(strdate);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        calendar.setTime(date);
        int dayOfWeek = calendar.get(Calendar.DAY_OF_WEEK) - 1;
        if (dayOfWeek < 0)
            dayOfWeek = 0;
        return dayNames[dayOfWeek];
    }
    
    /**
     * 获取当前时间前几个月的时间
     * 
     * @param n
     *            前几个月
     * @return
     */
    public static Date getPreMonth(int n) {
        n = -n;
        Calendar c = Calendar.getInstance();
        c.add(Calendar.MONTH, n);
        return c.getTime();

    }
    
    /**
     * 获取多少天以后时间
     * @param n
     * @return
     */
    public static Date getAfterDay(int n) {
        Calendar c = Calendar.getInstance();
        c.add(Calendar.DATE, n);
        return c.getTime();
    }
    
    /**
     * 获取多少天以后时间
     * @param n
     * @param date
     * @return
     */
    public static Date getAfterDay(int n,Date date) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        c.add(Calendar.DATE, n);
        return c.getTime();
    }

    /**
     * 获取前n天时间
     * @param n
     * @return
     */
    public static Date getPreDay(int n) {
        n = -n;
        Calendar c = Calendar.getInstance();
        c.add(Calendar.DATE, n);
        return c.getTime();
    }
    
    /**
     * 获取前n天开始时间
     * @param n
     * @return
     */
    public static Date getStartPreDay(int n) {
        n = -n;
        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.DATE, n);
        cal.set(Calendar.HOUR_OF_DAY, 0);
        cal.set(Calendar.MINUTE, 0);
        cal.set(Calendar.SECOND, 0);
        return cal.getTime();
    }
   
    /**
     * 获取前n天结束时间
     * @param n
     * @return
     */
    public static Date getEndPreDay(int n) {
        n = -n;
        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.DATE, n);
        cal.set(Calendar.HOUR_OF_DAY, 23);
        cal.set(Calendar.MINUTE, 59);
        cal.set(Calendar.SECOND, 59);
        return cal.getTime();
    }
    
    /**
     * 获取n天后结束时间
     * @param n
     * @return
     */
    public static Date getEndAfterDay(int n) {
        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.DATE, n);
        cal.set(Calendar.HOUR_OF_DAY, 23);
        cal.set(Calendar.MINUTE, 59);
        cal.set(Calendar.SECOND, 59);
        return cal.getTime();
    }
    
    /**
     * 获取某年某月的第一天
     * @param year
     * @param month 1月传1
     * @return
     */
    public static Date getFirstDayOfMonth(int year,int month)
    {
      Calendar cal = Calendar.getInstance();
      //设置年份
      cal.set(Calendar.YEAR,year);
      //设置月份
      cal.set(Calendar.MONTH, month-1);
      //获取某月最小天数
      int firstDay = cal.getActualMinimum(Calendar.DAY_OF_MONTH);
      //设置日历中月份的最小天数
      cal.set(Calendar.DAY_OF_MONTH, firstDay);
      cal.set(Calendar.HOUR_OF_DAY, 0);
      cal.set(Calendar.MINUTE, 0);
      cal.set(Calendar.SECOND, 0);
      cal.set(Calendar.MILLISECOND, 0);
      return cal.getTime();
    }
    
    /**
     * 获取某年某月的第一天
     * @param year
     * @param month
     * @return
     */
    public static Date getLastDayOfMonth(int year,int month)
    {
      Calendar cal = Calendar.getInstance();
      //设置年份
      cal.set(Calendar.YEAR,year);
      //设置月份
      cal.set(Calendar.MONTH, month-1);
      //获取某月最大天数
      int lastDay = cal.getActualMaximum(Calendar.DAY_OF_MONTH);
      //设置日历中月份的最小天数
      cal.set(Calendar.DAY_OF_MONTH, lastDay);
      cal.set(Calendar.HOUR_OF_DAY, 23);
      cal.set(Calendar.MINUTE, 59);
      cal.set(Calendar.SECOND, 59);
      return cal.getTime();
    }
    
    /** 
	 * 获取某年第一天日期 
	 * @param year 年份 
	 * @return Date 
	 */  
	public static Date getCurrYearFirst(int year){  
	    Calendar calendar = Calendar.getInstance();  
	    calendar.clear();  
	    calendar.set(Calendar.YEAR, year); 
	    calendar.getTime();  
	    return calendar.getTime();  
	}  
  
	/** 
	 * 获取某年最后一天日期 
	 * @param year 年份 
	 * @return Date 
	 */  
	public static Date getCurrYearLast(int year){  
	    Calendar calendar = Calendar.getInstance();  
	    calendar.clear();  
	    calendar.set(Calendar.YEAR, year);  
	    calendar.roll(Calendar.DAY_OF_YEAR, -1);  
	    return  calendar.getTime();  
	}

    /**
     * 获取当前年份/月份/天等
     * @return Date
     */
    public static int getCurrDateType(int calendarType){
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        return  calendar.get(calendarType);
    }
	
	/**
	 * 查询时间段的所有日期
	 * @param dBegin
	 * @param dEnd
	 * @return
	 */
    public static List<String> findDates(Date dBegin, Date dEnd) {  
        List<String> lDate = new ArrayList<String>();  
        lDate.add(dateTime(dBegin));  
        Calendar calBegin = Calendar.getInstance();  
        // 使用给定的 Date 设置此 Calendar 的时间    
        calBegin.setTime(dBegin);  
        Calendar calEnd = Calendar.getInstance();  
        // 使用给定的 Date 设置此 Calendar 的时间    
        calEnd.setTime(dEnd);  
        // 测试此日期是否在指定日期之后    
        while (dEnd.after(calBegin.getTime())) {  
            // 根据日历的规则，为给定的日历字段添加或减去指定的时间量    
            calBegin.add(Calendar.DAY_OF_MONTH, 1);  
            lDate.add(dateTime(calBegin.getTime()));  
        }  
        return lDate;  
    } 
   
	/**
	 * 查询时间段的所有月份
	 * @param dBegin
	 * @param dEnd
	 * @return
	 */
    public static List<String> findMonths(Date dBegin, Date dEnd) {  
        List<String> lDate = new ArrayList<String>();  
        lDate.add(monthTime(dBegin));  
        Calendar calBegin = Calendar.getInstance();  
        // 使用给定的 Date 设置此 Calendar 的时间    
        calBegin.setTime(dBegin);  
        Calendar calEnd = Calendar.getInstance();  
        // 使用给定的 Date 设置此 Calendar 的时间    
        calEnd.setTime(dEnd);  
        // 测试此日期是否在指定日期之后    
        while (calBegin.before(calEnd)) {  
            // 根据日历的规则，为给定的日历字段添加或减去指定的时间量    
            calBegin.add(Calendar.MONTH, 1);  
            lDate.add(monthTime(calBegin.getTime()));  
        }  
        return lDate;  
    } 
    
    
    /**
     * 根据所给的起止时间来计算间隔的天数
     * 
     * @param startDate
     *            起始时间
     * @param endDate
     *            结束时间
     * @return int 返回间隔天数
     */
    public static int getIntervalDays(Date startDate, Date endDate) {
        long startdate = startDate.getTime();
        long enddate = endDate.getTime();
        long interval = enddate - startdate;
        int intervalday = (int) (interval / (1000 * 60 * 60 * 24));
        return intervalday;
    }
    
    /**
     * 将秒数转换时分
     * @param second
     * @return
     */
    public static String formatSecond(long second){  
        String  timeFormat="0秒";  
        if(second > 0){  
            String format ="";  
            Object[] array = null;  
            Integer hours =(int) (second/(60*60));  
            Integer minutes = (int) (second/60-hours*60);  
            Integer seconds = (int) (second-minutes*60-hours*60*60);  
            if(hours>0){  
                format="%1$,d时%2$,d分";  
                array=new Object[]{hours,minutes};  
            }else if(minutes>0){  
                format="%1$,d分";  
                array=new Object[]{minutes};  
            }else{  
                format="%1$,d秒";  
                array=new Object[]{seconds};  
            }  
            timeFormat= String.format(format, array);  
        }  
       return timeFormat;
    }
}
