package cn.superdesk.uniorder.dto.param.sharecenter;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.AccessLevel;
import lombok.Data;
import lombok.Setter;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.math.BigDecimal;
import java.math.RoundingMode;

/**
 * 行类型  cost 数据 核销/费用明细（微信手续费，按6‰系统自动计算金额）
 */
@Data
public class ActualReceivedReportAccountingLineForCost implements Serializable {

    public ActualReceivedReportAccountingLineForCost() {
        this.lineType = "cost";
        this.sourceCurrency = "CNY";
        this.rateValue = BigDecimal.ONE.setScale(2, RoundingMode.UP);
    }

    /**
     * 行类型  cost
     */
    @ApiModelProperty("行类型  cost")
    @NotBlank
    @Setter(AccessLevel.NONE)
    private String lineType;
    /**
     * 核销/费用 业务小类
     */
    @ApiModelProperty("核销/费用 业务小类")
    @NotBlank
    private String operationSubTypeCode;
    /**
     * 责任部门编码
     */
    @ApiModelProperty("责任部门编码")
    @NotBlank
    private String expenseDeptId;
    /**
     * 责任部门名称
     */
    @ApiModelProperty("责任部门名称")
    @NotBlank
    private String expenseDeptName;
    /**
     * 成本中心编码
     */
    @ApiModelProperty("成本中心编码")
    @NotBlank
    private String costcenterId;
    /**
     * 成本中心名称
     */
    @ApiModelProperty("成本中心名称")
    @NotBlank
    private String costcenterName;
    /**
     * 币种
     */
    @ApiModelProperty("币种")
    @NotBlank
    @Setter(AccessLevel.NONE)
    private String sourceCurrency;
    /**
     * 汇率
     */
    @ApiModelProperty("汇率")
    @NotNull
    @Setter(AccessLevel.NONE)
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    private BigDecimal rateValue;
    /**
     * 本位币金额
     */
    @ApiModelProperty("本位币金额")
    @NotNull
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    private BigDecimal currencyAmount = BigDecimal.ZERO;
    /**
     * 原币金额
     */
    @ApiModelProperty("原币金额")
    @NotNull
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    private BigDecimal amount = BigDecimal.ZERO;
    /**
     * 客户编码
     */
    @ApiModelProperty("客户编码")
    @NotBlank
    private String customerNum;
    /**
     * 客户名称
     */
    @ApiModelProperty("客户名称")
    private String customerName = "";
    /**
     * 合同编号
     */
    @ApiModelProperty("合同编号")
    private String contractNum = "";
    /**
     * 合同名称
     */
    @ApiModelProperty("合同名称")
    private String contractName = "";
    /**
     * 项目编码
     */
    @ApiModelProperty("项目编码")
    private String projectNum = "";
    /**
     * 项目名称
     */
    @ApiModelProperty("项目名称")
    private String projectName = "";

    /** 供应商编码 */
    @ApiModelProperty("供应商编码")
    private String supplierNum;
}
