package cn.superdesk.uniorder.dto.param.sharecenter;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.AccessLevel;
import lombok.Data;
import lombok.Setter;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.math.BigDecimal;
import java.math.RoundingMode;

/**
 * 行类型  income 数据 收入明细
 */
@Data
public class ActualReceivedReportAccountingLineForIncome implements Serializable {

    public ActualReceivedReportAccountingLineForIncome() {
        this.lineType = "income";
        this.sourceCurrency = "CNY";
        this.rateValue = BigDecimal.ONE.setScale(2, RoundingMode.UP);
    }

    /**
     * 行类型  income
     */
    @ApiModelProperty("行类型  income")
    @NotBlank
    @Setter(AccessLevel.NONE)
    private String lineType;
    /**
     * 收入明细业务小类
     */
    @ApiModelProperty("收入明细业务小类")
    @NotBlank
    private String operationSubTypeCode;
    /**
     * 责任部门编码
     */
    @ApiModelProperty("责任部门编码")
    @NotBlank
    private String expenseDeptId;
    /**
     * 责任部门名称
     */
    @ApiModelProperty("责任部门名称")
    @NotBlank
    private String expenseDeptName;
    /**
     * 成本中心编码
     */
    @ApiModelProperty("成本中心编码")
    @NotBlank
    private String costcenterId;
    /**
     * 成本中心名称
     */
    @ApiModelProperty("成本中心名称")
    @NotBlank
    private String costcenterName;
    /**
     * 币种
     */
    @ApiModelProperty("币种")
    @NotBlank
    @Setter(AccessLevel.NONE)
    private String sourceCurrency;
    /**
     * 汇率
     */
    @ApiModelProperty("汇率")
    @NotNull
    @Setter(AccessLevel.NONE)
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    private BigDecimal rateValue;
    /**
     * 本位币金额
     */
    @ApiModelProperty("本位币金额")
    @NotNull
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    private BigDecimal currencyAmount = BigDecimal.ZERO;
    /**
     * 原币金额
     */
    @ApiModelProperty("原币金额")
    @NotNull
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    private BigDecimal amount = BigDecimal.ZERO;
    /**
     * 税额
     */
    @ApiModelProperty("税额")
    @NotNull
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    private BigDecimal taxAmount = BigDecimal.ZERO;
    /**
     * 税率
     */
    @ApiModelProperty("税率")
    @NotNull
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    private BigDecimal taxRate = BigDecimal.ZERO;
    /**
     * 税种
     */
    @ApiModelProperty("税种")
    @NotBlank
    private String taxType = "";
    /**
     * 客户编码
     */
    @ApiModelProperty("客户编码")
    @NotBlank
    private String customerNum = "";
    /**
     * 客户名称
     */
    @ApiModelProperty("客户名称")
    private String customerName = "";
    /**
     * 合同编号
     */
    @ApiModelProperty("合同编号")
    private String contractNum = "";
    /**
     * 合同名称
     */
    @ApiModelProperty("合同名称")
    private String contractName = "";
    /**
     * 项目编码
     */
    @ApiModelProperty("项目编码")
    private String projectNum = "";
    /**
     * 项目名称
     */
    @ApiModelProperty("项目名称")
    private String projectName = "";
    /**
     * 业态类别编码
     */
    @ApiModelProperty("业态类别编码")
    @NotBlank
    private String formatType = "";
    /**
     * 服务及商品编码
     */
    @ApiModelProperty("服务及商品编码")
    private String serviceType = "";
    /**
     * 营运单元编码 空
     */
    @ApiModelProperty("营运单元编码 空")
    private String yydy = "";

    /** 供应商编码 */
    @ApiModelProperty("供应商编码")
    private String supplierNum = "";
}
