package cn.superdesk.uniorder.dto.vo.accounting.score.customer;

import cn.superdesk.uniorder.dto.param.sharecenter.ActualReceivedReportAccountingHeader;
import cn.superdesk.uniorder.dto.vo.sys.SysBusinessUserVo;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * @Description: 积分结算2.0-对账单-推送报账单详情
 */
@NoArgsConstructor
@AllArgsConstructor
@Builder(toBuilder = true)
@Data
public class PushRmAccountingVo implements Serializable {

	@ApiModelProperty("报账单详情")
	private PushRmAccountingDetailVo  accounting;
	@ApiModelProperty("应收报账单明细集合")
	private List<PushReceivableVo> receivableVoList;
	@ApiModelProperty("应付报账单明细集合")
	private List<PushReceivableVo> payableVoList;
	@ApiModelProperty("业务员信息")
	private SysBusinessUserVo businessUserVo;
	@Data
	public static class PushRmAccountingDetailVo extends AccountingReceivablePayableVo{
		@ApiModelProperty(value = "业态类别")
		private String businessCategory;

		@ApiModelProperty(value = "业务类型编码")
		private String operationTypeCode;

		@ApiModelProperty(value = "业务单位编码")
		private String companyNum;

		@ApiModelProperty(value = "业务单位名称")
		private String companyName;

		@ApiModelProperty(value = "总行数")
		private Integer totalCount;
		@ApiModelProperty(value = "总金额")
		private String totalAmount;
		@ApiModelProperty(value = "总税额")
		private String totalTaxAmount;
	}

	@Data
	public static class PushReceivableVo  implements Serializable{
		@ApiModelProperty(value = "明细ID")
		private Long id;
		@ApiModelProperty(value = "类型,默认:收入")
		private String lineType;
		@ApiModelProperty(value = "金额")
		private String amount;
		@ApiModelProperty(value = "税额")
		private String taxAmount;
		@ApiModelProperty(value = "税率")
		private String taxRate;
		@ApiModelProperty(value = "税种")
		private String taxType;
		@ApiModelProperty(value = "客户Id")
		private String customerId;
		@ApiModelProperty(value = "客户编码")
		private String customerNum;
		@ApiModelProperty(value = "客户名称")
		private String customerName;
		@ApiModelProperty(value = "收款方式编码")
		private String billingMethod;
		@ApiModelProperty(value = "币种")
		private String sourceCurrency;
		@ApiModelProperty(value = "汇率")
		private String rateValue;
		@ApiModelProperty(value = "业务小类")
		private String operationSubTypeCode;
		@ApiModelProperty(value = "成本中心编码")
		private String costcenterId;
		@ApiModelProperty(value = "成本中心名称")
		private String costcenterName;
		@ApiModelProperty(value = "项目编码")
		private String projectNum;
		@ApiModelProperty(value = "项目名称")
		private String projectName;
		@ApiModelProperty(value = "业态类别编码")
		private String formatType;
		@ApiModelProperty(value = "服务及商品编码")
		private String serviceType;
		@ApiModelProperty(value = "责任部门编码")
		private String expenseDeptId;
		@ApiModelProperty(value = "责任部门名称")
		private String expenseDeptName;
		@ApiModelProperty(value = "合同编Id")
		private String contractId;
		@ApiModelProperty(value = "合同编号")
		private String contractNum;
		@ApiModelProperty(value = "合同名称")
		private String contractName;
		@ApiModelProperty(value = "成本类别编码")
		private String dimensionCblb;
		@ApiModelProperty(value = "供应商Id")
		private String vendorId;
		@ApiModelProperty(value = "供应商名称")
		private String vendorName;
		@ApiModelProperty(value = "供应商编码")
		private String vendorCode;
	}

	@Data
	public static class PushPayableVo  implements Serializable{
		@ApiModelProperty(value = "明细ID")
		private Long id;
		@ApiModelProperty(value = "金额")
		private String expenseAmount;
		@ApiModelProperty(value = "税额")
		private String taxAmount;
		@ApiModelProperty(value = "税率")
		private String taxRate;
		@ApiModelProperty(value = "税种")
		private String taxCode;
		@ApiModelProperty(value = "供应商Id")
		private String vendorId;
		@ApiModelProperty(value = "供应商名称")
		private String vendorName;
		@ApiModelProperty(value = "供应商编码")
		private String vendorCode;
		@ApiModelProperty(value = "客户Id")
		private String customerId;
		@ApiModelProperty(value = "客户编码")
		private String customerNum;
		@ApiModelProperty(value = "客户名称")
		private String customerName;
		@ApiModelProperty(value = "合同编Id")
		private String contractId;
		@ApiModelProperty(value = "合同编号")
		private String contractNum;
		@ApiModelProperty(value = "合同名称")
		private String contractName;
		@ApiModelProperty(value = "成本中心编码")
		private String ccCode;
		@ApiModelProperty(value = "成本中心名称")
		private String ccName;
		@ApiModelProperty(value = "业态类别编码")
		private String businessType;
		@ApiModelProperty(value = "成本类别编码")
		private String dimensionCblb;
		@ApiModelProperty(value = "服务及商品编码")
		private String dimensionFwjsp;
		@ApiModelProperty(value = "业务小类")
		private String operationSubTypeCode;
		@ApiModelProperty(value = "项目编码")
		private String projectCode;
		@ApiModelProperty(value = "项目名称")
		private String projectName;
		@ApiModelProperty(value = "责任部门编码")
		private String expenseDeptId;
		@ApiModelProperty(value = "责任部门名称")
		private String expenseDeptName;
	}
}
