package cn.superdesk.uniorder.dubbo.facade.accounting.customer;

import cn.superdesk.uniorder.common.exception.BadRequestException;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.common.security.dto.LoginSysUser;
import cn.superdesk.uniorder.dao.entity.accounting.score.customer.AccountingScoreCustomer;
import cn.superdesk.uniorder.dao.entity.biz.AccountingReceivablePayable;
import cn.superdesk.uniorder.dto.param.accounting.score.customer.AccountingOrderScoreCustomerQueryCriteria;
import cn.superdesk.uniorder.dto.param.accounting.score.customer.AccountingOrderScoreCustomerCreateParam;
import cn.superdesk.uniorder.dto.param.accounting.score.customer.AccountingOrderScoreCustomerOpDetailQueryCriteria;
import cn.superdesk.uniorder.dto.param.accounting.unified.customer.AccountingReceivablePayableParam;
import cn.superdesk.uniorder.dto.param.accounting.unified.customer.PushAccountingParam;
import cn.superdesk.uniorder.dto.vo.accounting.score.customer.*;
import cn.superdesk.uniorder.dto.vo.invoice.customer.InvoiceApplyDetailDimCreditItemVo;

import java.util.List;
import java.util.Map;

public interface DubboAccountingScoreCustomerService {


    /****************************************** 积分结算 对账单 start ******************************************/

    List<Long> getAccountingXCreditIdsByAccountingOrderId(Long id);

    R<List<AccountingOrderScoreCustomerListItemVo>> pageQueryAccountingOrderList(AccountingOrderScoreCustomerQueryCriteria queryCriteria, LoginSysUser loginSysUser);

    List<AccountingOrderScoreCustomerListItemVo> queryAccountingOrderList(AccountingOrderScoreCustomerQueryCriteria queryCriteria, LoginSysUser loginSysUser);

    AccountingOrderScoreCustomerListItemVo getAccountingOrderListByAccountingOrderSn(String accountingOrderSn, LoginSysUser loginSysUser);

    String exportExcelAccountingOrderScoreCustomerList(AccountingOrderScoreCustomerQueryCriteria queryCriteria, LoginSysUser loginSysUser);

    AccountingOrderScoreCustomerHeadVo getAccountingOrderListHeadSum(AccountingOrderScoreCustomerQueryCriteria queryCriteria);

    /**
     * 创建结算单(对账单)(积分) ---- 与 addDzMainNew 对映
     * 效率优化 mark by linjinyu at 20211203
     *
     * @param scoreOrderReconciliationNotYetVoList
     * @param scoreAccountingCustomerCreateParam
     * @param loginSysUser
     * @return
     * @throws Exception
     */
    AccountingScoreCustomer createScoreAccountingCustomer(
            List<InvoiceApplyDetailDimCreditItemVo> scoreOrderReconciliationNotYetVoList,
            AccountingOrderScoreCustomerCreateParam scoreAccountingCustomerCreateParam, LoginSysUser loginSysUser) throws Exception;

    /****************************************** 积分结算 对账单   end ******************************************/

    /****************************************** 积分结算 对账单 详情 start ******************************************/

    AccountingOrderScoreCustomerDetailVo commonStatementOfAccountDetailDimCredit(AccountingOrderScoreCustomerOpDetailQueryCriteria queryCriteria, LoginSysUser loginSysUser);

    CreateRmAccountingVo getCreateRmAccountDetail(Long id, LoginSysUser loginSysUser);

    PushRmAccountingVo getPushRmAccountDetail(Long id, LoginSysUser loginSysUser);

    String exportExcelAccountingOrderScoreCustomerDetailDimCredit(AccountingOrderScoreCustomerOpDetailQueryCriteria queryCriteria, LoginSysUser loginSysUser);

    //导出应收应付流水
    String exportAccountingFlow(Long id, LoginSysUser loginSysUser);

    /****************************************** 积分结算 对账单 详情   end ******************************************/

    /****************************************** 积分结算 对账单 操作 start ******************************************/

    /**
     * 积分结算-op重新对账
     *
     * @param accountingOrderId
     * @param loginSysUser
     * @return
     */
    R opCancelAccounting(Long accountingOrderId, LoginSysUser loginSysUser);

    /**
     * 拒绝对账
     *
     * @param accountingOrderId
     * @param loginSysUser
     * @return
     */
    R opRefuseAccounting(Long accountingOrderId, LoginSysUser loginSysUser);

    /**
     * 确认对账
     *
     * @param accountingOrderId
     * @param loginSysUser
     * @return
     */
    R opConfirmAccounting(Long accountingOrderId, LoginSysUser loginSysUser);

    int opDeleteAccounting(Long accountingOrderId, LoginSysUser loginSysUser);

    /**
     * 生成应收&应付数据
     */
    R opStartFlowData(Long accountingOrderId, LoginSysUser loginSysUser);

    /**
     * 创建报账单
     */
    R createRmAccounting(Long accountingOrderId, LoginSysUser loginSysUser);

    /**
     * 推送报账单
     */
    R pushRmAccounting(PushAccountingParam pushAccountingParam, LoginSysUser loginSysUser);

    /**
     * 查询推送报账单
     */
    R queryPushRmAccounting(Long id,String accountingOrderSn, LoginSysUser loginSysUser);

    /**
     * 删除报账单
     */
    R delRmAccounting(Long id, LoginSysUser loginSysUser);

    /**
     * 设置备注-报账单
     */
    R setRemarkRmAccounting(AccountingReceivablePayableParam accountingParam, LoginSysUser loginSysUser);

    /****************************************** 积分结算 对账单 操作 start ******************************************/


    void deleteObj(Long id);

    int opUpdateByPrimaryKeySelective(AccountingScoreCustomer scoreAccountingCustomer, LoginSysUser loginSysUser);

    int opUpdateFieldRemark(AccountingScoreCustomer scoreAccountingCustomer, LoginSysUser loginSysUser);


    AccountingScoreCustomer findById(Long id);

    AccountingScoreCustomer findByAccountingOrderSn(String accountingOrderSn);


    List<AccountingScoreCustomer> findByIds(List<Long> accountingOrderIds);

    void addObj(AccountingScoreCustomer scoreAccountingCustomer);

    List<AccountingScoreCustomer> findAll(Map<String, Object> queryMap);
}
