package cn.superdesk.uniorder.dubbo.provider.biz;

import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.dao.entity.biz.BizContract;
import cn.superdesk.uniorder.dao.entity.organisation.UnifiedSupplier;
import cn.superdesk.uniorder.dao.entity.organisation.UnifiedSupplierExample;
import cn.superdesk.uniorder.dao.mapper.biz.BizContractMapper;
import cn.superdesk.uniorder.dao.mapper.organisation.UnifiedSupplierMapper;
import cn.superdesk.uniorder.dto.param.biz.BizContractParam;
import cn.superdesk.uniorder.dubbo.facade.biz.IBizContractService;
import com.alibaba.dubbo.config.annotation.Service;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.Date;
import java.util.List;

@Service(version = "1.0.0", retries = -1, timeout = 10000)
public class BizContractServiceImpl implements IBizContractService {

    @Autowired
    private BizContractMapper bizContractMapper;
    @Autowired
    private UnifiedSupplierMapper unifiedSupplierMapper;

    /**
     * 查询合同信息
     *
     * @param id 合同信息主键
     * @return 合同信息
     */
    @Override
    public BizContract selectBizContractById(Long id)
    {
        return bizContractMapper.selectBizContractById(id);
    }

    /**
     * 合同列表
     * @param contractParam 分页参数
     * @return List
     */
    @Override
    public R<List<BizContract>> getListPage(BizContractParam contractParam){
        BizContract bizContract = new BizContract();
        BeanUtils.copyProperties(contractParam,bizContract);

        Page<BizContract> page = PageHelper.startPage(contractParam.getPageNumber(), contractParam.getPageSize());
        bizContract.setDelFlag("0");
        List<BizContract> list = bizContractMapper.selectBizContractList(bizContract);

        PageHelper.clearPage();
        return new R<>(page.getPageNum(), page.getPageSize(), (int) page.getTotal(), list);
    }

    /**
     * 查询合同信息列表
     *
     * @param bizContract 合同信息
     * @return 合同信息
     */
    @Override
    public List<BizContract> selectBizContractList(BizContract bizContract)
    {
        Long supplierId = bizContract.getSupplierId();
        String supplierCode = bizContract.getSupplierCode();

        if((supplierId == null || supplierId < 1L ) && StringUtils.isNotBlank(supplierCode)){
            UnifiedSupplierExample example = new UnifiedSupplierExample();
            example.createCriteria().andSaleIdEqualTo(Long.valueOf(supplierCode));
            List<UnifiedSupplier> supplierList = unifiedSupplierMapper.selectByExample(example);
            if(CollectionUtils.isNotEmpty(supplierList)){
                supplierId = supplierList.get(0).getId();
                bizContract.setSupplierId(supplierId);
            }
        }
        if(supplierId == null || supplierId < 1L ){
            return null;
        }

        return bizContractMapper.selectBizContractList(bizContract);
    }

    /**
     * 新增合同信息
     *
     * @param bizContract 合同信息
     * @return 结果
     */
    @Override
    public int insertBizContract(BizContract bizContract)
    {
        bizContract.setCreateTime(new Date());
        return bizContractMapper.insertBizContract(bizContract);
    }

    /**
     * 修改合同信息
     *
     * @param bizContract 合同信息
     * @return 结果
     */
    @Override
    public int updateBizContract(BizContract bizContract)
    {
        bizContract.setUpdateTime(new Date());
        return bizContractMapper.updateBizContract(bizContract);
    }

    /**
     * 修改合同信息默认为N
     *
     * @return 结果
     */
    @Override
    public int updateAllDefaultNo(){
        return bizContractMapper.updateAllDefaultNo();
    }

    /**
     * 批量删除合同信息
     *
     * @param ids 需要删除的合同信息主键
     * @return 结果
     */
    @Override
    public int deleteBizContractByIds(Long[] ids)
    {
        return bizContractMapper.deleteBizContractByIds(ids);
    }

    /**
     * 删除合同信息信息
     *
     * @param id 合同信息主键
     * @return 结果
     */
    @Override
    public int deleteBizContractById(Long id)
    {
        return bizContractMapper.deleteBizContractById(id);
    }
}

