package cn.superdesk.uniorder.dubbo.provider.invoice.customer;

import cn.superdesk.uniorder.common.exception.UnifiedOrderRuntimeException;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.common.rest.response.ResponseResultOuterUtils;
import cn.superdesk.uniorder.common.security.dto.LoginSysUser;
import cn.superdesk.uniorder.dto.param.invoice.customer.reportaccounting.*;
import cn.superdesk.uniorder.dto.vo.invoice.reportaccounting.ActualReceivedReportAccountingDetailVO;
import cn.superdesk.uniorder.dto.vo.invoice.reportaccounting.ActualReceivedReportAccountingVO;
import cn.superdesk.uniorder.dto.vo.invoice.reportaccounting.ActualReportAccountingPreCheckVO;
import cn.superdesk.uniorder.dubbo.facade.invoice.customer.DubboReportAccountingService;
import cn.superdesk.uniorder.service.invoice.customer.ReportAccountingService;
import com.alibaba.dubbo.config.annotation.Service;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.Resource;


/**
 * @Description: Java类作用描述
 * @Author: zhufeng
 * @CreateDate: 2022/11/28下午6:34
 * @Version: 1.0
 */
@Slf4j
@Service(version = "1.0.0", retries = -1, timeout = 10000)
public class DubboReportAccountingServiceImpl implements DubboReportAccountingService {

    @Resource
    private ReportAccountingService reportAccountingService;

    @Override
    public void repairInvoiceStatus(InvoiceStatusRepair repair, LoginSysUser loginSysUser) {
        reportAccountingService.repairInvoiceStatus(repair, loginSysUser);
    }

    @Override
    public ActualReceivedReportAccountingVO getActualReceivedAccountingVO(ReportAccountingQueryParam param, LoginSysUser loginSysUser) {
        return reportAccountingService.getActualReceivedAccountingVO(param, loginSysUser);
    }

    @Override
    public ActualReceivedReportAccountingDetailVO getActualReceivedReportAccountingDetail(ActualReportAccountingQueryParam param, LoginSysUser loginSysUser) {
        return reportAccountingService.getActualReceivedReportAccountingDetail(param, loginSysUser);
    }

    @Override
    public R<ActualReportAccountingPreCheckVO> preCheckByCreateActualReceivedAccounting(ActualReportAccountingPreCheckParam param, LoginSysUser loginSysUser) {
        return reportAccountingService.preCheckByCreateActualReceivedAccounting(param, loginSysUser);
    }

    @Override
    public R<?> createActualReceivedAccounting(ActualReportAccountingCreateParam param, LoginSysUser loginSysUser) {
        try {
            return reportAccountingService.createActualReceivedAccounting(param, loginSysUser);
        } catch (UnifiedOrderRuntimeException ue) {
            return ResponseResultOuterUtils.createErrorResult(String.valueOf(ue.getCode()), ue.getMessage());
        }
    }

    @Override
    public void modifyActualReceivedAccounting(ActualReportAccountingModifyParam param, LoginSysUser loginSysUser) {
        reportAccountingService.modifyActualReceivedAccounting(param, loginSysUser);
    }

    @Override
    public R<?> confirmActualReceivedAccounting(ActualReportAccountingConfirmParam param, LoginSysUser loginSysUser) {
        return reportAccountingService.confirmActualReceivedAccounting(param, loginSysUser);
    }
}
