package cn.superdesk.uniorder.executor.scheduler.xxljob.accounting.customer;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import cn.superdesk.uniorder.common.enums.common.CommonNeedReviseMarkEnum;
import cn.superdesk.uniorder.common.exception.BadRequestException;
import cn.superdesk.uniorder.common.security.dto.LoginSysUser;
import cn.superdesk.uniorder.dao.entity.biz.AccountingReceivablePayable;
import cn.superdesk.uniorder.dao.mapper.biz.AccountingReceivablePayableMapper;
import cn.superdesk.uniorder.dto.param.accounting.score.customer.AccountingOrderScoreCustomerQueryCriteria;
import cn.superdesk.uniorder.dto.param.invoice.customer.reportaccounting.ActualReportAccountingConfirmParam;
import cn.superdesk.uniorder.dto.vo.accounting.score.customer.AccountingOrderScoreCustomerListItemVo;
import cn.superdesk.uniorder.service.accounting.score.customer.AccountingScoreCustomerService;
import cn.superdesk.uniorder.service.invoice.customer.ReportAccountingService;
import cn.superdesk.uniorder.service.invoice.supplier.AccountingReportPaymenService;
import com.google.common.collect.Lists;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @Description: Java类作用描述
 * @Author: linjinyu
 * @CreateDate: 2022/5/19 7:11 下午
 * @UpdateUser: linjinyu
 * @UpdateDate: 2022/5/19 7:11 下午
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class AccountingOrderScoreCustomerTask {

    private final AccountingScoreCustomerService accountingScoreCustomerService;
    @Autowired
    private AccountingReportPaymenService accountingReportPaymenService;
    @Autowired
    private ReportAccountingService reportAccountingService;
    @Autowired
    private AccountingReceivablePayableMapper accountingReceivablePayableMapper;
    /**
     * 补丁类
     */
    @XxlJob("AccountingOrderScoreCustomerPatchNeedReviseMarkStatistics")
    public void patchNeedReviseMarkStatistics() {

        String jobParam = XxlJobHelper.getJobParam();
        log.info("XxlJob patchNeedReviseMarkStatistics jobParam : {}", jobParam);

        List<AccountingOrderScoreCustomerListItemVo> resultList = null;
        LoginSysUser loginSysUser = LoginSysUser
                .builder()
                .sysUserId(996L)
                .fullName("任务执行者")
                .sysUserName("xxljob")
                .build();

        /**************************************** 任务开始 start ****************************************/

        resultList = accountingScoreCustomerService.getAccountingOrderList(
                AccountingOrderScoreCustomerQueryCriteria
                        .builder()
                        .needReviseMark(CommonNeedReviseMarkEnum.修正.getKey())

                        .pageNumber(1)
                        .pageSize(100)

                        .build(),
                loginSysUser
        );

        Integer taskSize = CollUtil.isEmpty(resultList) ? 0 : resultList.size();

        XxlJobHelper.log(StrUtil.format("任务 patchNeedReviseMarkStatistics size {}", taskSize));

        accountingScoreCustomerService.patchNeedReviseMarkStatistics(resultList, loginSysUser);


        /**************************************** 任务开始   end ****************************************/
    }

    /**
     * 补丁类
     */
    @XxlJob("AccountingOrderScoreCustomerPatchNeedReviseMarkInvoiceStatus")
    public void patchNeedReviseMarkInvoiceStatus() {

        String jobParam = XxlJobHelper.getJobParam();
        log.info("XxlJob patchNeedReviseMarkInvoiceStatus jobParam : {}", jobParam);

        List<AccountingOrderScoreCustomerListItemVo> resultList = null;
        LoginSysUser loginSysUser = LoginSysUser
                .builder()
                .sysUserId(996L)
                .fullName("任务执行者")
                .sysUserName("xxljob")
                .build();

        /**************************************** 任务开始 start ****************************************/

        resultList = accountingScoreCustomerService.getAccountingOrderList(
                AccountingOrderScoreCustomerQueryCriteria
                        .builder()
                        .needReviseMark(CommonNeedReviseMarkEnum.修数据_20220518.getKey())

                        .pageNumber(1)
                        .pageSize(100)

                        .build(),
                loginSysUser
        );

        Integer taskSize = CollUtil.isEmpty(resultList) ? 0 : resultList.size();

        XxlJobHelper.log(StrUtil.format("任务 patchNeedReviseMarkInvoiceStatus size {}", taskSize));

        accountingScoreCustomerService.patchNeedReviseMarkInvoiceStatus(resultList, loginSysUser);


        /**************************************** 任务开始   end ****************************************/
    }

    /**
     * 补丁类
     */
    @XxlJob("AccountingOrderScoreCustomerPatchNeedReviseMarkMailTickStatus")
    public void patchNeedReviseMarkMailTickStatus() {

        String jobParam = XxlJobHelper.getJobParam();
        log.info("XxlJob patchNeedReviseMarkMailTickStatus jobParam : {}", jobParam);

        List<AccountingOrderScoreCustomerListItemVo> resultList = null;
        LoginSysUser loginSysUser = LoginSysUser
                .builder()
                .sysUserId(996L)
                .fullName("任务执行者")
                .sysUserName("xxljob")
                .build();

        /**************************************** 任务开始 start ****************************************/

        resultList = accountingScoreCustomerService.getAccountingOrderList(
                AccountingOrderScoreCustomerQueryCriteria
                        .builder()
                        .needReviseMark(CommonNeedReviseMarkEnum.修数据_20220505.getKey())

                        .pageNumber(1)
                        .pageSize(100)

                        .build(),
                loginSysUser
        );

        Integer taskSize = CollUtil.isEmpty(resultList) ? 0 : resultList.size();

        XxlJobHelper.log(StrUtil.format("任务 patchNeedReviseMarkMailTickStatus size {}", taskSize));

        accountingScoreCustomerService.patchNeedReviseMarkMailTickStatus(resultList, loginSysUser);


        /**************************************** 任务开始   end ****************************************/
    }

    /**
     * 补丁类
     */
    @XxlJob("AccountingOrderScoreCustomerPatchNeedReviseMarkServiceNoSourceFrom")
    public void patchNeedReviseMarkServiceNoSourceFrom() {

        String jobParam = XxlJobHelper.getJobParam();
        log.info("XxlJob patchNeedReviseMarkServiceNoSourceFrom jobParam : {}", jobParam);

        List<AccountingOrderScoreCustomerListItemVo> resultList = null;
        LoginSysUser loginSysUser = LoginSysUser
                .builder()
                .sysUserId(996L)
                .fullName("任务执行者")
                .sysUserName("xxljob")
                .build();

        /**************************************** 任务开始 start ****************************************/

        resultList = accountingScoreCustomerService.getAccountingOrderList(
                AccountingOrderScoreCustomerQueryCriteria
                        .builder()
                        .needReviseMark(CommonNeedReviseMarkEnum.修数据_20220519.getKey())

                        .pageNumber(1)
                        .pageSize(100)

                        .build(),
                loginSysUser
        );

        Integer taskSize = CollUtil.isEmpty(resultList) ? 0 : resultList.size();

        XxlJobHelper.log(StrUtil.format("任务 patchNeedReviseMarkServiceNoSourceFrom size {}", taskSize));

        accountingScoreCustomerService.patchNeedReviseMarkServiceNoSourceFrom(resultList, loginSysUser);


        /**************************************** 任务开始   end ****************************************/
    }

    /**
     * 查询推送报账单定时任务
     */
    @XxlJob("AccountingScoreCustomerQueryPushRmAccountingJob")
    public void AccountingScoreCustomerQueryPushRmAccountingJob() {

        String jobParam = XxlJobHelper.getJobParam();
        log.info("XxlJob QueryPushRmAccountingJob jobParam : {}", jobParam);

        LoginSysUser loginSysUser = LoginSysUser
                .builder()
                .sysUserId(996L)
                .fullName("任务执行者")
                .sysUserName("xxljob")
                .build();

        //获取报账单集合
        AccountingReceivablePayable accountingReceivablePayableParam = new AccountingReceivablePayable();
        accountingReceivablePayableParam.setAccountingState(2);
        List<AccountingReceivablePayable> allAccountingList = accountingReceivablePayableMapper.selectAccountingReceivablePayableList(accountingReceivablePayableParam);
        if (CollectionUtils.isEmpty(allAccountingList)) {
            log.info("XxlJob QueryPushRmAccountingJob accountingList is null");
            return;
        }

        Integer taskSize = CollUtil.isEmpty(allAccountingList) ? 0 : allAccountingList.size();

        XxlJobHelper.log(StrUtil.format("任务 QueryPushRmAccountingJob size {}", taskSize));

        //应收应付报账单查询
        List<String> accountingList = allAccountingList.stream()
                .filter(data -> Lists.newArrayList(0,1,2).contains(data.getBillType()))
                .map(data ->data.getAccountingOrderSn()).distinct().collect(Collectors.toList());
        if(CollectionUtils.isNotEmpty(accountingList)){
            for (String accountingOrderSn : accountingList){
                try {
                    accountingScoreCustomerService.queryPushRmAccounting(null,accountingOrderSn,loginSysUser);
                } catch (BadRequestException e) {
                    log.error("queryPushRmAccountingJob error accountingOrderSn={}",accountingOrderSn,e);
                }
            }
        }

        //实收报账单查询
        List<String> orderSnList = allAccountingList.stream()
                .filter(data -> Lists.newArrayList(3,4).contains(data.getBillType()))
                .map(data ->data.getAccountingOrderSn()).distinct().collect(Collectors.toList());
        if(CollectionUtils.isNotEmpty(orderSnList)){
            for (String accountingOrderSn : orderSnList){
                try {
                    ActualReportAccountingConfirmParam param = new ActualReportAccountingConfirmParam();
                    param.setAccountingOrderSn(accountingOrderSn);
                    reportAccountingService.confirmActualReceivedAccounting(param, loginSysUser);
                } catch (BadRequestException e) {
                    log.error("queryPushRmAccountingJob received error accountingOrderSn={}",accountingOrderSn,e);
                }
            }
        }

        //实付报账单查询
        orderSnList = allAccountingList.stream()
                .filter(data -> Lists.newArrayList(5).contains(data.getBillType()))
                .map(data ->data.getAccountingOrderSn()).distinct().collect(Collectors.toList());
        if(CollectionUtils.isNotEmpty(orderSnList)){
            for (String accountingOrderSn : orderSnList){
                try {
                    ActualReportAccountingConfirmParam param = new ActualReportAccountingConfirmParam();
                    param.setAccountingOrderSn(accountingOrderSn);
                    accountingReportPaymenService.confirmPaymentReportList(param, loginSysUser);
                } catch (BadRequestException e) {
                    log.error("queryPushRmAccountingJob payment error accountingOrderSn={}",accountingOrderSn,e);
                }
            }
        }
    }
}
