package cn.superdesk.uniorder.executor.scheduler.xxljob.thirdpart.jd;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import cn.superdesk.uniorder.common.enums.scheduler.jd.ScheduleTaskAccountingStatusEnum;
import cn.superdesk.uniorder.common.enums.scheduler.jd.ScheduleTaskProcessStateEnum;
import cn.superdesk.uniorder.common.enums.scheduler.jd.ScheduleTaskRunStatusEnum;
import cn.superdesk.uniorder.common.json.AnJsonUtils;
import cn.superdesk.uniorder.common.security.dto.LoginSysUser;
import cn.superdesk.uniorder.dto.param.thirdpart.jd.SysQuartzScheduleTaskStatementOfAccountJdRunExcelImportForm;
import cn.superdesk.uniorder.dto.vo.accounting.unified.supplier.AccountingOrderUnifiedSupplierListItemVo;
import cn.superdesk.uniorder.dto.vo.thirdpart.jd.SysQuartzScheduleTaskStatementOfAccountJdListItemVo;
import cn.superdesk.uniorder.service.thirdpart.jd.SysQuartzScheduleTaskStatementOfAccountJdService;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @Description: Java类作用描述
 * @Author: linjinyu
 * @CreateDate: 2022/5/20 12:14 下午
 * @UpdateUser: linjinyu
 * @UpdateDate: 2022/5/20 12:14 下午
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class AccountingOrderUnifiedSupplierJdTask {

    private final SysQuartzScheduleTaskStatementOfAccountJdService sysQuartzScheduleTaskStatementOfAccountJdService;

    @XxlJob("AccountingOrderUnifiedSupplierJdTaskDeQueueOneImportExcel")
    public void deQueueOneImportExcel() {

        StringBuffer report = new StringBuffer();

        String jobParam = XxlJobHelper.getJobParam();

        report.append("\nXxlJob deQueueOneImportExcel")
                .append(StrUtil.format("\njobParam : {}", jobParam));

        LoginSysUser loginSysUser = LoginSysUser
                .builder()
                .sysUserId(996L)
                .fullName("任务执行者")
                .sysUserName("xxljob")
                .build();

        report.append(StrUtil.format("\nloginSysUser : {}", loginSysUser));

        /**************************************** 任务开始 start ****************************************/

        SysQuartzScheduleTaskStatementOfAccountJdListItemVo taskOne = sysQuartzScheduleTaskStatementOfAccountJdService
                .deQueueOneImportExcel(
                        loginSysUser
                );

        report.append(StrUtil.format("\ntaskOne : {}", JSONUtil.toJsonStr(taskOne)));

        if (ObjectUtil.isNotNull(taskOne)) {

            SysQuartzScheduleTaskStatementOfAccountJdRunExcelImportForm form =
                    AnJsonUtils.toObject(taskOne.getParams(), SysQuartzScheduleTaskStatementOfAccountJdRunExcelImportForm.class);

            report.append(StrUtil.format("\nform : {}", JSONUtil.toJsonStr(form)));

            log.info(report.toString());
//            XxlJobHelper.log(report.toString());
//            System.out.println(report.toString());

            try {

                sysQuartzScheduleTaskStatementOfAccountJdService.dispatcherExcelImportLiteVersion(form, loginSysUser);

            } catch (Exception e2) {
                log.error("XxlJob deQueueOneImportExcel 执行 异常:{}", JSONUtil.toJsonStr(form), e2);

                sysQuartzScheduleTaskStatementOfAccountJdService.towImportExcel(
                        form,
                        ScheduleTaskRunStatusEnum.已导入,
                        ScheduleTaskProcessStateEnum.导入失败,
                        loginSysUser
                );

                // TODO mark by linjinyu at 2022-06-167 01:00:28 - 告诉xxljob 此任务失败
            }


        } else {
            // 空转情况

            report.append(StrUtil.format("\nform : {}", "NULL"));

            log.info(report.toString());
//            XxlJobHelper.log(report.toString());
//            System.out.println(report.toString());
        }


        /**************************************** 任务开始   end ****************************************/
    }

    @XxlJob("AccountingOrderUnifiedSupplierJdTaskDeQueueOneCompareOrderAndCheckUpThenReport")
    public void deQueueOneCompareOrderAndCheckUpThenReport() {

        StringBuffer report = new StringBuffer();

        String jobParam = XxlJobHelper.getJobParam();

        report.append("\nXxlJob deQueueOneCompareOrderAndCheckUpThenReport")
                .append(StrUtil.format("\njobParam : {}", jobParam));

        List<AccountingOrderUnifiedSupplierListItemVo> resultList = null;
        LoginSysUser loginSysUser = LoginSysUser
                .builder()
                .sysUserId(996L)
                .fullName("任务执行者")
                .sysUserName("xxljob")
                .build();

        report.append(StrUtil.format("\nloginSysUser : {}", loginSysUser));

        /**************************************** 任务开始 start ****************************************/

        SysQuartzScheduleTaskStatementOfAccountJdListItemVo taskOne = sysQuartzScheduleTaskStatementOfAccountJdService
                .deQueueOneCompareOrderAndCheckUpThenReport(
                        loginSysUser
                );

        report.append(StrUtil.format("\ntaskOne : {}", JSONUtil.toJsonStr(taskOne)));

        if (ObjectUtil.isNotNull(taskOne)) {

            SysQuartzScheduleTaskStatementOfAccountJdRunExcelImportForm form =
                    AnJsonUtils.toObject(taskOne.getParams(), SysQuartzScheduleTaskStatementOfAccountJdRunExcelImportForm.class);

            report.append(StrUtil.format("\nform : {}", JSONUtil.toJsonStr(form)));

            log.info(report.toString());
//            XxlJobHelper.log(report.toString());
//            System.out.println(report.toString());

            try {

                sysQuartzScheduleTaskStatementOfAccountJdService.dispatcherCompareOrderAndCheckUpThenReport(form, loginSysUser);
            } catch (Exception e2) {

                log.error("\nXxlJob deQueueOneCompareOrderAndCheckUpThenReport 执行 异常:{}", JSONUtil.toJsonStr(form), e2);

                sysQuartzScheduleTaskStatementOfAccountJdService.towCompareOrderAndCheckUpThenReport(
                        form,
                        ScheduleTaskRunStatusEnum.已对账,
                        ScheduleTaskAccountingStatusEnum.对账完成,
                        loginSysUser
                );

                // TODO mark by linjinyu at 2022-06-167 01:00:28 - 告诉xxljob 此任务失败
            }


        } else {

            // 空转情况

            report.append(StrUtil.format("\nform : {}", "NULL"));

            log.info(report.toString());
//            XxlJobHelper.log(report.toString());
//            System.out.println(report.toString());
        }

        /**************************************** 任务开始   end ****************************************/
    }
}
