package cn.superdesk.uniorder.service.invoice.common.reportaccounting;

import cn.superdesk.uniorder.common.enums.common.CommonReportAccountingDataTypeEnum;
import cn.superdesk.uniorder.common.enums.common.CommonReportAccountingStateEnum;
import cn.superdesk.uniorder.common.enums.common.CommonReportAccountingTypeEnum;
import cn.superdesk.uniorder.common.security.dto.LoginSysUser;
import cn.superdesk.uniorder.dao.entity.biz.AccountingPushShareData;
import cn.superdesk.uniorder.dao.entity.biz.AccountingReceivablePayable;
import cn.superdesk.uniorder.dto.param.sharecenter.ActualAccountingAttachment;
import cn.superdesk.uniorder.dto.param.sharecenter.ActualPayableHeader;
import cn.superdesk.uniorder.dto.param.sharecenter.ActualPayableReportAccounting;
import cn.superdesk.uniorder.dto.param.sharecenter.ActualReceivedReportAccounting;
import cn.superdesk.uniorder.dto.vo.invoice.reportaccounting.ActualReportAccountingPreCheckVO;
import cn.superdesk.uniorder.dto.vo.invoice.reportaccounting.ReportAccountingDetailVO;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;

import java.util.Date;

import static cn.superdesk.uniorder.common.numerical.AnBigDecimalUtils.getBigDecimalPoint2;

public class ReportAccountingAdapter {

    public static ReportAccountingDetailVO convert2ReportAccounting(AccountingReceivablePayable data) {
        ReportAccountingDetailVO detailVO = new ReportAccountingDetailVO();
        detailVO.setId(data.getId());
        detailVO.setAccountingOrderSn(data.getAccountingOrderSn());
        detailVO.setBillType(data.getBillType());
        detailVO.setBillTypeName(CommonReportAccountingTypeEnum.get(data.getBillType()).getDesc());
        detailVO.setSettlementCode(data.getSettlementCode());
        detailVO.setSettlementName(data.getSettlementName());
        detailVO.setServiceNo(data.getBizPlatformId());
        detailVO.setServiceNoName(data.getBizPlatformName());
        detailVO.setBillAmount(getBigDecimalPoint2(data.getBillAmount()));
        detailVO.setBillNo(data.getBillNo());
        detailVO.setBillCode(data.getBillCode());
        detailVO.setDataType(data.getDataType());
        detailVO.setDataTypeName(CommonReportAccountingDataTypeEnum.get(data.getDataType()).getDesc());
        detailVO.setAccountingState(data.getAccountingState());
        detailVO.setAccountingStateName(CommonReportAccountingStateEnum.get(data.getAccountingState()).getDesc());
        detailVO.setApproveDate(data.getApproveDate());
        detailVO.setPusContent(data.getPusContent());
        detailVO.setRemark(data.getRemark());
        detailVO.setCreateBy(data.getCreateBy());
        detailVO.setCreateId(data.getCreateId());
        detailVO.setCreateTime(data.getCreateTime());
        detailVO.setUpdateBy(data.getUpdateBy());
        detailVO.setUpdateId(data.getUpdateId());
        detailVO.setUpdateTime(data.getUpdateTime());
        return detailVO;
    }

    public static AccountingReceivablePayable createAccountingReceivablePayable(LoginSysUser loginSysUser, ActualReportAccountingPreCheckVO vo) {

        CommonReportAccountingDataTypeEnum dataType = CommonReportAccountingTypeEnum.get(vo.getBillType()).getDataType();
        AccountingReceivablePayable accountingReceivablePayable = new AccountingReceivablePayable();
        accountingReceivablePayable.setAccountingOrderSn(vo.getAccountingOrderSn());
        accountingReceivablePayable.setAccountType(2);
        accountingReceivablePayable.setBillNo(vo.getActualReceivedReportAccounting().getHeader().getBatchNo());
        accountingReceivablePayable.setBillCode("");
        accountingReceivablePayable.setBillType(vo.getBillType());
        accountingReceivablePayable.setDataType(dataType.getKey());
        // 单据金额取收入明细的金额
        accountingReceivablePayable.setBillAmount(vo.getActualReceivedReportAccounting().getActualReceivedReportAccountingLineForIncome().getAmount());
        accountingReceivablePayable.setAccountingState(CommonReportAccountingStateEnum.ONE.getKey());
        accountingReceivablePayable.setShareBillNo("");
        accountingReceivablePayable.setShareBillCode("");
        accountingReceivablePayable.setShareDataState("");
        accountingReceivablePayable.setShareApproveState("");
        accountingReceivablePayable.setSharePayState("");
        accountingReceivablePayable.setApproveDate(null);

        accountingReceivablePayable.setFromSettlementId(vo.getEnterpriseId());
        // 报账主体结算中心id：数据类型为“微信”，取该对账单所属业务平台的“零星客户”企业结算中心id；
        //·报账主体业务方编号：数据类型为“微信”，取该对账单所属业务平台的“零星客户”企业业务方编号；
        //·报账主体名称：数据类型为“微信”，取该对账单所属业务平台的“零星客户”企业名称；
        accountingReceivablePayable.setSettlementId(vo.getEnterpriseId());
        accountingReceivablePayable.setSettlementCode(vo.getEnterpriseId().toString());
        accountingReceivablePayable.setSettlementName(vo.getEnterpriseName());
        accountingReceivablePayable.setBizPlatformId(vo.getServiceNo());
        accountingReceivablePayable.setBizPlatformName(vo.getServiceNoName());

        accountingReceivablePayable.setBizUserId(vo.getActualReceivedReportAccounting().getBizUserId());
        accountingReceivablePayable.setBizUserName(vo.getActualReceivedReportAccounting().getHeader().getEmployeeName());
        accountingReceivablePayable.setPusContent("");
        accountingReceivablePayable.setFileTitle("");
        accountingReceivablePayable.setFileDetailLink("");
        accountingReceivablePayable.setDelFlag("0");
        accountingReceivablePayable.setCommitBy(loginSysUser.getFullName());
        accountingReceivablePayable.setCommitId(loginSysUser.getSysUserId());
        accountingReceivablePayable.setCommitTime(new Date());
        accountingReceivablePayable.setRemark("");
        accountingReceivablePayable.setCreateBy(loginSysUser.getFullName());
        accountingReceivablePayable.setCreateId(loginSysUser.getSysUserId());
        accountingReceivablePayable.setCreateTime(new Date());
        accountingReceivablePayable.setUpdateBy(loginSysUser.getFullName());
        accountingReceivablePayable.setUpdateId(loginSysUser.getSysUserId());
        accountingReceivablePayable.setUpdateTime(new Date());
        return accountingReceivablePayable;
    }

    public static AccountingPushShareData createAccountingPushShareData(AccountingReceivablePayable accountingReceivablePayable, ActualPayableReportAccounting actualPayableReportAccounting, LoginSysUser loginSysUser) {

        AccountingPushShareData accountingPushShareData = new AccountingPushShareData();
        accountingPushShareData.setAccountingOrderSn(accountingReceivablePayable.getAccountingOrderSn());
        accountingPushShareData.setAccountingRpId(accountingReceivablePayable.getId());
        accountingPushShareData.setAccountType(3);
        accountingPushShareData.setBillType(accountingReceivablePayable.getBillType());
        accountingPushShareData.setDataType(accountingReceivablePayable.getDataType());
        ActualPayableHeader head = new ActualPayableHeader();
        head.setBoeHeader(actualPayableReportAccounting.getBoeHeader());
        head.setBoeHeaderChild(actualPayableReportAccounting.getBoeHeaderChild());
        accountingPushShareData.setHeadData(JSONObject.toJSONString(head));
        JSONArray lines = new JSONArray();
        lines.add(JSONObject.toJSON(actualPayableReportAccounting.getActualReceivedReportAccountingLineForZfs()));
        lines.add(JSONObject.toJSON(actualPayableReportAccounting.getActualPayableReportAccountingLineForApportion()));
        accountingPushShareData.setRowData(lines.toJSONString());

        JSONArray files = new JSONArray();
        for (ActualAccountingAttachment attachment : actualPayableReportAccounting.getAttachments()) {
            files.add(JSONObject.toJSON(attachment));
        }
        accountingPushShareData.setFileData(files.toJSONString());
        accountingPushShareData.setPushState(1);
        accountingPushShareData.setDelFlag("0");
        accountingPushShareData.setCreateBy(loginSysUser.getFullName());
        accountingPushShareData.setCreateId(loginSysUser.getSysUserId());
        accountingPushShareData.setCreateTime(new Date());
        accountingPushShareData.setUpdateBy(loginSysUser.getFullName());
        accountingPushShareData.setUpdateId(loginSysUser.getSysUserId());
        accountingPushShareData.setUpdateTime(new Date());
        return accountingPushShareData;
    }

    public static AccountingPushShareData createAccountingPushShareData(AccountingReceivablePayable accountingReceivablePayable, ActualReceivedReportAccounting actualReceivedReportAccounting, LoginSysUser loginSysUser) {

        AccountingPushShareData accountingPushShareData = new AccountingPushShareData();
        accountingPushShareData.setAccountingOrderSn(accountingReceivablePayable.getAccountingOrderSn());
        accountingPushShareData.setAccountingRpId(accountingReceivablePayable.getId());
        accountingPushShareData.setAccountType(3);
        accountingPushShareData.setBillType(accountingReceivablePayable.getBillType());
        accountingPushShareData.setDataType(accountingReceivablePayable.getDataType());
        accountingPushShareData.setHeadData(JSONObject.toJSONString(actualReceivedReportAccounting.getHeader()));
        JSONArray lines = new JSONArray();
        lines.add(JSONObject.toJSON(actualReceivedReportAccounting.getActualReceivedReportAccountingLineForZfs()));
        lines.add(JSONObject.toJSON(actualReceivedReportAccounting.getActualReceivedReportAccountingLineForIncome()));
        if (null != actualReceivedReportAccounting.getActualReceivedReportAccountingLineForCost()) {
            lines.add(JSONObject.toJSON(actualReceivedReportAccounting.getActualReceivedReportAccountingLineForCost()));
        }
        accountingPushShareData.setRowData(lines.toJSONString());

        JSONArray files = new JSONArray();
        for (ActualAccountingAttachment attachment : actualReceivedReportAccounting.getAttachments()) {
            files.add(JSONObject.toJSON(attachment));
        }
        accountingPushShareData.setFileData(files.toJSONString());
        accountingPushShareData.setPushState(1);
        accountingPushShareData.setDelFlag("0");
        accountingPushShareData.setCreateBy(loginSysUser.getFullName());
        accountingPushShareData.setCreateId(loginSysUser.getSysUserId());
        accountingPushShareData.setCreateTime(new Date());
        accountingPushShareData.setUpdateBy(loginSysUser.getFullName());
        accountingPushShareData.setUpdateId(loginSysUser.getSysUserId());
        accountingPushShareData.setUpdateTime(new Date());
        return accountingPushShareData;
    }

}
