package cn.superdesk.uniorder.service.invoice.customer.impl;

import cn.hutool.core.collection.CollUtil;
import cn.superdesk.uniorder.common.enums.accounting.AccountingOrderTypeDimEnum;
import cn.superdesk.uniorder.common.enums.common.CommonReportAccountingCreditTypeEnum;
import cn.superdesk.uniorder.common.enums.common.CommonReportAccountingDataTypeEnum;
import cn.superdesk.uniorder.common.enums.common.CommonReportAccountingStateEnum;
import cn.superdesk.uniorder.common.lamda.AnLamdaUtils;
import cn.superdesk.uniorder.common.numerical.AnBigDecimalUtils;
import cn.superdesk.uniorder.common.security.dto.LoginSysUser;
import cn.superdesk.uniorder.common.utils.DateUtils;
import cn.superdesk.uniorder.dao.entity.accounting.score.customer.AccountingScoreCustomer;
import cn.superdesk.uniorder.dao.entity.biz.AccountingReceivablePayable;
import cn.superdesk.uniorder.dao.entity.biz.FlowReceivableSummary;
import cn.superdesk.uniorder.dao.entity.invoice.customer.InvoiceApply;
import cn.superdesk.uniorder.dao.mapper.accounting.score.customer.AccountingScoreCustomerMapper;
import cn.superdesk.uniorder.dao.mapper.biz.AccountingPushShareDataMapper;
import cn.superdesk.uniorder.dao.mapper.biz.AccountingReceivablePayableMapper;
import cn.superdesk.uniorder.dao.mapper.biz.FlowReceivableDetailMapper;
import cn.superdesk.uniorder.dao.mapper.biz.FlowReceivableSummaryMapper;
import cn.superdesk.uniorder.dto.param.invoice.customer.reportaccounting.ActualReportAccountingConfirmParam;
import cn.superdesk.uniorder.dto.param.invoice.customer.reportaccounting.InvoiceStatusRepair;
import cn.superdesk.uniorder.dto.param.invoice.customer.status.OpBackAccountStatusForm;
import cn.superdesk.uniorder.dto.param.invoice.customer.status.OpChangeStatusBaseItem;
import cn.superdesk.uniorder.dto.param.sharecenter.ActualReceivedReportAccounting;
import cn.superdesk.uniorder.dto.vo.invoice.reportaccounting.ActualReportAccountingPreCheckVO;
import cn.superdesk.uniorder.dto.vo.sharecenter.BoeStatusResult;
import cn.superdesk.uniorder.dto.vo.sharecenter.BoeStatusVO;
import cn.superdesk.uniorder.service.accounting.score.customer.AccountingScoreCustomerService;
import cn.superdesk.uniorder.service.invoice.common.reportaccounting.ReportAccountingAdapter;
import cn.superdesk.uniorder.service.invoice.customer.InvoiceApplyCustomerService;
import cn.superdesk.uniorder.service.invoice.customer.state.InvoiceApplyCustomerFieldBackAccountStatusService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.transaction.Transactional;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 报账单执行器
 */
@Service
@Slf4j
public class ReceivedReportAccountingExecutor {
    @Resource
    private FlowReceivableDetailMapper flowReceivableDetailMapper;
    @Resource
    private InvoiceApplyCustomerFieldBackAccountStatusService invoiceApplyCustomerFieldBackAccountStatusService;
    @Resource
    private InvoiceApplyCustomerService invoiceApplyCustomerService;
    @Resource
    private AccountingScoreCustomerService accountingScoreCustomerService;
    @Resource
    private AccountingScoreCustomerMapper accountingScoreCustomerMapper;
    @Resource
    private AccountingReceivablePayableMapper accountingReceivablePayableMapper;
    @Resource
    private FlowReceivableSummaryMapper flowReceivableSummaryMapper;
    @Resource
    private AccountingPushShareDataMapper accountingPushShareDataMapper;

    public void repairInvoiceStatus(InvoiceStatusRepair repair, LoginSysUser loginSysUser) {
        OpBackAccountStatusForm form = new OpBackAccountStatusForm();

        List<InvoiceApply> invoiceApplies = invoiceApplyCustomerService.findByAccountingOrderSn(repair.getAccountingOrderSn(), AccountingOrderTypeDimEnum.积分订单);

        List<Integer> backAccountStatus = invoiceApplies.stream().filter(AnLamdaUtils.distinctByKey(InvoiceApply::getBackAccountStatus)).map(InvoiceApply::getBackAccountStatus).collect(Collectors.toList());

        if (CollUtil.isEmpty(backAccountStatus) || backAccountStatus.size() > 1) {
            throw new UnsupportedOperationException("提交的数据异常");
        }
        form.setSubmitList(invoiceApplies.stream().map(entity -> {
            OpChangeStatusBaseItem item = new OpChangeStatusBaseItem();
            item.setAccountingOrderId(repair.getAccountingOrderId());
            item.setAccountingOrderSn(repair.getAccountingOrderSn());
            item.setInvoiceApplyId(entity.getId());
            item.setInvoiceApplySn(entity.getInvoiceApplySn());
            return item;
        }).collect(Collectors.toList()));
        // 调 回款登记-确认回款
        invoiceApplyCustomerFieldBackAccountStatusService.opComplete(invoiceApplies, form, loginSysUser);
        // 应收流水明细更新
        flowReceivableDetailMapper.chargeReceivableDetail(repair.getAccountingOrderSn(), loginSysUser, new Date());
    }

    @Transactional
    public AccountingReceivablePayable create(LoginSysUser loginSysUser, ActualReportAccountingPreCheckVO vo) {
        AccountingReceivablePayable accountingReceivablePayable = ReportAccountingAdapter.createAccountingReceivablePayable(loginSysUser, vo);
        accountingReceivablePayableMapper.insertAccountingReceivablePayable(accountingReceivablePayable);
        return accountingReceivablePayable;
    }

    @Transactional
    public void saveActualReceivedAccounting(ActualReceivedReportAccounting actualReceivedReportAccounting, LoginSysUser loginSysUser, AccountingReceivablePayable accountingReceivablePayable) {

        // 更新状态
        accountingReceivablePayableMapper.changeStatus(accountingReceivablePayable.getId(), CommonReportAccountingStateEnum.TWO.getKey(), CommonReportAccountingStateEnum.ONE.getKey());
        accountingPushShareDataMapper.insertAccountingPushShareData(ReportAccountingAdapter.createAccountingPushShareData(accountingReceivablePayable, actualReceivedReportAccounting, loginSysUser));
    }

    @Transactional
    public void confirmActualReceivedAccounting(ActualReportAccountingConfirmParam param, LoginSysUser loginSysUser, Map<String, AccountingReceivablePayable> resultMap, BoeStatusResult boeStatusResult) {

        List<BoeStatusVO> boeStatusVOS = boeStatusResult.getBoeStatusVOS();
        BigDecimal receivedAccountAmount = BigDecimal.ZERO;
        BigDecimal wxReceivedAccountAmount = BigDecimal.ZERO;
        for (BoeStatusVO boeStatusVO : boeStatusVOS) {
            if (StringUtils.isBlank(boeStatusVO.getDataStatus())) {
                continue;
            }
            AccountingReceivablePayable accountingReceivablePayable = resultMap.get(boeStatusVO.getBatchNo().trim());
            accountingReceivablePayable.setUpdateId(loginSysUser.getSysUserId());
            accountingReceivablePayable.setUpdateBy(loginSysUser.getFullName());
            accountingReceivablePayable.setUpdateTime(new Date());
            accountingReceivablePayable.setShareDataState(boeStatusVO.getDataStatus());
            if ("E".equals(boeStatusVO.getDataStatus())) {
                if (StringUtils.isNotBlank(boeStatusVO.getBoeNo())) {
                    accountingReceivablePayable.setBillCode(boeStatusVO.getBoeNo());
                }
                accountingReceivablePayable.setAccountingState(CommonReportAccountingStateEnum.FOUR.getKey());
                accountingReceivablePayable.setPusContent(boeStatusVO.getErrorMessage());
            } else if ("S".equals(boeStatusVO.getDataStatus())) {
                accountingReceivablePayable.setBillCode(boeStatusVO.getBoeNo());
                accountingReceivablePayable.setApproveDate(StringUtils.isNotBlank(boeStatusVO.getEndTime()) ? DateUtils.parseDate(boeStatusVO.getEndTime()) : null);
                accountingReceivablePayable.setShareApproveState(boeStatusVO.getBoeStatus());
                accountingReceivablePayable.setSharePayState(boeStatusVO.getBoePayStatus());
                if ("30".equals(boeStatusVO.getBoeStatus())) {
                    accountingReceivablePayable.setApproveDate(StringUtils.isNotBlank(boeStatusVO.getEndTime()) ? DateUtils.parseDate(boeStatusVO.getEndTime()) : null);
                    accountingReceivablePayable.setAccountingState(CommonReportAccountingStateEnum.THREE.getKey());
                    if (Objects.equals(CommonReportAccountingDataTypeEnum.ZERO.getKey(), accountingReceivablePayable.getDataType())) {
                        receivedAccountAmount = receivedAccountAmount.add(accountingReceivablePayable.getBillAmount());
                    } else if (Objects.equals(CommonReportAccountingDataTypeEnum.ONE.getKey(), accountingReceivablePayable.getDataType())) {
                        wxReceivedAccountAmount = wxReceivedAccountAmount.add(accountingReceivablePayable.getBillAmount());
                    } else {
                        throw new IllegalArgumentException("数据类型错误");
                    }

                    processFlowReceivedSummary(accountingReceivablePayable, loginSysUser);

                } else if ("0".equals(boeStatusVO.getBoeStatus())) {
                    accountingReceivablePayable.setApproveDate(StringUtils.isNotBlank(boeStatusVO.getEndTime()) ? DateUtils.parseDate(boeStatusVO.getEndTime()) : null);
                    accountingReceivablePayable.setAccountingState(CommonReportAccountingStateEnum.FIVE.getKey());
                }
                // 其它场景仍保持审批中状态
            }
            // 其它状态只更新更新人及数据状态
            accountingReceivablePayableMapper.updateAccountingReceivablePayable(accountingReceivablePayable);
        }
        // 锁读
        AccountingScoreCustomer accountingScoreCustomer;
        if (null== param.getAccountingOrderId()) {
            accountingScoreCustomer = accountingScoreCustomerMapper.selectBySnForUpdate(param.getAccountingOrderSn());
        } else {
            accountingScoreCustomer = accountingScoreCustomerMapper.selectByPrimaryKeyForUpdate(param.getAccountingOrderId());
        }
        if (!Objects.equals(3, accountingScoreCustomer.getReceivedAccountStatus())) {
            if (receivedAccountAmount.signum() > 0) {
                BigDecimal amount = AnBigDecimalUtils.getBigDecimal(accountingScoreCustomer.getReceivedAccountAmount()).add(receivedAccountAmount);
                accountingScoreCustomer.setReceivedAccountAmount(amount);
                if (Objects.equals(accountingScoreCustomer.getAccountingAmount(), amount)) {
                    accountingScoreCustomer.setReceivedAccountStatus(2);
                } else {
                    accountingScoreCustomer.setReceivedAccountStatus(1);
                }
            }
        }
        if (!Objects.equals(3, accountingScoreCustomer.getWxReceivedAccountStatus())) {
            if (wxReceivedAccountAmount.signum() > 0) {
                BigDecimal amount = AnBigDecimalUtils.getBigDecimal(accountingScoreCustomer.getWxReceivedAccountAmount()).add(wxReceivedAccountAmount);
                accountingScoreCustomer.setWxReceivedAccountAmount(amount);
                if (Objects.equals(accountingScoreCustomer.getUnAccountingAmount(), amount)) {
                    accountingScoreCustomer.setWxReceivedAccountStatus(2);
                } else {
                    accountingScoreCustomer.setWxReceivedAccountStatus(1);
                }
            }
        }
        accountingScoreCustomerService.updateByPrimaryKeySelective(accountingScoreCustomer);

        if ((Objects.equals(2, accountingScoreCustomer.getWxReceivedAccountStatus()) || Objects.equals(3, accountingScoreCustomer.getWxReceivedAccountStatus()))
                && (Objects.equals(2, accountingScoreCustomer.getReceivedAccountStatus()) || Objects.equals(3, accountingScoreCustomer.getReceivedAccountStatus()))) {
            InvoiceStatusRepair repair = new InvoiceStatusRepair();
            repair.setAccountingOrderId(accountingScoreCustomer.getId());
            repair.setAccountingOrderSn(accountingScoreCustomer.getAccountingOrderSn());
            repairInvoiceStatus(repair, loginSysUser);
        }
    }

    private void processFlowReceivedSummary(AccountingReceivablePayable accountingReceivablePayable, LoginSysUser loginSysUser) {
        FlowReceivableSummary receivableSummary = new FlowReceivableSummary();
        Date curDate = new Date();
        receivableSummary.setRmAccountState(2);
        receivableSummary.setInAccountDate(curDate);
        receivableSummary.setInAccountType(3);
        receivableSummary.setInAccountWay(0);
        receivableSummary.setRmAccountUserId(loginSysUser.getSysUserId());
        receivableSummary.setRmAccountUserName(loginSysUser.getFullName());
        receivableSummary.setType(1);
        receivableSummary.setShareBillNo(accountingReceivablePayable.getBillNo());
        receivableSummary.setShareBillCode(accountingReceivablePayable.getBillCode());
        receivableSummary.setShareApproveDate(accountingReceivablePayable.getApproveDate());
        receivableSummary.setAccountingOrderSn(accountingReceivablePayable.getAccountingOrderSn());
        receivableSummary.setBizPlatformId(accountingReceivablePayable.getBizPlatformId());
        receivableSummary.setBizPlatformName(accountingReceivablePayable.getBizPlatformName());

        receivableSummary.setCusSettlementId(accountingReceivablePayable.getSettlementId());
        receivableSummary.setCusBizCode(accountingReceivablePayable.getSettlementCode());
        receivableSummary.setCusBizName(accountingReceivablePayable.getSettlementName());
        CommonReportAccountingCreditTypeEnum creditType = getCreditType(accountingReceivablePayable.getDataType());
        receivableSummary.setCreditType(creditType.getKey());
        receivableSummary.setCreditName(creditType.getDesc());
        receivableSummary.setOperationTaxRate(null);
        receivableSummary.setIncomeNoTax(null);
        receivableSummary.setTaxAmount(null);
        receivableSummary.setReceivableAmount(accountingReceivablePayable.getBillAmount());
        receivableSummary.setCreateId(loginSysUser.getSysUserId());
        receivableSummary.setCreateBy(loginSysUser.getFullName());
        receivableSummary.setCreateTime(curDate);
        receivableSummary.setUpdateId(loginSysUser.getSysUserId());
        receivableSummary.setUpdateBy(loginSysUser.getFullName());
        receivableSummary.setUpdateTime(curDate);
        flowReceivableSummaryMapper.insertFlowReceivableSummary(receivableSummary);
    }

    private CommonReportAccountingCreditTypeEnum getCreditType(Integer dataType) {
        if (Objects.equals(CommonReportAccountingDataTypeEnum.ZERO.getKey(), dataType)) {
            return CommonReportAccountingCreditTypeEnum.ONE;
        }
        if (Objects.equals(CommonReportAccountingDataTypeEnum.ONE.getKey(), dataType)) {
            return CommonReportAccountingCreditTypeEnum.ZERO;
        }
        throw new IllegalArgumentException("数据类型错误");
    }

    public List<AccountingReceivablePayable> findEffectiveAccountings(AccountingReceivablePayable accountingReceivablePayable) {
        List<AccountingReceivablePayable> reportAccountings = accountingReceivablePayableMapper.selectAccountingReceivablePayableList(accountingReceivablePayable);
        return reportAccountings.stream().filter(report -> report.getAccountingState() > CommonReportAccountingStateEnum.ONE.getKey()).collect(Collectors.toList());
    }

}
