package cn.superdesk.uniorder.service.invoice.customer.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.superdesk.uniorder.common.enums.accounting.AccountingOrderInvoiceApplyStatusEnum;
import cn.superdesk.uniorder.common.enums.accounting.AccountingOrderTypeDimEnum;
import cn.superdesk.uniorder.common.enums.accounting.TransferEnum;
import cn.superdesk.uniorder.common.enums.accounting.customer.AccountingOrderCustomerAccountingStatusEnum;
import cn.superdesk.uniorder.common.enums.common.CommonReportAccountingStateEnum;
import cn.superdesk.uniorder.common.enums.common.CommonReportAccountingTypeEnum;
import cn.superdesk.uniorder.common.enums.common.CommonServiceNoEnum;
import cn.superdesk.uniorder.common.enums.invoice.customer.InvoiceApplyCustomerInvoiceProgressingStatusEnum;
import cn.superdesk.uniorder.common.exception.UnifiedOrderRuntimeException;
import cn.superdesk.uniorder.common.lamda.AnLamdaUtils;
import cn.superdesk.uniorder.common.numerical.AnBigDecimalUtils;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.common.rest.response.ResponseResultOuterUtils;
import cn.superdesk.uniorder.common.security.dto.LoginSysUser;
import cn.superdesk.uniorder.common.utils.IncUtils;
import cn.superdesk.uniorder.dao.entity.accounting.score.customer.AccountingScoreCustomer;
import cn.superdesk.uniorder.dao.entity.biz.AccountingPushShareData;
import cn.superdesk.uniorder.dao.entity.biz.AccountingReceivablePayable;
import cn.superdesk.uniorder.dao.entity.invoice.customer.InvoiceApply;
import cn.superdesk.uniorder.dao.entity.organisation.SysEnterprise;
import cn.superdesk.uniorder.dao.mapper.biz.AccountingPushShareDataMapper;
import cn.superdesk.uniorder.dao.mapper.biz.AccountingReceivablePayableMapper;
import cn.superdesk.uniorder.dao.mapper.organisation.SysBusinessUserMapper;
import cn.superdesk.uniorder.dto.param.invoice.customer.reportaccounting.*;
import cn.superdesk.uniorder.dto.param.sharecenter.*;
import cn.superdesk.uniorder.dto.vo.invoice.reportaccounting.ActualReceivedReportAccountingDetailVO;
import cn.superdesk.uniorder.dto.vo.invoice.reportaccounting.ActualReceivedReportAccountingVO;
import cn.superdesk.uniorder.dto.vo.invoice.reportaccounting.ActualReportAccountingPreCheckVO;
import cn.superdesk.uniorder.dto.vo.sharecenter.ActualReceivedAccountingVO;
import cn.superdesk.uniorder.dto.vo.sharecenter.BoeStatusResult;
import cn.superdesk.uniorder.dto.vo.sys.SysAccountRealityReceivableVo;
import cn.superdesk.uniorder.dto.vo.sys.SysBusinessUserVo;
import cn.superdesk.uniorder.service.accounting.score.customer.AccountingScoreCustomerService;
import cn.superdesk.uniorder.service.invoice.common.reportaccounting.ReportAccountingAdapter;
import cn.superdesk.uniorder.service.invoice.customer.InvoiceApplyCustomerService;
import cn.superdesk.uniorder.service.invoice.customer.ReportAccountingService;
import cn.superdesk.uniorder.service.organisation.SysEnterpriseService;
import cn.superdesk.uniorder.service.sharecenter.ShareCenterService;
import cn.superdesk.uniorder.service.sys.SysAccountRealityReceivableService;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.transaction.Transactional;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.function.Function;
import java.util.stream.Collectors;

import static cn.superdesk.uniorder.common.numerical.AnBigDecimalUtils.getBigDecimalPoint2;
import static cn.superdesk.uniorder.dto._constant.ReportAccountingConstant.*;

@Slf4j
@Service
public class ReportAccountingServiceImpl implements ReportAccountingService {

    @Resource
    private AccountingScoreCustomerService accountingScoreCustomerService;

    @Resource
    private SysEnterpriseService sysEnterpriseService;

    @Resource
    private AccountingPushShareDataMapper accountingPushShareDataMapper;

    @Resource
    private AccountingReceivablePayableMapper accountingReceivablePayableMapper;

    @Resource
    private SysBusinessUserMapper sysBusinessUserMapper;

    @Resource
    private SysAccountRealityReceivableService sysAccountRealityReceivableService;

    @Resource
    private ShareCenterService shareCenterService;

    @Resource
    private ReceivedReportAccountingExecutor receivedReportAccountingExecutor;
    @Resource
    private InvoiceApplyCustomerService invoiceApplyCustomerService;

    @Resource
    private IncUtils incUtils;

    @Override
    @Transactional
    public void repairInvoiceStatus(InvoiceStatusRepair repair, LoginSysUser loginSysUser) {
        receivedReportAccountingExecutor.repairInvoiceStatus(repair, loginSysUser);
    }

    @Override
    public ActualReceivedReportAccountingVO getActualReceivedAccountingVO(ReportAccountingQueryParam param, LoginSysUser loginSysUser) {
        AccountingScoreCustomer accountingScoreCustomer = accountingScoreCustomerService.findById(param.getAccountingOrderId());

        ActualReceivedReportAccountingVO vo = new ActualReceivedReportAccountingVO();
        vo.setAccountingOrderId(accountingScoreCustomer.getId());
        vo.setAccountingOrderSn(accountingScoreCustomer.getAccountingOrderSn());

        SysEnterprise sysEnterprise = sysEnterpriseService.findByOutVirtId(accountingScoreCustomer.getEnterpriseId());
        vo.setEnterpriseId(sysEnterprise.getEnterpriseId());
        vo.setEnterpriseName(sysEnterprise.getEnterpriseName());
        vo.setServiceNo(accountingScoreCustomer.getServiceNo());
        vo.setServiceNoName(CommonServiceNoEnum.getByServiceNo(accountingScoreCustomer.getServiceNo()).getServiceName());
        vo.setAccountingStatus(accountingScoreCustomer.getAccountingStatus());
        vo.setAccountingStatusName(AccountingOrderCustomerAccountingStatusEnum.get(accountingScoreCustomer.getAccountingStatus()).getDesc());
        vo.setInvoiceApplyAmount(getBigDecimalPoint2(accountingScoreCustomer.getStatisticsTotalAmount()));
        vo.setWeChatAmount(getBigDecimalPoint2(accountingScoreCustomer.getUnAccountingAmount()));
        vo.setSettlementAmount(getBigDecimalPoint2(accountingScoreCustomer.getAccountingAmount()));
        vo.setActualAmountWeChat(getBigDecimalPoint2(accountingScoreCustomer.getWxReceivedAccountAmount()));
        vo.setActualAmountNonWeChat(getBigDecimalPoint2(accountingScoreCustomer.getReceivedAccountAmount()));
        vo.setReceivedAccountStatus(accountingScoreCustomer.getReceivedAccountStatus());
        vo.setWxReceivedAccountStatus(accountingScoreCustomer.getWxReceivedAccountStatus());
        vo.setTip(TIP);

        AccountingReceivablePayable accountingReceivablePayable = new AccountingReceivablePayable();
        accountingReceivablePayable.setAccountingOrderSn(param.getAccountingOrderSn());
        accountingReceivablePayable.setAccountType(2);
        accountingReceivablePayable.setBillTypeList(Lists.newArrayList(3, 4));

        List<AccountingReceivablePayable> reportAccountings = receivedReportAccountingExecutor.findEffectiveAccountings(accountingReceivablePayable);

        vo.setDetails(reportAccountings.stream().map(ReportAccountingAdapter::convert2ReportAccounting).collect(Collectors.toList()));
        return vo;
    }

    @Override
    public ActualReceivedReportAccountingDetailVO getActualReceivedReportAccountingDetail(ActualReportAccountingQueryParam param, LoginSysUser loginSysUser) {
        ActualReceivedReportAccountingDetailVO vo = new ActualReceivedReportAccountingDetailVO();
        AccountingReceivablePayable data = accountingReceivablePayableMapper.selectAccountingReceivablePayableById(param.getId());
        vo.setReportAccountingDetailVO(ReportAccountingAdapter.convert2ReportAccounting(data));
        AccountingPushShareData accountingPushShareData = new AccountingPushShareData();
        accountingPushShareData.setAccountingRpId(data.getId());
        accountingPushShareData.setAccountingOrderSn(data.getAccountingOrderSn());
        AccountingPushShareData result = accountingPushShareDataMapper.selectAccountingPushShareDataList(accountingPushShareData).get(0);
        ActualReceivedReportAccountingHeader headData = JSONObject.parseObject(result.getHeadData(), ActualReceivedReportAccountingHeader.class);
        ActualReceivedReportAccounting actualReportAccounting = new ActualReceivedReportAccounting();
        actualReportAccounting.setBindOperationUserid(data.getCreateId());
        actualReportAccounting.setBindOperationUser(data.getCreateBy());
        actualReportAccounting.setHeader(headData);
        JSONArray lines = JSONObject.parseArray(result.getRowData());
        for (int i = 0; i < lines.size(); i++) {
            JSONObject line = lines.getJSONObject(i);
            switch ((String) line.get("lineType")) {
                case "cost":
                    actualReportAccounting.setActualReceivedReportAccountingLineForCost(line.toJavaObject(ActualReceivedReportAccountingLineForCost.class));
                    break;
                case "income":
                    actualReportAccounting.setActualReceivedReportAccountingLineForIncome(line.toJavaObject(ActualReceivedReportAccountingLineForIncome.class));
                    break;
                case "zfsBoeBankStatements":
                    actualReportAccounting.setActualReceivedReportAccountingLineForZfs(line.toJavaObject(ActualReceivedReportAccountingLineForZfs.class));
                    break;
                default:
                    throw new IllegalArgumentException("行类型错误");
            }
        }
        actualReportAccounting.setAttachments(JSONObject.parseArray(result.getFileData(), ActualAccountingAttachment.class));
        vo.setActualReceivedReportAccounting(actualReportAccounting);
        return vo;
    }

    @Override
    public R<ActualReportAccountingPreCheckVO> preCheckByCreateActualReceivedAccounting(ActualReportAccountingPreCheckParam param, LoginSysUser loginSysUser) {

        try {
            AccountingScoreCustomer accountingScoreCustomer = accountingScoreCustomerService.findById(param.getAccountingOrderId());
            ActualReportAccountingPreCheckVO vo = preCheck(param.getBillType(), accountingScoreCustomer, loginSysUser);
            // 给前端时生成单据流水号
            vo.getActualReceivedReportAccounting().getHeader().setBatchNo(incUtils.getBillNo(vo.getServiceNo()));
            return new R<>(vo);
        } catch (UnifiedOrderRuntimeException ue) {
            return ResponseResultOuterUtils.createErrorResult(String.valueOf(ue.getCode()), ue.getMessage());
        }
    }

    private ActualReportAccountingPreCheckVO preCheck(Integer billType, AccountingScoreCustomer accountingScoreCustomer, LoginSysUser loginSysUser) {

        ActualReportAccountingPreCheckVO vo = new ActualReportAccountingPreCheckVO();
        vo.setAccountingOrderId(accountingScoreCustomer.getId());
        vo.setAccountingOrderSn(accountingScoreCustomer.getAccountingOrderSn());
        // 微信的取零星客户（ID为 1000000）
        SysEnterprise sysEnterprise = sysEnterpriseService.findByOutVirtId(billType == 3 ? accountingScoreCustomer.getEnterpriseId() : 1000000);
        // ①非微信的实收报账单：取原对账单所属企业；
        // ②微信的实收报账单：取该业务平台的“零星客户”企业；
        vo.setEnterpriseId(sysEnterprise.getEnterpriseId());
        vo.setEnterpriseNo(sysEnterprise.getEnterpriseNo());
        vo.setEnterpriseName(sysEnterprise.getEnterpriseName());
        vo.setServiceNo(accountingScoreCustomer.getServiceNo());
        String serviceName = CommonServiceNoEnum.getByServiceNo(accountingScoreCustomer.getServiceNo()).getServiceName();
        vo.setServiceNoName(serviceName);
        SysBusinessUserVo sysBusinessUserVo = sysBusinessUserMapper.selectByBindOperationUserid(loginSysUser.getSysUserId());
        if (null == sysBusinessUserVo) {
            throw new UnifiedOrderRuntimeException(500, "当前操作账号未关联业务人员，请联系管理员关联绑定");
        }
        vo.setBillType(billType);
        vo.setTip(TIP_DETAIL);

        vo.setActualReceivedReportAccounting(createActualReceivedReportAccounting(billType, vo, accountingScoreCustomer, serviceName, sysEnterprise, sysBusinessUserVo, loginSysUser));

        return vo;
    }

    private ActualReceivedReportAccounting createActualReceivedReportAccounting(Integer billType, ActualReportAccountingPreCheckVO vo, AccountingScoreCustomer accountingScoreCustomer, String serviceName, SysEnterprise sysEnterprise, SysBusinessUserVo sysBusinessUserVo, LoginSysUser loginSysUser) {

        SysAccountRealityReceivableVo sysAccountRealityReceivableVo = sysAccountRealityReceivableService.find();
        if (1 != sysAccountRealityReceivableVo.getDataIsComplete()) {
            throw new UnifiedOrderRuntimeException(500, "共享参数配置信息不完整：请前往“系统管理-共享参数配置-完善“实收数据同步接口”相关信息");
        }
        String date = DateUtil.format(new Date(), "yyyy-MM-dd");

        ActualReceivedReportAccounting actualReceivedReportAccounting = new ActualReceivedReportAccounting();
        actualReceivedReportAccounting.setBizUserId(sysBusinessUserVo.getId());
        actualReceivedReportAccounting.setBindOperationUserid(loginSysUser.getSysUserId());
        actualReceivedReportAccounting.setBindOperationUser(loginSysUser.getFullName());
        actualReceivedReportAccounting.setHeader(createActualReceivedReportAccountingHeader(accountingScoreCustomer, serviceName, sysEnterprise, sysAccountRealityReceivableVo, sysBusinessUserVo, date));
        actualReceivedReportAccounting.setAttachments(Lists.newArrayList());
        // 非微信
        if (3 == billType) {
            vo.setAccountingAmount(getBigDecimalPoint2(accountingScoreCustomer.getAccountingAmount()));
            vo.setReceivedAmount(getBigDecimalPoint2(accountingScoreCustomer.getReceivedAccountAmount()));

            actualReceivedReportAccounting.setActualReceivedReportAccountingLineForIncome(createActualReceivedReportAccountingLineForIncome(accountingScoreCustomer, sysAccountRealityReceivableVo, sysBusinessUserVo, BigDecimal.ZERO, sysEnterprise));
            actualReceivedReportAccounting.setActualReceivedReportAccountingLineForZfs(createActualReceivedReportAccountingLineForZfs(accountingScoreCustomer, serviceName, sysEnterprise, sysAccountRealityReceivableVo, BigDecimal.ZERO, null, ""));
        } else if (4 == billType) {
            // 微信
            vo.setAccountingAmount(getBigDecimalPoint2(accountingScoreCustomer.getUnAccountingAmount()));
            vo.setReceivedAmount(getBigDecimalPoint2(accountingScoreCustomer.getWxReceivedAccountAmount()));

            BigDecimal feeAmount = accountingScoreCustomer.getUnAccountingAmount().multiply(RATE).setScale(2, RoundingMode.UP);
            BigDecimal currencyAmount = accountingScoreCustomer.getUnAccountingAmount().subtract(feeAmount).setScale(2, RoundingMode.UP);
            actualReceivedReportAccounting.setActualReceivedReportAccountingLineForIncome(createActualReceivedReportAccountingLineForIncome(accountingScoreCustomer, sysAccountRealityReceivableVo, sysBusinessUserVo, accountingScoreCustomer.getUnAccountingAmount(), sysEnterprise));

            actualReceivedReportAccounting.setActualReceivedReportAccountingLineForZfs(createActualReceivedReportAccountingLineForZfs(accountingScoreCustomer, serviceName, sysEnterprise, sysAccountRealityReceivableVo, currencyAmount, date, sysAccountRealityReceivableVo.getWechatTradeNo()));
            actualReceivedReportAccounting.setActualReceivedReportAccountingLineForCost(createActualReceivedReportAccountingLineForCost(sysAccountRealityReceivableVo, sysBusinessUserVo, feeAmount, sysEnterprise));
        } else {
            throw new IllegalArgumentException("报账单类型错误");
        }
        return actualReceivedReportAccounting;
    }

    private ActualReceivedReportAccountingHeader createActualReceivedReportAccountingHeader(AccountingScoreCustomer accountingScoreCustomer, String serviceName, SysEnterprise sysEnterprise, SysAccountRealityReceivableVo sysAccountRealityReceivableVo, SysBusinessUserVo sysBusinessUserVo, String date) {

        ActualReceivedReportAccountingHeader header = new ActualReceivedReportAccountingHeader();
        header.setDataSource(DATASOURCE);
        header.setBatchNo("");
        header.setGroupCode(GROUP_CODE);
        header.setBoeDate(date);
        header.setBusinessCategory(sysAccountRealityReceivableVo.getBusinessCategory());
        header.setPaperAccessories(PAPER_ACCESSORIES);
        header.setOperationTypeCode(sysAccountRealityReceivableVo.getOperationTypeCode());
        header.setEmployeeId(sysBusinessUserVo.getWorkNumber());
        header.setEmployeeName(sysBusinessUserVo.getUserName());
        header.setEmployeePostId(sysBusinessUserVo.getJobCode());
        header.setEmployeePostName(sysBusinessUserVo.getJobName());
        header.setEmployeeDepartmentId(sysBusinessUserVo.getDepartmentCode());
        header.setEmployeeDepartmentName(sysBusinessUserVo.getDepartmentName());
        header.setCompanyNum(sysAccountRealityReceivableVo.getCompanyNum());
        header.setCompanyName(sysAccountRealityReceivableVo.getCompanyName());
        header.setRemark(String.format(REMARK, DateUtil.format(accountingScoreCustomer.getStartTime(), FORMAT_DATE), DateUtil.format(accountingScoreCustomer.getEndTime(), FORMAT_DATE), serviceName, sysEnterprise.getEnterpriseName(), accountingScoreCustomer.getAccountingOrderSn()));
        return header;
    }

    private ActualReceivedReportAccountingLineForIncome createActualReceivedReportAccountingLineForIncome(AccountingScoreCustomer accountingScoreCustomer, SysAccountRealityReceivableVo sysAccountRealityReceivableVo, SysBusinessUserVo sysBusinessUserVo, BigDecimal amount, SysEnterprise sysEnterprise) {

        ActualReceivedReportAccountingLineForIncome actualReceivedReportAccountingLineForIncome = new ActualReceivedReportAccountingLineForIncome();
        actualReceivedReportAccountingLineForIncome.setOperationSubTypeCode(sysAccountRealityReceivableVo.getIncomeDetailTypeCode());
        actualReceivedReportAccountingLineForIncome.setExpenseDeptId(sysBusinessUserVo.getDutyDepartmentCode());
        actualReceivedReportAccountingLineForIncome.setExpenseDeptName(sysBusinessUserVo.getDutyDepartmentName());
        actualReceivedReportAccountingLineForIncome.setCostcenterId(sysAccountRealityReceivableVo.getCostcenterId());
        actualReceivedReportAccountingLineForIncome.setCostcenterName(sysAccountRealityReceivableVo.getCostcenterName());
        actualReceivedReportAccountingLineForIncome.setCurrencyAmount(amount);
        // 非微信不输出，前端根据数据值自动计算
        actualReceivedReportAccountingLineForIncome.setAmount(getBigDecimalPoint2(amount));
        actualReceivedReportAccountingLineForIncome.setTaxAmount(BigDecimal.ZERO);
        actualReceivedReportAccountingLineForIncome.setTaxRate(BigDecimal.ZERO);
        actualReceivedReportAccountingLineForIncome.setTaxType("YBNS01");
        // ①非微信的实收报账单：取该对账单所属企业的“数据湖_客户名称”字段的数据；
        // ②微信的实收报账单：取该对账单所属业务平台“零星客户”企业的“数据湖_客户名称”字段的数据；
        actualReceivedReportAccountingLineForIncome.setCustomerNum(sysEnterprise.getCustomerUuid());
        actualReceivedReportAccountingLineForIncome.setCustomerName(sysEnterprise.getCustomerName());
        actualReceivedReportAccountingLineForIncome.setContractNum("");
        actualReceivedReportAccountingLineForIncome.setContractName("");
        // 新增需求，从 sysAccountRealityReceivableVo 取项目信息
        actualReceivedReportAccountingLineForIncome.setProjectNum(sysAccountRealityReceivableVo.getProjectNum());
        actualReceivedReportAccountingLineForIncome.setProjectName(sysAccountRealityReceivableVo.getProjectName());
        actualReceivedReportAccountingLineForIncome.setFormatType(sysAccountRealityReceivableVo.getFormatType());
        // ①若该报账单的业务平台为“福利商城”：取“福利商城_服务及商品编码”字段数据；
        // ②若该报账单的业务平台为“企业商城”：取“企业商城_服务及商品编码”字段数据；根据业务平台判断
        actualReceivedReportAccountingLineForIncome.setServiceType(getServiceType(accountingScoreCustomer, sysAccountRealityReceivableVo));
        actualReceivedReportAccountingLineForIncome.setYydy("");
        actualReceivedReportAccountingLineForIncome.setSupplierNum("");
        return actualReceivedReportAccountingLineForIncome;
    }

    private String getServiceType(AccountingScoreCustomer accountingScoreCustomer, SysAccountRealityReceivableVo sysAccountRealityReceivableVo) {
        if ("QYSC_QCH".equals(accountingScoreCustomer.getServiceNo())) {
            return sysAccountRealityReceivableVo.getServiceType();
        } else if ("FLSC_QCH".equals(accountingScoreCustomer.getServiceNo())) {
            return sysAccountRealityReceivableVo.getWelfareShopServiceType();
        } else {
            throw new IllegalArgumentException("对账单ServiceNo 错误");
        }
    }

    private ActualReceivedReportAccountingLineForZfs createActualReceivedReportAccountingLineForZfs(AccountingScoreCustomer accountingScoreCustomer, String serviceName, SysEnterprise sysEnterprise, SysAccountRealityReceivableVo sysAccountRealityReceivableVo, BigDecimal amount, String date, String recordId) {

        ActualReceivedReportAccountingLineForZfs actualReceivedReportAccountingLineForZfs = new ActualReceivedReportAccountingLineForZfs();
        actualReceivedReportAccountingLineForZfs.setReceiveType(TransferEnum.OCTYHZZ.name());
        actualReceivedReportAccountingLineForZfs.setReceiptBankAccNum(sysAccountRealityReceivableVo.getReceiptBankAccNum());
        // 账单周期（开始日期至结束日期）”+业务平台“福利商城/企业商城”_该报账单的报账主体企业名称_该报账单所属的客户对账单编号_实收数据
        actualReceivedReportAccountingLineForZfs.setBoeAbstract(String.format(BOE_ABSTRACT, DateUtil.format(accountingScoreCustomer.getStartTime(), FORMAT_DATE), DateUtil.format(accountingScoreCustomer.getEndTime(), FORMAT_DATE), serviceName, sysEnterprise.getEnterpriseName(), accountingScoreCustomer.getAccountingOrderSn()));
        actualReceivedReportAccountingLineForZfs.setCurrencyAmount(getBigDecimalPoint2(amount));
        actualReceivedReportAccountingLineForZfs.setAmount(getBigDecimalPoint2(amount));
        // 微信为当天日期 年-月-日，非微信不输出
        if (null != date) {
            actualReceivedReportAccountingLineForZfs.setSignDate(date);
        }
        // 非微信不输出
        actualReceivedReportAccountingLineForZfs.setRecordId(recordId);
        return actualReceivedReportAccountingLineForZfs;
    }

    private ActualReceivedReportAccountingLineForCost createActualReceivedReportAccountingLineForCost(SysAccountRealityReceivableVo sysAccountRealityReceivableVo, SysBusinessUserVo sysBusinessUserVo, BigDecimal feeAmount, SysEnterprise sysEnterprise) {

        ActualReceivedReportAccountingLineForCost actualReceivedReportAccountingLineForCost = new ActualReceivedReportAccountingLineForCost();
        actualReceivedReportAccountingLineForCost.setOperationSubTypeCode(sysAccountRealityReceivableVo.getOperationSubTypeCode());
        actualReceivedReportAccountingLineForCost.setExpenseDeptId(sysBusinessUserVo.getDutyDepartmentCode());
        actualReceivedReportAccountingLineForCost.setExpenseDeptName(sysBusinessUserVo.getDutyDepartmentName());
        actualReceivedReportAccountingLineForCost.setCostcenterId(sysAccountRealityReceivableVo.getCostcenterId());
        actualReceivedReportAccountingLineForCost.setCostcenterName(sysAccountRealityReceivableVo.getCostcenterName());
        actualReceivedReportAccountingLineForCost.setCurrencyAmount(getBigDecimalPoint2(feeAmount));
        actualReceivedReportAccountingLineForCost.setAmount(getBigDecimalPoint2(feeAmount));
        // 取该报账单所属业务平台“零星客户”企业的“数据湖_客户名称”字段的数据；
        // 取该报账单所属业务平台“零星客户”企业的“数据湖_客户uuid”字段的数据；
        actualReceivedReportAccountingLineForCost.setCustomerNum(sysEnterprise.getCustomerUuid());
        actualReceivedReportAccountingLineForCost.setCustomerName(sysEnterprise.getCustomerName());

        actualReceivedReportAccountingLineForCost.setContractNum("");
        actualReceivedReportAccountingLineForCost.setContractName("");
        // 新增需求，从 sysAccountRealityReceivableVo 取项目信息
        actualReceivedReportAccountingLineForCost.setProjectNum(sysAccountRealityReceivableVo.getProjectNum());
        actualReceivedReportAccountingLineForCost.setProjectName(sysAccountRealityReceivableVo.getProjectName());
        actualReceivedReportAccountingLineForCost.setSupplierNum("");
        return actualReceivedReportAccountingLineForCost;
    }

    @Override
    public R<?> createActualReceivedAccounting(ActualReportAccountingCreateParam param, LoginSysUser loginSysUser) {
        AccountingScoreCustomer accountingScoreCustomer = accountingScoreCustomerService.findById(param.getAccountingOrderId());
        Integer accountingStatus = accountingScoreCustomer.getAccountingStatus();
        if (!Objects.equals(AccountingOrderCustomerAccountingStatusEnum.已确认.getKey(), accountingStatus)) {
            return ResponseResultOuterUtils.createErrorResult("500", "该报账单所属的对账单状态发生变更，不允许推送，请返回检查");
        }

        Integer invoiceStatus = accountingScoreCustomer.getInvoiceStatus();
        if (!Objects.equals(AccountingOrderInvoiceApplyStatusEnum.全部申请.getKey(), invoiceStatus)) {
            return ResponseResultOuterUtils.createErrorResult("500", "该对账单申请状态未全部申请，不允许推送，请返回检查");
        }

        List<InvoiceApply> invoiceApplies = invoiceApplyCustomerService.findByAccountingOrderSn(param.getAccountingOrderSn(), AccountingOrderTypeDimEnum.积分订单);
        List<Integer> status = invoiceApplies.stream().filter(AnLamdaUtils.distinctByKey(InvoiceApply::getInvoiceStatus)).map(InvoiceApply::getInvoiceStatus).collect(Collectors.toList());
        if (CollUtil.isEmpty(status) || status.size() > 1  || !Objects.equals(status.get(0), InvoiceApplyCustomerInvoiceProgressingStatusEnum.待回款.getKey())) {
            return ResponseResultOuterUtils.createErrorResult("500", "该对账单部分开票申请的申请状态“非待回款”，不允许推送，请返回检查");
        }

        ActualReportAccountingPreCheckVO actualReportAccountingPreCheckVO = preCheck(param.getBillType(), accountingScoreCustomer, loginSysUser);

        ActualReceivedReportAccounting actualReceivedReportAccounting = actualReportAccountingPreCheckVO.getActualReceivedReportAccounting();
        // 使用输入的单据流水号
        actualReceivedReportAccounting.getHeader().setBatchNo(param.getBillNo());
        actualReceivedReportAccounting.getHeader().setRemark(param.getRemark());
        actualReceivedReportAccounting.getActualReceivedReportAccountingLineForZfs().setRecordId(param.getRecordId());
        actualReceivedReportAccounting.getActualReceivedReportAccountingLineForZfs().setSignDate(param.getSignDate());
        actualReceivedReportAccounting.getActualReceivedReportAccountingLineForZfs().setBoeAbstract(param.getBoeAbstract());
        actualReceivedReportAccounting.getActualReceivedReportAccountingLineForZfs().setAmount(param.getAmount());
        actualReceivedReportAccounting.getActualReceivedReportAccountingLineForZfs().setCurrencyAmount(param.getAmount());

        if (Objects.equals(param.getBillType(), CommonReportAccountingTypeEnum.THREE.getKey())) {
            // 若为非微信报账单，收入明细的金额 = 收款金额
            actualReceivedReportAccounting.getActualReceivedReportAccountingLineForIncome().setAmount(param.getAmount());
            actualReceivedReportAccounting.getActualReceivedReportAccountingLineForIncome().setCurrencyAmount(param.getAmount());
        }
        AccountingReceivablePayable accountingReceivablePayable = accountingReceivablePayableMapper.selectAccountingReceivablePayableByBillNo(param.getBillNo());
        if (null != accountingReceivablePayable) {
            if (accountingReceivablePayable.getAccountingState() > CommonReportAccountingStateEnum.ONE.getKey()) {
                return ResponseResultOuterUtils.createErrorResult("500", "该报账单已提交，请勿重复提交，请返回查看");
            }
        } else {
            try {
                // 若报账单不存在，先报账单落库
                accountingReceivablePayable = receivedReportAccountingExecutor.create(loginSysUser, actualReportAccountingPreCheckVO);
            } catch (DuplicateKeyException e) {
                return ResponseResultOuterUtils.createErrorResult("500", "该报账单已提交，请勿重复提交，请返回查看");
            }
        }
        // 单据金额取收入明细的金额
        BigDecimal billAmount = actualReceivedReportAccounting.getActualReceivedReportAccountingLineForIncome().getAmount();
        BigDecimal approvePaymentAmount = accountingReceivablePayableMapper.sumApprovePaymentAmount(param.getAccountingOrderSn(), param.getBillType());
        if (Objects.equals(param.getBillType(), CommonReportAccountingTypeEnum.FOUR.getKey())) {
            if (AnBigDecimalUtils.getBigDecimal(approvePaymentAmount).add(billAmount).compareTo(accountingScoreCustomer.getUnAccountingAmount()) > 0) {
                return ResponseResultOuterUtils.createErrorResult("500", "收款金额不能大于该报账单的实收账款总金额，请重新输入");
            }
        } else if (Objects.equals(param.getBillType(), CommonReportAccountingTypeEnum.THREE.getKey())) {
            if (AnBigDecimalUtils.getBigDecimal(approvePaymentAmount).add(billAmount).compareTo(accountingScoreCustomer.getAccountingAmount()) > 0) {
                return ResponseResultOuterUtils.createErrorResult("500", "收款金额不能大于该报账单的实收账款总金额，请重新输入");
            }
        } else {
            return ResponseResultOuterUtils.createErrorResult("500", "报账单类型错误");
        }

        ActualReceivedAccountingVO actualReceivedAccountingVO;
        try {
            actualReceivedAccountingVO = shareCenterService.synNewCollectionFundBoe(actualReceivedReportAccounting);
        } catch (Exception e) {
            log.error("创建实收报账单异常", e);
            return ResponseResultOuterUtils.createErrorResult("500", "共享中心数据同步接口：调用失败无响应");
        }
        if (0 != actualReceivedAccountingVO.getCode()) {
            return ResponseResultOuterUtils.createErrorResult("500", actualReceivedAccountingVO.getMsg());
        }
        receivedReportAccountingExecutor.saveActualReceivedAccounting(actualReceivedReportAccounting, loginSysUser, accountingReceivablePayable);
        return ResponseResultOuterUtils.createSuccesResult();
    }

    @Override
    public void modifyActualReceivedAccounting(ActualReportAccountingModifyParam param, LoginSysUser loginSysUser) {
        AccountingReceivablePayable accountingReceivablePayable = new AccountingReceivablePayable();
        accountingReceivablePayable.setId(param.getId());
        accountingReceivablePayable.setRemark(param.getRemark());
        accountingReceivablePayable.setUpdateBy(loginSysUser.getFullName());
        accountingReceivablePayable.setUpdateId(loginSysUser.getSysUserId());
        accountingReceivablePayable.setUpdateTime(new Date());
        accountingReceivablePayableMapper.updateAccountingReceivablePayable(accountingReceivablePayable);
    }

    @Override
    public R<?> confirmActualReceivedAccounting(ActualReportAccountingConfirmParam param, LoginSysUser loginSysUser) {
        AccountingReceivablePayable accountingReceivablePayableParam = new AccountingReceivablePayable();
        accountingReceivablePayableParam.setAccountingOrderSn(param.getAccountingOrderSn());
        accountingReceivablePayableParam.setAccountType(2);
        accountingReceivablePayableParam.setBillTypeList(Lists.newArrayList(CommonReportAccountingTypeEnum.THREE.getKey(), CommonReportAccountingTypeEnum.FOUR.getKey()));
        accountingReceivablePayableParam.setAccountingState(2);
        Map<String, AccountingReceivablePayable> resultMap = accountingReceivablePayableMapper.selectAccountingReceivablePayableList(accountingReceivablePayableParam)
                .stream().collect(Collectors.toMap(result -> result.getBillNo().trim(), Function.identity()));
        BoeStatusResult boeStatusResult;

        List<BoeStatusParam> params = resultMap.values().stream().map(result -> {
            BoeStatusParam boeStatusParam = new BoeStatusParam();
            boeStatusParam.setDataSource(DATASOURCE);
            boeStatusParam.setGroupCode(GROUP_CODE);
            boeStatusParam.setBatchNo(result.getBillNo());
            boeStatusParam.setBoeNo("");
            return boeStatusParam;
        }).collect(Collectors.toList());
        List<List<BoeStatusParam>> partitions = Lists.partition(params, 10);
        for (List<BoeStatusParam> partition : partitions) {
            try {
                boeStatusResult = shareCenterService.queryBoeStatus(partition);
                if (0 != boeStatusResult.getCode()) {
                    continue;
                }
            } catch (Exception e) {
                log.error("数据审批状态查询接口：调用失败无响应或超时", e);
                return ResponseResultOuterUtils.createErrorResult("500", "数据审批状态查询接口：调用失败无响应或超时");
            }
            receivedReportAccountingExecutor.confirmActualReceivedAccounting(param, loginSysUser, resultMap, boeStatusResult);
        }
        return ResponseResultOuterUtils.createSuccesResult();
    }

}
