package cn.superdesk.uniorder.service.invoice.supplier.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.superdesk.uniorder.common.enums.accounting.TransferEnum;
import cn.superdesk.uniorder.common.enums.accounting.customer.AccountingOrderCustomerAccountingStatusEnum;
import cn.superdesk.uniorder.common.enums.common.CommonReportAccountingStateEnum;
import cn.superdesk.uniorder.common.enums.common.CommonReportAccountingTypeEnum;
import cn.superdesk.uniorder.common.enums.common.CommonServiceNoEnum;
import cn.superdesk.uniorder.common.enums.invoice.supplier.InvoiceApplySupplierInvoiceProgressingStatusEnum;
import cn.superdesk.uniorder.common.enums.invoice.supplier.PaymentStatusEnum;
import cn.superdesk.uniorder.common.exception.UnifiedOrderRuntimeException;
import cn.superdesk.uniorder.common.lamda.AnLamdaUtils;
import cn.superdesk.uniorder.common.numerical.AnBigDecimalUtils;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.common.rest.response.ResponseResultOuterUtils;
import cn.superdesk.uniorder.common.security.dto.LoginSysUser;
import cn.superdesk.uniorder.common.utils.IncUtils;
import cn.superdesk.uniorder.dao.entity.accounting.unified.supplier.AccountingUnifiedSupplier;
import cn.superdesk.uniorder.dao.entity.biz.AccountingPushShareData;
import cn.superdesk.uniorder.dao.entity.biz.AccountingReceivablePayable;
import cn.superdesk.uniorder.dao.entity.invoice.supplier.InvoiceApplySupplier;
import cn.superdesk.uniorder.dao.entity.organisation.UnifiedSupplier;
import cn.superdesk.uniorder.dao.mapper.biz.AccountingPushShareDataMapper;
import cn.superdesk.uniorder.dao.mapper.biz.AccountingReceivablePayableMapper;
import cn.superdesk.uniorder.dao.mapper.invoice.supplier.AccountingReportPaymentMapper;
import cn.superdesk.uniorder.dao.mapper.organisation.SysBusinessUserMapper;
import cn.superdesk.uniorder.dao.mapper.sys.SysAccountRealityPayableMapper;
import cn.superdesk.uniorder.dto.param.invoice.customer.reportaccounting.ActualPayableReportAccountingCreateParam;
import cn.superdesk.uniorder.dto.param.invoice.customer.reportaccounting.ActualReportAccountingConfirmParam;
import cn.superdesk.uniorder.dto.param.invoice.customer.reportaccounting.InvoiceStatusRepair;
import cn.superdesk.uniorder.dto.param.sharecenter.*;
import cn.superdesk.uniorder.dto.vo.invoice.reportaccounting.ActualPayableReportAccountingDetailVO;
import cn.superdesk.uniorder.dto.vo.invoice.supplier.paymentReport.AddPaymentReportEchoVo;
import cn.superdesk.uniorder.dto.vo.invoice.supplier.paymentReport.PaymentReportEchoVo;
import cn.superdesk.uniorder.dto.vo.sharecenter.ActualPayableAccountingVO;
import cn.superdesk.uniorder.dto.vo.sharecenter.BoeData;
import cn.superdesk.uniorder.dto.vo.sharecenter.BoeStatusResult;
import cn.superdesk.uniorder.dto.vo.sys.SysAccountRealityPayableVo;
import cn.superdesk.uniorder.dto.vo.sys.SysBusinessUserVo;
import cn.superdesk.uniorder.service.accounting.unified.supplier.AccountingUnifiedSupplierExportExcelService;
import cn.superdesk.uniorder.service.accounting.unified.supplier.AccountingUnifiedSupplierService;
import cn.superdesk.uniorder.service.invoice.common.reportaccounting.ReportAccountingAdapter;
import cn.superdesk.uniorder.service.invoice.supplier.AccountingReportPaymenService;
import cn.superdesk.uniorder.service.invoice.supplier.InvoiceApplySupplierService;
import cn.superdesk.uniorder.service.organisation.SysSupplierService;
import cn.superdesk.uniorder.service.sharecenter.ShareCenterService;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.transaction.Transactional;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static cn.superdesk.uniorder.common.numerical.AnBigDecimalUtils.getBigDecimalPoint2;
import static cn.superdesk.uniorder.dto._constant.ReportAccountingConstant.*;

/**
 * zhuy 2022-11-27
 */
@Slf4j
@Service
public class AccountingReportPaymentServiceImpl implements AccountingReportPaymenService {
    @Resource
    private AccountingReportPaymentMapper accountingReportPaymentMapper;
    @Resource
    private SysBusinessUserMapper sysBusinessUserMapper;
    @Resource
    private AccountingReceivablePayableMapper accountingReceivablePayableMapper;
    @Resource
    private AccountingPushShareDataMapper accountingPushShareDataMapper;
    @Resource
    private ShareCenterService shareCenterService;
    @Resource
    private PaidReportAccountingExecutor paidReportAccountingExecutor;
    @Resource
    private AccountingUnifiedSupplierService accountingUnifiedSupplierService;
    @Resource
    private InvoiceApplySupplierService invoiceApplySupplierService;
    @Resource
    private SysAccountRealityPayableMapper sysAccountRealityPayableMapper;
    @Resource
    private IncUtils incUtils;
    @Resource
    private SysSupplierService sysSupplierService;
    @Resource
    private AccountingUnifiedSupplierExportExcelService accountingUnifiedSupplierExportExcelService;
    @Value("${uniorder.fileBaseUrl:''}")
    private String fileBaseUrl;

    @Override
    public PaymentReportEchoVo getPaymentReportEcho(String accountingOrderId) {
        PaymentReportEchoVo paymentReportEcho = accountingReportPaymentMapper.getPaymentReportEcho(accountingOrderId);
        paymentReportEcho.setAccountingOrderAmountWithFreight(getBigDecimalPoint2(paymentReportEcho.getAccountingOrderAmountWithFreight()));
        paymentReportEcho.setServiceNoName(CommonServiceNoEnum.getByServiceNo(paymentReportEcho.getServiceNo()).getServiceName());
        AccountingReceivablePayable accountingReceivablePayable = new AccountingReceivablePayable();
        accountingReceivablePayable.setAccountingOrderSn(paymentReportEcho.getAccountingOrderSn());
        accountingReceivablePayable.setAccountType(3);
        accountingReceivablePayable.setBillTypeList(Lists.newArrayList(5));
        paymentReportEcho.setPaymentReportStatusName(PaymentStatusEnum.getDesc(paymentReportEcho.getPaymentReportStatus()));
        List<AccountingReceivablePayable> reportAccountings = paidReportAccountingExecutor.findEffectiveAccountings(accountingReceivablePayable);

        paymentReportEcho.setDetails(reportAccountings.stream().map(ReportAccountingAdapter::convert2ReportAccounting).collect(Collectors.toList()));
        return paymentReportEcho;
    }

    @Override
    public R<AddPaymentReportEchoVo> getAddPaymentReportEcho(Long accountingOrderId, LoginSysUser loginSysUser) throws UnifiedOrderRuntimeException {
        AccountingUnifiedSupplier accountingUnifiedSupplier = accountingUnifiedSupplierService.findById(accountingOrderId);

        AddPaymentReportEchoVo vo = getAddPaymentReport(accountingUnifiedSupplier, loginSysUser);
        // 给前端时生成单据流水号
        vo.getActualPayableReportAccounting().getBoeHeader().setBatchNo(incUtils.getBillNo(vo.getServiceNo()));
        ActualAccountingAttachment attachment = accountingUnifiedSupplierExportExcelService.exportAccountingUnifiedSupplierWithGoods(accountingUnifiedSupplier, loginSysUser);
        vo.getActualPayableReportAccounting().setAttachments(Stream.of(attachment).collect(Collectors.toList()));
        return new R<>(vo);
    }

    public AddPaymentReportEchoVo getAddPaymentReport(AccountingUnifiedSupplier accountingUnifiedSupplier, LoginSysUser loginSysUser) {
        // 业务人相关
        SysBusinessUserVo sysBusinessUserVo = sysBusinessUserMapper.selectByBindOperationUserid(loginSysUser.getSysUserId());
        if (null == sysBusinessUserVo) {
            throw new UnifiedOrderRuntimeException(500, "当前操作账号未关联业务人员，请联系管理员关联绑定");
        }
        // 结算中心-系统管理-共享参数配置-福利商城&企业商城-付款数据同步接口
        SysAccountRealityPayableVo sysAccountRealityPayableVo = sysAccountRealityPayableMapper.selectByPrimaryKey(1L);
        if (1 != sysAccountRealityPayableVo.getDataIsComplete()) {
            throw new UnifiedOrderRuntimeException(500, "共享参数配置信息不完整：请前往“系统管理-共享参数配置-完善“付款数据同步接口”相关信息");
        }
        AddPaymentReportEchoVo addPaymentReportEchoVo = new AddPaymentReportEchoVo();
        addPaymentReportEchoVo.setAccountingOrderId(accountingUnifiedSupplier.getId());
        addPaymentReportEchoVo.setAccountingOrderSn(accountingUnifiedSupplier.getAccountingOrderSn());
        UnifiedSupplier unifiedSupplier = sysSupplierService.findBySaleId(accountingUnifiedSupplier.getSaleId());
        addPaymentReportEchoVo.setId(unifiedSupplier.getId());
        addPaymentReportEchoVo.setSaleId(accountingUnifiedSupplier.getSaleId());
        addPaymentReportEchoVo.setSaleName(accountingUnifiedSupplier.getSaleName());
        addPaymentReportEchoVo.setSupplierCode(unifiedSupplier.getSupplierCode());
        addPaymentReportEchoVo.setServiceNo(accountingUnifiedSupplier.getServiceNo());
        String serviceName = CommonServiceNoEnum.getByServiceNo(accountingUnifiedSupplier.getServiceNo()).getServiceName();
        addPaymentReportEchoVo.setServiceNoName(serviceName);
        addPaymentReportEchoVo.setAccountingAmount(getBigDecimalPoint2(accountingUnifiedSupplier.getAccountingOrderCostAmountWithFreight()));
        addPaymentReportEchoVo.setPaidAmount(getBigDecimalPoint2(accountingUnifiedSupplier.getPaymentReportCompletedAmount()));
        addPaymentReportEchoVo.setBillType(5);
        addPaymentReportEchoVo.setTip(TIP_PAYMENT);
        addPaymentReportEchoVo.setActualPayableReportAccounting(createPayableReport(accountingUnifiedSupplier, sysBusinessUserVo, sysAccountRealityPayableVo, unifiedSupplier, serviceName, loginSysUser));
        return addPaymentReportEchoVo;
    }

    private ActualPayableReportAccounting createPayableReport(AccountingUnifiedSupplier accountingUnifiedSupplier, SysBusinessUserVo sysBusinessUserVo, SysAccountRealityPayableVo sysAccountRealityPayableVo, UnifiedSupplier unifiedSupplier, String serviceName, LoginSysUser loginSysUser) {
        ActualPayableReportAccounting actualPayableReportAccounting = new ActualPayableReportAccounting();
        actualPayableReportAccounting.setBizUserId(sysBusinessUserVo.getId());
        actualPayableReportAccounting.setBindOperationUserid(loginSysUser.getSysUserId());
        actualPayableReportAccounting.setBindOperationUser(loginSysUser.getFullName());
        actualPayableReportAccounting.setBoeHeader(createBoeHeader(accountingUnifiedSupplier, sysBusinessUserVo, sysAccountRealityPayableVo, serviceName));
        actualPayableReportAccounting.setActualPayableReportAccountingLineForApportion(createApportion(sysBusinessUserVo, sysAccountRealityPayableVo));
        actualPayableReportAccounting.setActualReceivedReportAccountingLineForZfs(createZfs(accountingUnifiedSupplier, serviceName));
        actualPayableReportAccounting.setBoeHeaderChild(createHeaderChild(sysAccountRealityPayableVo, unifiedSupplier));
        actualPayableReportAccounting.setAttachments(createAttachments());
        return actualPayableReportAccounting;
    }

    private List<ActualAccountingAttachment> createAttachments() {
        // 附件
        ActualAccountingAttachment actualAccountingAttachment = new ActualAccountingAttachment();
        return Stream.of(actualAccountingAttachment).collect(Collectors.toList());
    }

    private ActualPayableReportAccountingHeaderChild createHeaderChild(SysAccountRealityPayableVo sysAccountRealityPayableVo, UnifiedSupplier unifiedSupplier) {
        ActualPayableReportAccountingHeaderChild boeHeaderChild = new ActualPayableReportAccountingHeaderChild();
        boeHeaderChild.setCustomerNum(unifiedSupplier.getSourceId());
        boeHeaderChild.setCustomerName(unifiedSupplier.getSourceName());
        boeHeaderChild.setContractNum("");
        boeHeaderChild.setContractName("");
        boeHeaderChild.setProjectNum(sysAccountRealityPayableVo.getProjectNum());
        boeHeaderChild.setProjectName(sysAccountRealityPayableVo.getProjectName());
        return boeHeaderChild;
    }

    private ActualPayableReportAccountingLineForZfs createZfs(AccountingUnifiedSupplier accountingUnifiedSupplier, String serviceName) {
        ActualPayableReportAccountingLineForZfs actualReceivedReportAccountingLineForZfs = new ActualPayableReportAccountingLineForZfs();
        actualReceivedReportAccountingLineForZfs.setBankAccountNum("");
        actualReceivedReportAccountingLineForZfs.setBankUnitedCode("");
        actualReceivedReportAccountingLineForZfs.setBankAccountName("");
        actualReceivedReportAccountingLineForZfs.setPaymentModeCode(TransferEnum.OCTYHZZ.name());
        actualReceivedReportAccountingLineForZfs.setPaymentMemo(String.format(PAYMENT_MEMO, DateUtil.format(accountingUnifiedSupplier.getStartTime(), FORMAT_DATE), DateUtil.format(accountingUnifiedSupplier.getEndTime(), FORMAT_DATE), accountingUnifiedSupplier.getSaleName(), serviceName));
        actualReceivedReportAccountingLineForZfs.setPaymentAmount(getBigDecimalPoint2(BigDecimal.ZERO));
        return actualReceivedReportAccountingLineForZfs;
    }

    private ActualPayableReportAccountingLineForApportion createApportion(SysBusinessUserVo sysBusinessUserVo, SysAccountRealityPayableVo sysAccountRealityPayableVo) {
        ActualPayableReportAccountingLineForApportion actualPayableReportAccountingLineForApportion = new ActualPayableReportAccountingLineForApportion();
        actualPayableReportAccountingLineForApportion.setExpenseDeptId(sysBusinessUserVo.getDutyDepartmentCode());
        actualPayableReportAccountingLineForApportion.setExpenseDeptName(sysBusinessUserVo.getDutyDepartmentName());
        actualPayableReportAccountingLineForApportion.setCostcenterId(sysAccountRealityPayableVo.getCostcenterId());
        actualPayableReportAccountingLineForApportion.setCostcenterName(sysAccountRealityPayableVo.getCostcenterName());
        actualPayableReportAccountingLineForApportion.setAmount(getBigDecimalPoint2(BigDecimal.ZERO));
        actualPayableReportAccountingLineForApportion.setOperationSubTypeCode(sysAccountRealityPayableVo.getOperationSubTypeCode());
        actualPayableReportAccountingLineForApportion.setServiceType("");
        actualPayableReportAccountingLineForApportion.setFormatType("");
        actualPayableReportAccountingLineForApportion.setSupplierCode("");
        actualPayableReportAccountingLineForApportion.setSupplierName("");
        return actualPayableReportAccountingLineForApportion;
    }

    private ActualPayableReportAccountingHeader createBoeHeader(AccountingUnifiedSupplier accountingUnifiedSupplier, SysBusinessUserVo sysBusinessUserVo, SysAccountRealityPayableVo sysAccountRealityPayableVo, String serviceName) {
        ActualPayableReportAccountingHeader boeHeader = new ActualPayableReportAccountingHeader();

        boeHeader.setDataSource(DATASOURCE);
        boeHeader.setBatchNo("");
        boeHeader.setGroupCode(GROUP_CODE);
        boeHeader.setBoeDate(DateUtil.format(new Date(), "yyyy-MM-dd"));
        boeHeader.setBusinessCategory(sysAccountRealityPayableVo.getBusinessCategory());
        boeHeader.setPaperAccessories("N");
        boeHeader.setOperationTypeCode(sysAccountRealityPayableVo.getOperationTypeCode());
        boeHeader.setEmployeePostId(sysBusinessUserVo.getJobCode());
        boeHeader.setEmployeePostName(sysBusinessUserVo.getJobName());
        boeHeader.setEmployeeId(sysBusinessUserVo.getWorkNumber());
        boeHeader.setEmployeeName(sysBusinessUserVo.getUserName());
        boeHeader.setCompanyNum(sysAccountRealityPayableVo.getCompanyNum());
        boeHeader.setCompanyName(sysAccountRealityPayableVo.getCompanyName());
        boeHeader.setEmployeeDepartmentId(sysBusinessUserVo.getDepartmentCode());
        boeHeader.setEmployeeDepartmentName(sysBusinessUserVo.getDepartmentName());
        boeHeader.setRemark(String.format(REMARK_PAYMENT, DateUtil.format(accountingUnifiedSupplier.getStartTime(), FORMAT_DATE), DateUtil.format(accountingUnifiedSupplier.getEndTime(), FORMAT_DATE), serviceName, accountingUnifiedSupplier.getSaleName(), accountingUnifiedSupplier.getAccountingOrderSn()));
        return boeHeader;
    }

    public ActualPayableReportAccountingDetailVO getPaymentReportDetail(Long id) {

        AccountingReceivablePayable data = accountingReceivablePayableMapper.selectAccountingReceivablePayableById(id);
        ActualPayableReportAccountingDetailVO vo = new ActualPayableReportAccountingDetailVO();

        AccountingPushShareData accountingPushShareData = new AccountingPushShareData();
        accountingPushShareData.setAccountingRpId(data.getId());
        accountingPushShareData.setAccountingOrderSn(data.getAccountingOrderSn());
        AccountingPushShareData result = accountingPushShareDataMapper.selectAccountingPushShareDataList(accountingPushShareData).get(0);
        ActualPayableHeader headData = JSONObject.parseObject(result.getHeadData(), ActualPayableHeader.class);
        ActualPayableReportAccounting actualReportAccounting = new ActualPayableReportAccounting();

        actualReportAccounting.setActualPayableReportAccountingLineForApportion(new ActualPayableReportAccountingLineForApportion());
        actualReportAccounting.setActualReceivedReportAccountingLineForZfs(new ActualPayableReportAccountingLineForZfs());
        actualReportAccounting.setBindOperationUserid(data.getCreateId());
        actualReportAccounting.setBindOperationUser(data.getCreateBy());

        actualReportAccounting.setBoeHeader(headData.getBoeHeader());
        actualReportAccounting.setBoeHeaderChild(headData.getBoeHeaderChild());
        JSONArray lines = JSONObject.parseArray(result.getRowData());
        for (int i = 0; i < lines.size(); i++) {
            JSONObject line = lines.getJSONObject(i);
            switch ((String) line.get("lineType")) {
                case "apportion":
                    actualReportAccounting.setActualPayableReportAccountingLineForApportion(line.toJavaObject(ActualPayableReportAccountingLineForApportion.class));
                    break;
                case "zfsBoePayments":
                    actualReportAccounting.setActualReceivedReportAccountingLineForZfs(line.toJavaObject(ActualPayableReportAccountingLineForZfs.class));
                    break;
                default:
                    throw new IllegalArgumentException("行类型错误");
            }
        }
        actualReportAccounting.setAttachments(JSONObject.parseArray(result.getFileData(), ActualAccountingAttachment.class));
        vo.setActualPayableReportAccounting(actualReportAccounting);
        vo.setReportAccountingDetailVO(ReportAccountingAdapter.convert2ReportAccounting(data));
        return vo;
    }

    @Override
    public R<?> confirmPaymentReportList(ActualReportAccountingConfirmParam param, LoginSysUser loginSysUser) {
        AccountingReceivablePayable accountingReceivablePayableParam = new AccountingReceivablePayable();
        accountingReceivablePayableParam.setAccountingOrderSn(param.getAccountingOrderSn());
        accountingReceivablePayableParam.setAccountType(3);
        accountingReceivablePayableParam.setBillTypeList(Lists.newArrayList(CommonReportAccountingTypeEnum.FIVE.getKey()));
        accountingReceivablePayableParam.setAccountingState(2);
        Map<String, AccountingReceivablePayable> resultMap = accountingReceivablePayableMapper.selectAccountingReceivablePayableList(accountingReceivablePayableParam)
                .stream().collect(Collectors.toMap(result -> result.getBillNo().trim(), Function.identity()));
        BoeStatusResult boeStatusResult;

        List<BoeStatusParam> params = resultMap.values().stream().map(result -> {
            BoeStatusParam boeStatusParam = new BoeStatusParam();
            boeStatusParam.setDataSource(DATASOURCE);
            boeStatusParam.setGroupCode(GROUP_CODE);
            boeStatusParam.setBatchNo(result.getBillNo());
            boeStatusParam.setBoeNo("");
            return boeStatusParam;
        }).collect(Collectors.toList());
        List<List<BoeStatusParam>> partitions = Lists.partition(params, 10);
        for (List<BoeStatusParam> partition : partitions) {
            try {
                boeStatusResult = shareCenterService.queryBoeStatus(partition);
                if (0 != boeStatusResult.getCode()) {
                    continue;
                }
            } catch (Exception e) {
                log.error("数据审批状态查询接口：调用失败无响应或超时", e);
                return ResponseResultOuterUtils.createErrorResult("500", "数据审批状态查询接口：调用失败无响应或超时");
            }
            paidReportAccountingExecutor.confirmActualPaidAccounting(param, loginSysUser, resultMap, boeStatusResult);
        }
        return ResponseResultOuterUtils.createSuccesResult();
    }

    @Override
    public R<?> addPaymentReport(ActualPayableReportAccountingCreateParam param, LoginSysUser loginSysUser) {
        AccountingUnifiedSupplier accountingUnifiedSupplier = accountingUnifiedSupplierService.findById(param.getAccountingOrderId());
        Integer accountingStatus = accountingUnifiedSupplier.getAccountingStatus();
        if (!Objects.equals(AccountingOrderCustomerAccountingStatusEnum.已确认.getKey(), accountingStatus)) {
            return ResponseResultOuterUtils.createErrorResult("500", "该报账单所属的对账单状态发生变更，不允许推送，请返回检查");
        }
        List<InvoiceApplySupplier> invoiceApplies = invoiceApplySupplierService.findByAccountingOrderSn(param.getAccountingOrderSn());
        List<Integer> backAccountStatus = invoiceApplies.stream().filter(AnLamdaUtils.distinctByKey(InvoiceApplySupplier::getInvoiceStatus)).map(InvoiceApplySupplier::getInvoiceStatus).collect(Collectors.toList());
        if (CollUtil.isEmpty(backAccountStatus) || backAccountStatus.size() > 1 || !Objects.equals(backAccountStatus.get(0), InvoiceApplySupplierInvoiceProgressingStatusEnum.待付款.getKey())) {
            return ResponseResultOuterUtils.createErrorResult("500", "该对账单部分开票申请的申请状态“非待付款”，不允许发起，请返回检查");
        }
        if (Objects.equals(accountingUnifiedSupplier.getPaymentReportStatus(), 2)) {
            return ResponseResultOuterUtils.createErrorResult("500", "该对账单的付款报账状态已全部回款，无需再报账，请返回检查");
        }

        BigDecimal approvePaymentAmount = accountingReceivablePayableMapper.sumApprovePaymentAmount(param.getAccountingOrderSn(), param.getBillType());
        if (AnBigDecimalUtils.getBigDecimal(approvePaymentAmount).add(param.getPaymentAmount()).compareTo(accountingUnifiedSupplier.getAccountingOrderCostAmountWithFreight()) > 0) {
            return ResponseResultOuterUtils.createErrorResult("500", "支付金额不能大于该报账单的实付账款总金额，请重新输入");
        }

        AddPaymentReportEchoVo addPaymentReportEcho = getAddPaymentReport(accountingUnifiedSupplier, loginSysUser);

        ActualPayableReportAccounting actualPayableReportAccounting = addPaymentReportEcho.getActualPayableReportAccounting();
        actualPayableReportAccounting.getBoeHeader().setBatchNo(param.getBillNo());
        actualPayableReportAccounting.getBoeHeader().setRemark(param.getRemark());

        actualPayableReportAccounting.getActualReceivedReportAccountingLineForZfs().setPaymentMemo(param.getPaymentMemo());
        actualPayableReportAccounting.getActualReceivedReportAccountingLineForZfs().setBankAccountNum(param.getBankAccountNum());
        actualPayableReportAccounting.getActualReceivedReportAccountingLineForZfs().setBankAccountName(param.getBankAccountName());
        actualPayableReportAccounting.getActualReceivedReportAccountingLineForZfs().setBankUnitedCode(param.getBankUnitedCode());
        actualPayableReportAccounting.getActualReceivedReportAccountingLineForZfs().setPaymentAmount(param.getPaymentAmount());

        actualPayableReportAccounting.getActualPayableReportAccountingLineForApportion().setAmount(param.getPaymentAmount());

        actualPayableReportAccounting.getBoeHeaderChild().setContractNum(param.getContractNum());
        actualPayableReportAccounting.getBoeHeaderChild().setContractName(param.getContractName());
        actualPayableReportAccounting.getBoeHeaderChild().setCustomerNum(param.getCustomerNum());
        actualPayableReportAccounting.getBoeHeaderChild().setCustomerName(param.getCustomerName());

        param.getAttachments().forEach(actualAccountingAttachment -> {
            actualAccountingAttachment.setFileUrl(fileBaseUrl + actualAccountingAttachment.getFileUrl());
        });
        actualPayableReportAccounting.setAttachments(param.getAttachments());

        AccountingReceivablePayable accountingReceivablePayable = accountingReceivablePayableMapper.selectAccountingReceivablePayableByBillNo(param.getBillNo());
        if (null != accountingReceivablePayable) {
            if (accountingReceivablePayable.getAccountingState() > CommonReportAccountingStateEnum.ONE.getKey()) {
                return ResponseResultOuterUtils.createErrorResult("500", "该报账单已提交，请勿重复提交，请返回查看");
            }
        } else {
            try {
                // 若报账单不存在，先报账单落库
                accountingReceivablePayable = paidReportAccountingExecutor.create(addPaymentReportEcho, loginSysUser);
            } catch (DuplicateKeyException e) {
                return ResponseResultOuterUtils.createErrorResult("500", "该报账单已提交，请勿重复提交，请返回查看");
            }
        }

        ActualPayableAccountingVO actualPayableAccountingVO;
        try {
            actualPayableAccountingVO = shareCenterService.pushBoePaymentInfo(Stream.of(actualPayableReportAccounting).collect(Collectors.toList()));
        } catch (Exception e) {
            return ResponseResultOuterUtils.createErrorResult("500", "共享中心数据同步接口：调用失败无响应");
        }
        if (0 != actualPayableAccountingVO.getCode() || CollectionUtil.isEmpty(actualPayableAccountingVO.getBoeDatas())) {
            return ResponseResultOuterUtils.createErrorResult("500", "推送失败：" + actualPayableAccountingVO.getMsg());
        }
        BoeData boeData = actualPayableAccountingVO.getBoeDatas().get(0);
        if (0 != boeData.getStatus()) {
            return ResponseResultOuterUtils.createErrorResult("500", "推送失败：" + boeData.getMsg());
        }
        paidReportAccountingExecutor.saveActualReceivedAccounting(actualPayableReportAccounting, loginSysUser, accountingReceivablePayable);
        return ResponseResultOuterUtils.createSuccesResult();
    }

    @Override
    @Transactional
    public void billPaymentStatusRepair(InvoiceStatusRepair repair, LoginSysUser loginSysUser) {
        AccountingUnifiedSupplier accountingUnifiedSupplier = accountingUnifiedSupplierService.findById(repair.getAccountingOrderId());
        paidReportAccountingExecutor.billPaymentStatusRepair(accountingUnifiedSupplier, loginSysUser);
    }

}
