package cn.superdesk.uniorder.service.invoice.supplier.impl;

import cn.hutool.core.collection.CollUtil;
import cn.superdesk.uniorder.common.enums.common.CommonReportAccountingStateEnum;
import cn.superdesk.uniorder.common.enums.invoice.supplier.InvoiceApplySupplierInvoiceProgressingStatusEnum;
import cn.superdesk.uniorder.common.lamda.AnLamdaUtils;
import cn.superdesk.uniorder.common.numerical.AnBigDecimalUtils;
import cn.superdesk.uniorder.common.security.dto.LoginSysUser;
import cn.superdesk.uniorder.common.utils.DateUtils;
import cn.superdesk.uniorder.dao.entity.accounting.unified.supplier.AccountingUnifiedSupplier;
import cn.superdesk.uniorder.dao.entity.accounting.unified.supplier.AccountingUnifiedSupplierXGoods;
import cn.superdesk.uniorder.dao.entity.base.UnifiedOrderGoods;
import cn.superdesk.uniorder.dao.entity.biz.AccountingReceivablePayable;
import cn.superdesk.uniorder.dao.entity.biz.FlowPayableDetail;
import cn.superdesk.uniorder.dao.entity.biz.FlowPayableSummary;
import cn.superdesk.uniorder.dao.entity.invoice.supplier.InvoiceApplySupplier;
import cn.superdesk.uniorder.dao.mapper.accounting.unified.supplier.AccountingUnifiedSupplierMapper;
import cn.superdesk.uniorder.dao.mapper.base.UnifiedOrderGoodsMapper;
import cn.superdesk.uniorder.dao.mapper.biz.AccountingPushShareDataMapper;
import cn.superdesk.uniorder.dao.mapper.biz.AccountingReceivablePayableMapper;
import cn.superdesk.uniorder.dao.mapper.biz.FlowPayableDetailMapper;
import cn.superdesk.uniorder.dao.mapper.biz.FlowPayableSummaryMapper;
import cn.superdesk.uniorder.dto.param.invoice.customer.reportaccounting.ActualReportAccountingConfirmParam;
import cn.superdesk.uniorder.dto.param.invoice.customer.reportaccounting.InvoiceStatusRepair;
import cn.superdesk.uniorder.dto.param.invoice.supplier.status.InvoiceApplySupplierOpCommonDimAccountingOrderForm;
import cn.superdesk.uniorder.dto.param.invoice.supplier.status.InvoiceApplySupplierOpCommonDimAccountingOrderItem;
import cn.superdesk.uniorder.dto.param.sharecenter.ActualAccountingAttachment;
import cn.superdesk.uniorder.dto.param.sharecenter.ActualPayableReportAccounting;
import cn.superdesk.uniorder.dto.vo.invoice.supplier.paymentReport.AddPaymentReportEchoVo;
import cn.superdesk.uniorder.dto.vo.sharecenter.BoeStatusResult;
import cn.superdesk.uniorder.dto.vo.sharecenter.BoeStatusVO;
import cn.superdesk.uniorder.service.accounting.unified.supplier.AccountingUnifiedSupplierService;
import cn.superdesk.uniorder.service.accounting.unified.supplier.AccountingUnifiedSupplierXGoodsService;
import cn.superdesk.uniorder.service.invoice.common.reportaccounting.ReportAccountingAdapter;
import cn.superdesk.uniorder.service.invoice.supplier.InvoiceApplySupplierFieldPayoffAccountStatusService;
import cn.superdesk.uniorder.service.invoice.supplier.InvoiceApplySupplierService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.transaction.Transactional;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 报账单执行器
 */
@Service
@Slf4j
public class PaidReportAccountingExecutor {
    @Resource
    private AccountingUnifiedSupplierService accountingUnifiedSupplierService;
    @Resource
    private AccountingUnifiedSupplierMapper accountingUnifiedSupplierMapper;
    @Resource
    private AccountingReceivablePayableMapper accountingReceivablePayableMapper;
    @Resource
    private FlowPayableSummaryMapper flowPayableSummaryMapper;
    @Resource
    private AccountingPushShareDataMapper accountingPushShareDataMapper;
    @Resource
    private InvoiceApplySupplierService invoiceApplySupplierService;
    @Resource
    private AccountingUnifiedSupplierXGoodsService accountingUnifiedSupplierXGoodsService;
    @Resource
    private FlowPayableDetailMapper flowPayableDetailMapper;
    @Resource
    private UnifiedOrderGoodsMapper unifiedOrderGoodsMapper;
    @Resource
    private InvoiceApplySupplierFieldPayoffAccountStatusService invoiceApplySupplierFieldPayoffAccountStatusService;

    public List<AccountingReceivablePayable> findEffectiveAccountings(AccountingReceivablePayable accountingReceivablePayable) {
        List<AccountingReceivablePayable> reportAccountings = accountingReceivablePayableMapper.selectAccountingReceivablePayableList(accountingReceivablePayable);
        return reportAccountings.stream().filter(report -> report.getAccountingState() > CommonReportAccountingStateEnum.ONE.getKey()).collect(Collectors.toList());
    }

    @Transactional
    public AccountingReceivablePayable create(AddPaymentReportEchoVo addPaymentReportEcho, LoginSysUser loginSysUser) {
        AccountingReceivablePayable accountingReceivablePayable = new AccountingReceivablePayable();
        ActualPayableReportAccounting actualPayableReportAccounting = addPaymentReportEcho.getActualPayableReportAccounting();
        accountingReceivablePayable.setAccountingOrderSn(addPaymentReportEcho.getAccountingOrderSn());
        accountingReceivablePayable.setAccountType(3);
        accountingReceivablePayable.setBillNo(actualPayableReportAccounting.getBoeHeader().getBatchNo());
        accountingReceivablePayable.setBillCode("");
        accountingReceivablePayable.setBillType(addPaymentReportEcho.getBillType());
        accountingReceivablePayable.setDataType(0);
        accountingReceivablePayable.setBillAmount(actualPayableReportAccounting.getActualPayableReportAccountingLineForApportion().getAmount());
        accountingReceivablePayable.setAccountingState(1);
        accountingReceivablePayable.setShareBillNo(null);
        accountingReceivablePayable.setShareBillCode(null);
        accountingReceivablePayable.setShareDataState(null);
        accountingReceivablePayable.setShareApproveState(null);
        accountingReceivablePayable.setSharePayState(null);
        accountingReceivablePayable.setApproveDate(null);
        accountingReceivablePayable.setFromSettlementId(addPaymentReportEcho.getId());
        accountingReceivablePayable.setSettlementId(addPaymentReportEcho.getSaleId());
        accountingReceivablePayable.setSettlementCode(addPaymentReportEcho.getSaleId().toString());
        accountingReceivablePayable.setSettlementName(addPaymentReportEcho.getSaleName());
        accountingReceivablePayable.setBizPlatformId(String.valueOf(addPaymentReportEcho.getServiceNo()));
        accountingReceivablePayable.setBizPlatformName(addPaymentReportEcho.getServiceNoName());
        accountingReceivablePayable.setBizUserId(accountingReceivablePayable.getBizUserId());
        accountingReceivablePayable.setBizUserName(accountingReceivablePayable.getBizUserName());
        accountingReceivablePayable.setPusContent(null);
        if (CollUtil.isNotEmpty(addPaymentReportEcho.getActualPayableReportAccounting().getAttachments())) {
            ActualAccountingAttachment attachment = addPaymentReportEcho.getActualPayableReportAccounting().getAttachments().get(0);
            accountingReceivablePayable.setFileTitle(attachment.getFileName());
            accountingReceivablePayable.setFileDetailLink(attachment.getFileUrl());
        }

        accountingReceivablePayable.setDelFlag("0");
        accountingReceivablePayable.setCommitBy(loginSysUser.getFullName());
        accountingReceivablePayable.setCommitId(loginSysUser.getSysUserId());
        accountingReceivablePayable.setCommitTime(new Date());
        accountingReceivablePayable.setRemark("");
        accountingReceivablePayable.setCreateBy(loginSysUser.getFullName());
        accountingReceivablePayable.setCreateId(loginSysUser.getSysUserId());
        accountingReceivablePayable.setCreateTime(new Date());
        accountingReceivablePayable.setUpdateBy(loginSysUser.getFullName());
        accountingReceivablePayable.setUpdateId(loginSysUser.getSysUserId());
        accountingReceivablePayable.setUpdateTime(new Date());

        accountingReceivablePayableMapper.insertAccountingReceivablePayable(accountingReceivablePayable);
        return accountingReceivablePayable;
    }

    @Transactional
    public void confirmActualPaidAccounting(ActualReportAccountingConfirmParam param, LoginSysUser loginSysUser, Map<String, AccountingReceivablePayable> resultMap, BoeStatusResult boeStatusResult) {

        List<BoeStatusVO> boeStatusVOS = boeStatusResult.getBoeStatusVOS();
        BigDecimal paidAccountAmount = BigDecimal.ZERO;
        for (BoeStatusVO boeStatusVO : boeStatusVOS) {
            if (StringUtils.isBlank(boeStatusVO.getDataStatus())) {
                continue;
            }
            AccountingReceivablePayable accountingReceivablePayable = resultMap.get(boeStatusVO.getBatchNo().trim());
            accountingReceivablePayable.setUpdateId(loginSysUser.getSysUserId());
            accountingReceivablePayable.setUpdateBy(loginSysUser.getFullName());
            accountingReceivablePayable.setUpdateTime(new Date());
            accountingReceivablePayable.setShareDataState(boeStatusVO.getDataStatus());
            if ("E".equals(boeStatusVO.getDataStatus())) {
                accountingReceivablePayable.setAccountingState(CommonReportAccountingStateEnum.FOUR.getKey());
                accountingReceivablePayable.setPusContent(boeStatusVO.getErrorMessage());
            } else if ("S".equals(boeStatusVO.getDataStatus())) {
                accountingReceivablePayable.setBillCode(boeStatusVO.getBoeNo());
                accountingReceivablePayable.setShareApproveState(boeStatusVO.getBoeStatus());
                accountingReceivablePayable.setSharePayState(boeStatusVO.getBoePayStatus());
                if ("30".equals(boeStatusVO.getBoeStatus())) {
                    if ("40".equals(boeStatusVO.getBoePayStatus())) {
                        accountingReceivablePayable.setApproveDate(StringUtils.isNotBlank(boeStatusVO.getEndTime()) ? DateUtils.parseDate(boeStatusVO.getEndTime()) : null);
                        accountingReceivablePayable.setAccountingState(CommonReportAccountingStateEnum.FIVE.getKey());
                    } else if ("30".equals(boeStatusVO.getBoePayStatus())) {
                        accountingReceivablePayable.setApproveDate(StringUtils.isNotBlank(boeStatusVO.getEndTime()) ? DateUtils.parseDate(boeStatusVO.getEndTime()) : null);
                        accountingReceivablePayable.setAccountingState(CommonReportAccountingStateEnum.THREE.getKey());
                        processFlowPaidSummary(accountingReceivablePayable, loginSysUser);
                        //
                        paidAccountAmount = paidAccountAmount.add(accountingReceivablePayable.getBillAmount());
                    }
                } else if ("0".equals(boeStatusVO.getBoeStatus())) {
                    accountingReceivablePayable.setApproveDate(StringUtils.isNotBlank(boeStatusVO.getEndTime()) ? DateUtils.parseDate(boeStatusVO.getEndTime()) : null);
                    accountingReceivablePayable.setAccountingState(CommonReportAccountingStateEnum.FIVE.getKey());
                }
                // 其它场景仍保持审批中状态
            }
            // 其它状态只更新更新人及数据状态
            accountingReceivablePayableMapper.updateAccountingReceivablePayable(accountingReceivablePayable);
        }
        // 锁读
        AccountingUnifiedSupplier accountingUnifiedSupplier;
        if (null == param.getAccountingOrderId()) {
            accountingUnifiedSupplier = accountingUnifiedSupplierMapper.selectBySnForUpdate(param.getAccountingOrderSn());
        } else {
            accountingUnifiedSupplier = accountingUnifiedSupplierMapper.selectByPrimaryKeyForUpdate(param.getAccountingOrderId());
        }
        if (paidAccountAmount.signum() > 0) {
            BigDecimal amount = AnBigDecimalUtils.getBigDecimal(accountingUnifiedSupplier.getPaymentReportCompletedAmount()).add(paidAccountAmount);
            accountingUnifiedSupplier.setPaymentReportCompletedAmount(amount);
            if (Objects.equals(AnBigDecimalUtils.getBigDecimal(accountingUnifiedSupplier.getAccountingOrderCostAmountWithFreight()), amount)) {
                accountingUnifiedSupplier.setPaymentReportStatus(2);
            } else {
                accountingUnifiedSupplier.setPaymentReportStatus(1);
            }
        }
        accountingUnifiedSupplierService.updateByPrimaryKeySelective(accountingUnifiedSupplier);

        List<InvoiceApplySupplier> invoiceApplies = invoiceApplySupplierService.findByAccountingOrderSn(param.getAccountingOrderSn());
        if (isNotCanPay(invoiceApplies)) {
            return;
        }
        InvoiceApplySupplierOpCommonDimAccountingOrderForm form = new InvoiceApplySupplierOpCommonDimAccountingOrderForm();
        form.setAccountingOrderId(accountingUnifiedSupplier.getId());
        form.setAccountingOrderSn(accountingUnifiedSupplier.getAccountingOrderSn());
        form.setOutSaleId(accountingUnifiedSupplier.getSaleId());
        form.setOutSaleName(accountingUnifiedSupplier.getSaleName());
        form.setInvoiceApplySupplierList(invoiceApplies.stream().map(invoiceApply -> {
            InvoiceApplySupplierOpCommonDimAccountingOrderItem item = new InvoiceApplySupplierOpCommonDimAccountingOrderItem();
            item.setInvoiceApplyId(invoiceApply.getId());
            item.setInvoiceApplySn(invoiceApply.getInvoiceApplySn());
            return item;
        }).collect(Collectors.toList()));

        // 调发起付款
        invoiceApplySupplierFieldPayoffAccountStatusService.opRegulatoryPipeline(invoiceApplies, form, loginSysUser);

        if ((Objects.equals(2, accountingUnifiedSupplier.getPaymentReportStatus()))) {
            // 调确认状态修复
            InvoiceStatusRepair repair = new InvoiceStatusRepair();
            repair.setAccountingOrderId(accountingUnifiedSupplier.getId());
            repair.setAccountingOrderSn(accountingUnifiedSupplier.getAccountingOrderSn());
            billPaymentStatusRepair(accountingUnifiedSupplier, loginSysUser);
        }
    }

    private void processFlowPaidSummary(AccountingReceivablePayable accountingReceivablePayable, LoginSysUser loginSysUser) {
        FlowPayableSummary summary = new FlowPayableSummary();
        Date curDate = new Date();
        summary.setShareBillNo(accountingReceivablePayable.getBillNo());
        summary.setType(1);
        summary.setShareBillCode(accountingReceivablePayable.getBillCode());
        summary.setRmAccountState(2);
        summary.setRmAccountUserId(loginSysUser.getSysUserId());
        summary.setRmAccountUserName(loginSysUser.getFullName());
        summary.setShareApproveDate(accountingReceivablePayable.getApproveDate());
        summary.setInAccountId("");
        summary.setInAccountDate(curDate);
        summary.setInAccountType(3);
        summary.setInAccountWay(0);
        summary.setAccountingOrderSn(accountingReceivablePayable.getAccountingOrderSn());
        summary.setCusSettlementId(accountingReceivablePayable.getSettlementId());
        summary.setCusBizCode(accountingReceivablePayable.getSettlementCode());
        summary.setCusBizName(accountingReceivablePayable.getSettlementName());
        summary.setBizPlatformId(accountingReceivablePayable.getBizPlatformId());
        summary.setBizPlatformName(accountingReceivablePayable.getBizPlatformName());
        summary.setOperationTaxRate(null);
        summary.setCostNoTax(null);
        summary.setTaxAmount(null);
        summary.setPayableAmount(accountingReceivablePayable.getBillAmount());
        summary.setDelFlag("0");
        summary.setRemark(null);
        summary.setCreateId(loginSysUser.getSysUserId());
        summary.setCreateBy(loginSysUser.getFullName());
        summary.setCreateTime(curDate);
        summary.setUpdateId(loginSysUser.getSysUserId());
        summary.setUpdateBy(loginSysUser.getFullName());
        summary.setUpdateTime(curDate);
        flowPayableSummaryMapper.insertFlowPayableSummary(summary);
    }

    @Transactional
    public void saveActualReceivedAccounting(ActualPayableReportAccounting actualPayableReportAccounting, LoginSysUser loginSysUser, AccountingReceivablePayable accountingReceivablePayable) {
        // 更新状态
        accountingReceivablePayableMapper.changeStatus(accountingReceivablePayable.getId(), CommonReportAccountingStateEnum.TWO.getKey(), CommonReportAccountingStateEnum.ONE.getKey());
        accountingPushShareDataMapper.insertAccountingPushShareData(ReportAccountingAdapter.createAccountingPushShareData(accountingReceivablePayable, actualPayableReportAccounting, loginSysUser));
    }

    public void billPaymentStatusRepair(AccountingUnifiedSupplier accountingUnifiedSupplier, LoginSysUser loginSysUser) {
        List<InvoiceApplySupplier> invoiceApplies = invoiceApplySupplierService.findByAccountingOrderSn(accountingUnifiedSupplier.getAccountingOrderSn());
        if (isNotCanPay(invoiceApplies)) {
            return;
        }
        // 调确认付款
        InvoiceApplySupplierOpCommonDimAccountingOrderForm form = new InvoiceApplySupplierOpCommonDimAccountingOrderForm();
        form.setAccountingOrderId(accountingUnifiedSupplier.getId());
        form.setAccountingOrderSn(accountingUnifiedSupplier.getAccountingOrderSn());
        form.setOutSaleId(accountingUnifiedSupplier.getSaleId());
        form.setOutSaleName(accountingUnifiedSupplier.getSaleName());
        form.setInvoiceApplySupplierList(invoiceApplies.stream().map(invoiceApply -> {
            InvoiceApplySupplierOpCommonDimAccountingOrderItem item = new InvoiceApplySupplierOpCommonDimAccountingOrderItem();
            item.setInvoiceApplyId(invoiceApply.getId());
            item.setInvoiceApplySn(invoiceApply.getInvoiceApplySn());
            return item;
        }).collect(Collectors.toList()));
        invoiceApplySupplierFieldPayoffAccountStatusService.opComplete(invoiceApplies, form, loginSysUser);

        List<AccountingUnifiedSupplierXGoods> supplierXGoods = accountingUnifiedSupplierXGoodsService.findByAccountingOrderSn(accountingUnifiedSupplier.getAccountingOrderSn());
        if (CollUtil.isNotEmpty(supplierXGoods)) {
            Map<Long, FlowPayableDetail> goodFlowPayableDetailMap = new HashMap<>();
            supplierXGoods.forEach(good -> {
                FlowPayableDetail flowPayableDetail = new FlowPayableDetail();
                flowPayableDetail.setOrderSn(good.getOrderSn());
                List<FlowPayableDetail> flowPayableDetails = flowPayableDetailMapper.selectFlowPayableDetailList(flowPayableDetail);
                Date date = new Date();
                flowPayableDetails.forEach(detail -> {
                    // 非未冲销的不处理
                    if (detail.getChargeStatus() != 1) {
                        return;
                    }
                    detail.setChargeStatus(2);
                    detail.setUpdateBy(loginSysUser.getFullName());
                    detail.setUpdateId(loginSysUser.getSysUserId());
                    detail.setUpdateTime(date);
                    detail.setCollectionOrderSn(accountingUnifiedSupplier.getAccountingOrderSn());
                    detail.setChargeUserId(loginSysUser.getSysUserId());
                    detail.setChargeUserName(loginSysUser.getFullName());
                    detail.setChargeCreatedAt(date);
                    detail.setChargeOrderGoodsId(good.getGoodsId());
                    goodFlowPayableDetailMap.put(detail.getGoodsId(), detail);
                    flowPayableDetailMapper.updateFlowPayableDetail(detail);
                });
            });
            if (CollUtil.isNotEmpty(goodFlowPayableDetailMap)) {
                List<UnifiedOrderGoods> goods = unifiedOrderGoodsMapper.findGoodsByIds(goodFlowPayableDetailMap.keySet());
                goods.forEach(good -> {
                    FlowPayableDetail detail = goodFlowPayableDetailMap.get(good.getId());
                    BigDecimal totalAmount = good.getPrice().multiply(BigDecimal.valueOf(good.getQuantity()));
                    if (totalAmount.abs().subtract(detail.getPayableAmount().abs()).signum() > 0) {
                        good.setPayableChargeState(2);
                    } else {
                        good.setPayableChargeState(3);
                    }
                    unifiedOrderGoodsMapper.updateByPrimaryKeySelective(good);
                });
            }
        }
    }

    /**
     * 判断是否不允许付款；发票申请非全部待付款时不允许
     */
    public boolean isNotCanPay(List<InvoiceApplySupplier> invoiceApplies) {
        List<Integer> backAccountStatus = invoiceApplies.stream().filter(AnLamdaUtils.distinctByKey(InvoiceApplySupplier::getInvoiceStatus)).map(InvoiceApplySupplier::getInvoiceStatus).collect(Collectors.toList());
        return CollUtil.isEmpty(backAccountStatus) || backAccountStatus.size() > 1 || !Objects.equals(backAccountStatus.get(0), InvoiceApplySupplierInvoiceProgressingStatusEnum.待付款.getKey());
    }
}
