package cn.superdesk.uniorder.service.sharecenter.impl;

import cn.superdesk.uniorder.common.api.APIUtil;
import cn.superdesk.uniorder.common.security.DESPlus;
import cn.superdesk.uniorder.dto.param.sharecenter.ActualAccountingAttachment;
import cn.superdesk.uniorder.dto.param.sharecenter.ActualPayableReportAccounting;
import cn.superdesk.uniorder.dto.param.sharecenter.ActualReceivedReportAccounting;
import cn.superdesk.uniorder.dto.param.sharecenter.BoeStatusParam;
import cn.superdesk.uniorder.dto.vo.sharecenter.*;
import cn.superdesk.uniorder.service.sharecenter.ShareCenterService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@Service
public class ShareCenterServiceImpl implements ShareCenterService {

    @Value("${uniorder.synNewCollectionFundBoe:1}")
    private String synNewCollectionFundBoe;

    @Value("${uniorder.pushBoePaymentInfo:1}")
    private String pushBoePaymentInfo;

    @Value("${uniorder.queryBoeStatus:1}")
    private String queryBoeStatus;

    /**
     * 业务系统推送实收数据至财务共享系统
     * @param actualReportAccounting 请求数据
     * @return 发送结果
     */
    @Override
    public ActualReceivedAccountingVO synNewCollectionFundBoe(ActualReceivedReportAccounting actualReportAccounting) throws Exception {

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("header", actualReportAccounting.getHeader());

        JSONArray lines = new JSONArray();
        lines.add(actualReportAccounting.getActualReceivedReportAccountingLineForZfs());
        lines.add(actualReportAccounting.getActualReceivedReportAccountingLineForIncome());
        if (null != actualReportAccounting.getActualReceivedReportAccountingLineForCost()) {
            lines.add(actualReportAccounting.getActualReceivedReportAccountingLineForCost());
        }
        jsonObject.put("lines", lines);
        jsonObject.put("fileLines", actualReportAccounting.getAttachments());
        try {
            log.info(jsonObject.toJSONString());
            String result = APIUtil.post(synNewCollectionFundBoe, jsonObject.toJSONString());
            log.info("synNewCollectionFundBoe result is {}", result);
            return JSONObject.parseObject(result, ActualReceivedAccountingVO.class);
        } catch (Exception e) {
            log.error("synNewCollectionFundBoe error", e);
            throw e;
        }
    }

    @Override
    public ActualPayableAccountingVO pushBoePaymentInfo(List<ActualPayableReportAccounting> actualPayableReportAccountings) throws Exception {
        JSONArray jsonArray = new JSONArray();
        jsonArray.addAll(actualPayableReportAccountings.stream().map(actualPayableReportAccounting -> {

            JSONObject jsonObject = new JSONObject();
            jsonObject.put("boeHeader", actualPayableReportAccounting.getBoeHeader());

            JSONArray lines = new JSONArray();
            lines.add(actualPayableReportAccounting.getActualReceivedReportAccountingLineForZfs());
            lines.add(actualPayableReportAccounting.getActualPayableReportAccountingLineForApportion());
            jsonObject.put("boeLines", lines);
            jsonObject.put("boeHeaderChild", actualPayableReportAccounting.getBoeHeaderChild());

            List<ActualAccountingAttachment> attachmentList = actualPayableReportAccounting.getAttachments();
            if(CollectionUtils.isNotEmpty(attachmentList)){
                for (ActualAccountingAttachment attachment : attachmentList) {
                    String fileUrl = attachment.getFileUrl();
                    //文件加密
                    if(StringUtils.isNotBlank(fileUrl)){
                        attachment.setFileUrl(DESPlus.encode(fileUrl));
                    }
                }
            }

            JSONArray attachments = new JSONArray();
            attachments.addAll(attachmentList);
            jsonObject.put("attachment", attachments);
            return jsonObject;
        }).collect(Collectors.toList()));
        try {
            log.info(jsonArray.toJSONString());
            String result = APIUtil.post(pushBoePaymentInfo, jsonArray.toJSONString());
            log.info("pushBoePaymentInfo result is {}", result);
            JSONObject resultJson = JSONObject.parseObject(result);
            ActualPayableAccountingVO actualPayableAccountingVO = new ActualPayableAccountingVO();
            Integer code = resultJson.getInteger("code");
            actualPayableAccountingVO.setCode(code);
            actualPayableAccountingVO.setType(resultJson.getString("type"));
            actualPayableAccountingVO.setMsg(resultJson.getString("msg"));
            if (0 == code) {
                actualPayableAccountingVO.setBoeDatas(resultJson.getJSONArray("data").toJavaList(BoeData.class));
            }
            return actualPayableAccountingVO;
        } catch (Exception e) {
            log.error("pushBoePaymentInfo error", e);
            throw e;
        }
    }

    /**
     * 数据审批状态同步
     *
     * @param boeStatusParams 请求数据
     * @return 发送结果
     */
    @Override
    public BoeStatusResult queryBoeStatus(List<BoeStatusParam> boeStatusParams) throws Exception {

        String apiParam = JSON.toJSONString(boeStatusParams);
        try {
            log.info(apiParam);
            String result = APIUtil.post(queryBoeStatus, apiParam);
            log.info("synNewCollectionFundBoe result is {}", result);
            JSONObject resultJson = JSONObject.parseObject(result);

            BoeStatusResult boeStatusResult = new BoeStatusResult();
            Integer code = resultJson.getInteger("code");
            boeStatusResult.setCode(code);
            boeStatusResult.setType(resultJson.getString("type"));
            boeStatusResult.setMsg(resultJson.getString("msg"));
            if (0 == code) {
                JSONArray datas = resultJson.getJSONArray("data");

                List<BoeStatusVO> vos = new ArrayList<>(boeStatusParams.size());
                for (int i = 0; i < datas.size(); i++) {
                    JSONObject data = datas.getJSONObject(i);
                    BoeStatusVO invoiceStatusResponse = new BoeStatusVO();
                    invoiceStatusResponse.setDataSource((String) data.get("dataSource"));
                    invoiceStatusResponse.setBatchNo(StringUtils.trim((String) data.get("batchNo")));
                    invoiceStatusResponse.setBoeNo((String) data.get("boeNo"));
                    invoiceStatusResponse.setDataStatus((String) data.get("DATA_STATUS"));
                    invoiceStatusResponse.setBoeStatus((String) data.get("BOE_STATUS"));
                    invoiceStatusResponse.setBoePayStatus((String) data.get("BOE_PAY_STATUS"));
                    invoiceStatusResponse.setBoePayTime((String) data.get("BOE_PAY_TIME"));
                    invoiceStatusResponse.setTime((String) data.get("time"));
                    invoiceStatusResponse.setEndTime((String) data.get("endtime"));
                    invoiceStatusResponse.setError((String) data.get("error"));
                    invoiceStatusResponse.setErrorMessage((String) data.get("ErMsg"));
                    vos.add(invoiceStatusResponse);
                }
                boeStatusResult.setBoeStatusVOS(vos);
            }
            return boeStatusResult;
        } catch (Exception e) {
            log.error("queryBoeStatus error", e);
            throw e;
        }
    }
}
