package cn.superdesk.uniorder.service.thirdpart.jd.impl;

import cn.hutool.core.util.StrUtil;
import com.alibaba.excel.write.handler.AbstractRowWriteHandler;
import com.alibaba.excel.write.metadata.holder.WriteSheetHolder;
import com.alibaba.excel.write.metadata.holder.WriteTableHolder;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFDateUtil;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFClientAnchor;
import org.apache.poi.xssf.usermodel.XSSFRichTextString;

import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * @Description: 自定义拦截器-注解定义, Excel Cell 批注反馈
 * 校验模式:
 * 1. 快速失败返回模式
 * 2. 普通模式
 * 反馈模式:
 * 1. 批注-错误的Cell
 * 2. 行尾-添加两列[结果,报告]
 * @Author: linjinyu
 * @CreateDate: 2021/8/24 上午11:14
 * @UpdateUser: linjinyu
 * @UpdateDate: 2021/8/24 上午11:14
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Slf4j
public class AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelRowWriteHandler extends AbstractRowWriteHandler {


	@Override
	public void beforeRowCreate(WriteSheetHolder writeSheetHolder,
	                            WriteTableHolder writeTableHolder,
	                            Integer rowIndex,
	                            Integer relativeRowIndex,
	                            Boolean isHead) {

//		ClassUtils.declaredFields(headClazz, sortedAllFiledMap, indexFiledMap, ignoreMap, convertAllFiled, needIgnore, writeSheetHolder);
//		log.info("ExcelImportExportAddPostilRowWriteHandler\nbeforeRowCreate\n{}", JsonUtils.toJson(sortedAllFiledMap));
		log.info("beforeRowCreate {} isHead {}", relativeRowIndex, isHead);

	}

	@Override
	public void afterRowCreate(WriteSheetHolder writeSheetHolder,
	                           WriteTableHolder writeTableHolder,
	                           Row row,
	                           Integer relativeRowIndex,
	                           Boolean isHead) {
		log.info("afterRowCreate {} isHead {}", relativeRowIndex, isHead);
	}

	@Override
	public void afterRowDispose(WriteSheetHolder writeSheetHolder,
	                            WriteTableHolder writeTableHolder,
	                            Row row /* POI的东西 */,
	                            Integer relativeRowIndex,
	                            Boolean isHead) {

		log.info("afterRowDispose [{}] start", relativeRowIndex);

		// 如果false 就读取行数据.
		if (!isHead) {

			Cell cellN13 = row.getCell(13);
			// 结算金额(订单)N = 订单金额(订单)H + 退款金额(订单)I * 商户承担运费(订单)K
			// 以第1行数据为例:N1 = H1 + I1 + K1
			int actualCellRowNum01 = cellN13.getRowIndex();
			String formulaStr01 = StrUtil.format("H{}+I{}+K{}", actualCellRowNum01, actualCellRowNum01, actualCellRowNum01);
			cellN13.setCellFormula(formulaStr01);

			log.info("第{}行,第{}列 公式 {} 完成", cellN13.getRowIndex(), cellN13.getColumnIndex(), formulaStr01);


			Cell cellO14 = row.getCell(14);
			// 结算金额(订单)O = 订单金额(商品)L + 退款金额(商品)M * 商户承担运费(订单)K
			// 以第1行数据为例:O1 = L1 + M1 + K1
			int actualCellRowNum02 = cellO14.getRowIndex();
			String formulaStr02 = StrUtil.format("L{}+M{}+K{}", actualCellRowNum02, actualCellRowNum02, actualCellRowNum02);
			cellO14.setCellFormula(formulaStr02);

			log.info("第{}行,第{}列 写入公式完成", cellO14.getRowIndex(), cellO14.getColumnIndex());

		}
	}

	private void setReport(StringBuffer reportStrBuf, Sheet sheet, Integer relativeRowIndex, Integer relativeCellIndex, String msg) {

		reportStrBuf.append(StrUtil.format("R[{}]C[{}]ERROR[{}]\n", relativeRowIndex, relativeCellIndex, msg));
	}

	private void setPostil(Sheet sheet, Integer relativeRowIndex, Integer relativeCellIndex, String msg) {

		Workbook workbook = sheet.getWorkbook();

		CellStyle cellStyle = workbook.createCellStyle();
		// 设置前景填充样式
		cellStyle.setFillPattern(FillPatternType.SOLID_FOREGROUND);
		// 设置前景色为黄色
		cellStyle.setFillForegroundColor(IndexedColors.YELLOW.getIndex());
		//设置垂直居中
		cellStyle.setVerticalAlignment(VerticalAlignment.CENTER);

		Drawing<?> drawingPatriarch = sheet.createDrawingPatriarch();

		// 在第一行 第二列创建一个批注
		Comment comment = drawingPatriarch.createCellComment(new XSSFClientAnchor(0, 0, 0, 0, (short) 3, 3, (short) 5, 6));
		// 输入批注信息
		comment.setString(new XSSFRichTextString(msg));

//		// 将批注添加到单元格对象中 对空处理的话是需要创建的 createCell
//		sheet.getRow(relativeRowIndex + 1).createCell(relativeCellIndex).setCellComment(comment);
//		sheet.getRow(relativeRowIndex + 1).createCell(relativeCellIndex).setCellStyle(cellStyle);

		sheet.getRow(relativeRowIndex + 1).getCell(relativeCellIndex).setCellComment(comment);
		sheet.getRow(relativeRowIndex + 1).getCell(relativeCellIndex).setCellStyle(cellStyle);
	}


	// 判断Excel导入的数据类型,转换成数据库可识别的数据类型
	@Deprecated
	private String getCellTypes(Cell cell) {

		String cellValue = null;

		if (null != cell) {

			// 以下是判断数据的类型
			switch (cell.getCellType()) {
				case HSSFCell.CELL_TYPE_NUMERIC: // 数字
					// 处理日期格式、时间格式
					if (HSSFDateUtil.isCellDateFormatted(cell)) {
						Date       d        = cell.getDateCellValue();
						DateFormat formater = new SimpleDateFormat("yyyy-MM-dd");
						cellValue = formater.format(d);
					} else {
						cellValue = cell.getNumericCellValue() + "";
					}


					break;
				// 字符串
				case HSSFCell.CELL_TYPE_STRING:
					cellValue = cell.getStringCellValue();
					break;
				// Boolean
				case HSSFCell.CELL_TYPE_BOOLEAN:
					cellValue = cell.getBooleanCellValue() + "";
					break;
				// 公式
				case HSSFCell.CELL_TYPE_FORMULA:
					// cellValue = cell.getCellFormula() + "";
					try {
						DecimalFormat df = new DecimalFormat("0.0000");
						cellValue = String.valueOf(df.format(cell.getNumericCellValue()));

					} catch (IllegalStateException e) {
						cellValue = String.valueOf(cell.getRichStringCellValue());
					}

					break;
				// 空值
				case HSSFCell.CELL_TYPE_BLANK:
					cellValue = "";
					break;
				// 故障
				case HSSFCell.CELL_TYPE_ERROR:
					cellValue = "非法字符";
					break;

				default:
					cellValue = "未知类型";
					break;
			}
		}
		return cellValue;
	}
}



