package cn.superdesk.uniorder.service.thirdpart.jd.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.io.FileUtil;
import cn.superdesk.uniorder.common.enums.common.CommonProcessResultEnum;
import cn.superdesk.uniorder.common.enums.thirdpart.jd.SysQuartzScheduleTaskStatementOfAccountRunStateEnum;
import cn.superdesk.uniorder.common.excel.ExcelResponse;
import cn.superdesk.uniorder.common.excel.strategy.style.EasyExcelCustomCellWriteHandler;
import cn.superdesk.uniorder.common.excel.strategy.style.EasyExcelWriteHandlerCellStyleUtils;
import cn.superdesk.uniorder.common.numerical.AnBeanUtils;
import cn.superdesk.uniorder.common.security.dto.LoginSysUser;
import cn.superdesk.uniorder.dao.entity.sys.notify.SysNotify;
import cn.superdesk.uniorder.dto._constant.ResourceConstant;
import cn.superdesk.uniorder.dto.param.thirdpart.jd.AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelRow;
import cn.superdesk.uniorder.dto.vo.thirdpart.jd.AccountingUnifiedSupplierOrderJdCheckUpReportListItemVo;
import cn.superdesk.uniorder.notify.NotifyUtils;
import cn.superdesk.uniorder.service.thirdpart.jd.AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelService;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @Description: Java类作用描述
 * @Author: linjinyu
 * @CreateDate: 2021/11/5 上午11:29
 * @UpdateUser: linjinyu
 * @UpdateDate: 2021/11/5 上午11:29
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Slf4j
@Service
public class AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelServiceImpl
        implements AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelService {

    @Autowired
    private ResourceConstant resourceConstant;

    private static final String TEMPLATE_EXCEL_EXPORT_ACCOUNTING_UNIFIED_SUPPLIER_ORDER_JD_CHECKUP_REPORT_FULL_VERSION = "exportExcelJdCheckUpReport_FullVersion_v20211108.xlsx";
    private static final String TEMPLATE_EXCEL_EXPORT_ACCOUNTING_UNIFIED_SUPPLIER_ORDER_JD_CHECKUP_REPORT_LITE_VERSION = "exportExcelJdCheckUpReport_LiteVersion_v20220614.xlsx";


    @Override
    public SysNotify exportExcelJdCheckUpReportFullVersion(List<AccountingUnifiedSupplierOrderJdCheckUpReportListItemVo> resultList, String batchId, LoginSysUser loginSysUser) {


        String excelExportPathRoot             = resourceConstant.getPathRoot();
        String excelExportRelativePathTemplate = resourceConstant.getPathTemplate();
        String excelExportRelativePathStorage  = resourceConstant.getPathStorage();

        String relativePath = File.separator + "excel" +
                File.separator + "accounting" +
                File.separator + "score" +
                File.separator + "jd" +
                File.separator;

        String templateRelativePath = excelExportPathRoot + excelExportRelativePathTemplate + relativePath;
        String templateFullPath;

        String targetFullPath = excelExportPathRoot + excelExportRelativePathStorage;

        String targetFileName          = "京东对账" + "_" + "比对报告_" + batchId + ExcelResponse.addTail() + ".xlsx";
        String targetWriteFileFullPath = targetFullPath + relativePath + targetFileName;

//		if (!FileUtil.exist(targetWriteFileFullPath)) {
//			FileUtil.mkdir(targetWriteFileFullPath);
//		}
        FileUtil.touch(targetWriteFileFullPath);


        templateFullPath = templateRelativePath + TEMPLATE_EXCEL_EXPORT_ACCOUNTING_UNIFIED_SUPPLIER_ORDER_JD_CHECKUP_REPORT_FULL_VERSION;

        ExcelWriter excelWriter = EasyExcel
                .write(targetWriteFileFullPath)
//				.inMemory(Boolean.TRUE)
//				.withTemplate(templateFullPath)
                .head(AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelRow.class)
                .build();

        WriteSheet writeSheet0 = EasyExcel
                .writerSheet(0)
//				.registerWriteHandler(new AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelCellWriteHandler())
//				.registerWriteHandler(new AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelRowWriteHandler())
                .build();

//		FillConfig fillConfig  = FillConfig.builder().forceNewRow(Boolean.FALSE).build();

        log.info("\n{}\n{}\n{}", templateFullPath, targetWriteFileFullPath, resultList.size());

        // 转换数据 AccountingUnifiedSupplierOrderJdCheckUpReport -> AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelRow

        List<AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelRow> convertResultList = convertAccountingUnifiedSupplierOrderJdCheckUpReport2AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelRow(resultList);

        // 数据填充
//		excelWriter.fill(new FillWrapper("detail", resultList), fillConfig, writeSheet0);
//		excelWriter.fill(new FillWrapper("detail", resultList), writeSheet0);
//		excelWriter.fill(convertResultList, writeSheet0);

        excelWriter
                .write(convertResultList, writeSheet0);

        // 设置强制计算公式:不然公式会以字符串的形式显示在excel中
//		Workbook workbook = excelWriter.writeContext().writeWorkbookHolder().getWorkbook();
//		workbook.setForceFormulaRecalculation(true);

        // 别忘记关闭流
        excelWriter
                .finish();

        return NotifyUtils.createSysNotify(
                targetFileName,
                relativePath + targetFileName,
                SysQuartzScheduleTaskStatementOfAccountRunStateEnum.成功
        );

    }

    @Override
    public SysNotify exportExcelJdCheckUpReportLiteVersion(
            List<AccountingUnifiedSupplierOrderJdCheckUpReportListItemVo> resultList,
            String batchId,
            LoginSysUser loginSysUser) {


        String excelExportPathRoot             = resourceConstant.getPathRoot();
        String excelExportRelativePathTemplate = resourceConstant.getPathTemplate();
        String excelExportRelativePathStorage  = resourceConstant.getPathStorage();

        String relativePath = File.separator + "excel" +
                File.separator + "accounting" +
                File.separator + "score" +
                File.separator + "jd" +
                File.separator;

        String templateRelativePath = excelExportPathRoot + excelExportRelativePathTemplate + relativePath;
        String templateFullPath;

        String targetFullPath = excelExportPathRoot + excelExportRelativePathStorage;

        String targetFileName          = "京东对账" + "_" + "比对报告_" + batchId + ExcelResponse.addTail() + ".xlsx";
        String targetWriteFileFullPath = targetFullPath + relativePath + targetFileName;

//		if (!FileUtil.exist(targetWriteFileFullPath)) {
//			FileUtil.mkdir(targetWriteFileFullPath);
//		}
        FileUtil.touch(targetWriteFileFullPath);


        templateFullPath = templateRelativePath + TEMPLATE_EXCEL_EXPORT_ACCOUNTING_UNIFIED_SUPPLIER_ORDER_JD_CHECKUP_REPORT_LITE_VERSION;

        /**************************************** fill start ****************************************/

//        ExcelWriterBuilder excelWriterBuilder = EasyExcel
//                .write(targetWriteFileFullPath);
//        ExcelWriter excelWriter = null;
//
//        WriteSheet writeSheet0 = EasyExcel
//                .writerSheet(0, "列表")
//                .build();
//        FillConfig fillConfig = FillConfig
//                .builder()
//                .forceNewRow(Boolean.FALSE)
//                .build();
//
//        excelWriter = excelWriterBuilder
//                .withTemplate(templateFullPath)
//                .build();
//        excelWriter
//                .fill(new FillWrapper("detail", resultList), fillConfig, writeSheet0);
//
//        // 别忘记关闭流
//        excelWriter
//                .finish();

        /**************************************** fill   end ****************************************/

        final List<Integer> mergeColumnIndex = new ArrayList<Integer>() {
            {
                add(0);
                add(1);
                add(2);
//                add(3);
//                add(4);
                add(5);
                add(6);
                add(7);
                add(8);
            }
        };

        EasyExcel
                .write(targetWriteFileFullPath, AccountingUnifiedSupplierOrderJdCheckUpReportListItemVo.class)
                .needHead(Boolean.TRUE)
                .includeColumnFiledNames(Arrays.asList(
                        "accountingUnifiedSupplierOrderJdId",
                        "jdOrderSn",
                        "jdOrderAmount",
                        "goddogOrderSn",
                        "goodsCostPriceXQuantity",
                        "wholeOrderAmount",
                        "processResultName",
                        "comparedWholeAmount",
                        "defineErrorName"
                ))
                .inMemory(Boolean.TRUE)
                // 合并单位格
                .registerWriteHandler(
                        new AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelJdCheckReportLiteVersionMergeStrategy(
                                1,
                                mergeColumnIndex.stream().mapToInt(Integer::valueOf).toArray(),
                                0
                        )

                )
                // 表头与内容样式
                .registerWriteHandler(EasyExcelWriteHandlerCellStyleUtils.getHorizontalCellStyleStrategy())
                // 列宽
                .registerWriteHandler(new EasyExcelCustomCellWriteHandler())
                .sheet("比对结果")
                .doWrite(resultList);


        return NotifyUtils.createSysNotify(
                targetFileName,
                relativePath + targetFileName,
                SysQuartzScheduleTaskStatementOfAccountRunStateEnum.成功
        );

    }

    private static List<AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelRow> convertAccountingUnifiedSupplierOrderJdCheckUpReport2AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelRow(
            List<AccountingUnifiedSupplierOrderJdCheckUpReportListItemVo> resultList) {

        if (CollUtil.isEmpty(resultList)) {
            return null;
        }

        List<AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelRow> convertResultList = new ArrayList<>();

        resultList.forEach(item -> {

            AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelRow tmp = AnBeanUtils.copy(item, AccountingUnifiedSupplierOrderJdCheckUpReportExportExcelRow.class);

            tmp.setFormulaOrderAmount(tmp.getOrderCostAmount()
                    .add(tmp.getRefundOrderCostAmount())
                    .add(tmp.getTransportMerchantAssumeAmount())
            );
            tmp.setFormulaGoodsAmount(tmp.getGoodsCostAmount()
                    .add(tmp.getRefundGoodsCostAmount())
                    .add(tmp.getTransportMerchantAssumeAmount())
            );

            tmp.setDifferOrderAmount(tmp.getJdOutStandingAmount().subtract(tmp.getFormulaOrderAmount()));
            tmp.setDifferGoodsAmount(tmp.getJdOutStandingAmount().subtract(tmp.getFormulaGoodsAmount()));

            tmp.setProcessResultStr(CommonProcessResultEnum.get(tmp.getProcessResult()).getDesc());

            convertResultList.add(tmp);

        });

        return convertResultList;

    }


}
