package cn.superdesk.uniorder.service.thirdpart.jd.impl;

import cn.superdesk.uniorder.common.enums.scheduler.jd.ScheduleTaskProcessStateEnum;
import cn.superdesk.uniorder.common.json.AnJsonUtils;
import cn.superdesk.uniorder.common.numerical.AnBigDecimalUtils;
import cn.superdesk.uniorder.common.security.dto.LoginSysUser;
import cn.superdesk.uniorder.dto.param.thirdpart.jd.AccountingUnifiedSupplierOrderJdImportLiteVersionExcelRow;
import cn.superdesk.uniorder.dto.param.thirdpart.jd.AccountingUnifiedSupplierOrderJdImportLiteVersionFacadeExcelRow;
import cn.superdesk.uniorder.service.thirdpart.jd.AccountingUnifiedSupplierOrderJdService;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import lombok.extern.slf4j.Slf4j;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @Description: 有个很重要的点
 * DataListener 不能被spring管理,
 * 要每次读取excel都要new,
 * 然后里面用到spring可以构造方法传进去
 * @Author: linjinyu
 * @CreateDate: 2021/3/16 下午7:06
 * @UpdateUser: linjinyu
 * @UpdateDate: 2021/3/16 下午7:06
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Slf4j
public class ThirdPartJdReconciliationScheduleLiteVersionExcelRowImportDataListener
        extends AnalysisEventListener<AccountingUnifiedSupplierOrderJdImportLiteVersionFacadeExcelRow> {

    /**
     * 每隔5条存储数据库,实际使用中可以3000条,然后清理list ,方便内存回收
     */
    private static final int BATCH_COUNT = 1000;

    private List<AccountingUnifiedSupplierOrderJdImportLiteVersionExcelRow> accountingUnifiedSupplierOrderJdImportLiteVersionExcelRows = new ArrayList<>();

    private AccountingUnifiedSupplierOrderJdService accountingUnifiedSupplierOrderJdDubboConsumerService;

    private LoginSysUser loginSysUser;
    private String       batchId;

    /**
     * 如果使用了spring,请使用这个构造方法。每次创建Listener的时候需要把spring管理的类传进来
     *
     * @param accountingUnifiedSupplierOrderJdDubboConsumerService
     * @param loginSysUser
     */
    public ThirdPartJdReconciliationScheduleLiteVersionExcelRowImportDataListener(
            AccountingUnifiedSupplierOrderJdService accountingUnifiedSupplierOrderJdDubboConsumerService,
            String batchId,
            LoginSysUser loginSysUser
    ) {
        this.accountingUnifiedSupplierOrderJdDubboConsumerService = accountingUnifiedSupplierOrderJdDubboConsumerService;

        this.loginSysUser = loginSysUser;
        this.batchId = batchId;
    }


    @Override
    public void invokeHeadMap(Map<Integer, String> headMap, AnalysisContext context) {

        log.info("解析到一条头数据:{}, currentRowHolder: {}", AnJsonUtils.toJson(headMap), context.readRowHolder().getRowIndex());

        if (context.readRowHolder().getRowIndex().compareTo(0) == 0) {

            log.info("[doggod] : {}", "狗头");
        }


//		list.add(headMap);
    }

    /**
     * 这个每一条数据解析都会来调用
     *
     * @param facadeExcelRow one row value. Is is same as {@link AnalysisContext#readRowHolder()}
     * @param context
     */
    @Override
    public void invoke(AccountingUnifiedSupplierOrderJdImportLiteVersionFacadeExcelRow facadeExcelRow, AnalysisContext context) {

        // 主要是这些狗没有规范，希望随心所欲，反正从第二行(rowIndex 1开始) facadeExcelRow为了限定只读第一列第二列
        if (context.readRowHolder().getRowIndex().compareTo(0) == 0) {
            log.info("[doggod] : {} : {}", "狗头", AnJsonUtils.toJson(facadeExcelRow));
            return;
        }

        log.info("解析到一条数据: {}", AnJsonUtils.toJson(facadeExcelRow));


        AccountingUnifiedSupplierOrderJdImportLiteVersionExcelRow excelRow = new AccountingUnifiedSupplierOrderJdImportLiteVersionExcelRow();

        excelRow.setJdOrderAmountStr(facadeExcelRow.getJdOrderAmountStr());
        excelRow.setJdOrderSn(facadeExcelRow.getJdOrderSn());

        excelRow.setJdOrderAmount(AnBigDecimalUtils.getBigDecimal(excelRow.getJdOrderAmountStr(), BigDecimal.ZERO));

        excelRow.setBatchId(batchId);
        excelRow.setProcessResult(ScheduleTaskProcessStateEnum.导入中.getKey());
        excelRow.setProcessReport("");

        accountingUnifiedSupplierOrderJdImportLiteVersionExcelRows.add(excelRow);

        // 达到BATCH_COUNT了,需要去存储一次数据库,防止数据几万条数据在内存,容易OOM
        if (accountingUnifiedSupplierOrderJdImportLiteVersionExcelRows.size() >= BATCH_COUNT) {

            saveOrUpdateData();
            // 存储完成清理 list
            accountingUnifiedSupplierOrderJdImportLiteVersionExcelRows.clear();
        }
    }

    /**
     * 所有数据解析完成了 都会来调用
     *
     * @param context
     */
    @Override
    public void doAfterAllAnalysed(AnalysisContext context) {

        // 这里也要保存数据,确保最后遗留的数据也存储到数据库
        saveOrUpdateData();

        log.info("所有数据解析完成！");
    }

    /**
     * 加上存储数据库
     */
    private void saveOrUpdateData() {

        log.info("{} 条数据,开始存储数据库！\n{}",
                accountingUnifiedSupplierOrderJdImportLiteVersionExcelRows.size(),
                AnJsonUtils.toJson(accountingUnifiedSupplierOrderJdImportLiteVersionExcelRows)
        );

        accountingUnifiedSupplierOrderJdDubboConsumerService
                .insertBatch4ThirdPartJdReconciliationScheduleLiteVersionExcelRowList(
                        accountingUnifiedSupplierOrderJdImportLiteVersionExcelRows,
                        loginSysUser
                );

        log.info("存储数据库成功！");
    }
}
