package cn.superdesk.uniorder.service.thirdpart.qchjc.statistics.impl;

import cn.hutool.core.io.FileUtil;
import cn.superdesk.uniorder.common.enums.accounting.customer.InAccountTypeEnum;
import cn.superdesk.uniorder.common.enums.accounting.customer.InAccountWayEnum;
import cn.superdesk.uniorder.common.enums.accounting.customer.RmAccountStateEnum;
import cn.superdesk.uniorder.common.enums.thirdpart.jd.SysQuartzScheduleTaskStatementOfAccountRunStateEnum;
import cn.superdesk.uniorder.common.excel.ExcelResponse;
import cn.superdesk.uniorder.common.numerical.AnDateUtils;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.dao.entity.biz.FlowPayableSummary;
import cn.superdesk.uniorder.dao.entity.biz.FlowReceivableSummary;
import cn.superdesk.uniorder.dao.entity.sys.SysUser;
import cn.superdesk.uniorder.dao.entity.sys.notify.SysNotify;
import cn.superdesk.uniorder.dao.mapper._statistics.PayableStatisticsMapper;
import cn.superdesk.uniorder.dao.mapper._statistics.ReceivableStatisticsMapper;
import cn.superdesk.uniorder.dao.mapper.biz.FlowPayableDetailMapper;
import cn.superdesk.uniorder.dao.mapper.biz.FlowPayableSummaryMapper;
import cn.superdesk.uniorder.dao.mapper.biz.FlowReceivableDetailMapper;
import cn.superdesk.uniorder.dao.mapper.biz.FlowReceivableSummaryMapper;
import cn.superdesk.uniorder.dto._constant.ResourceConstant;
import cn.superdesk.uniorder.dto.param.accounting.statistics.PayableBalanceQueryParam;
import cn.superdesk.uniorder.dto.param.accounting.statistics.ReceivableBalanceQueryParam;
import cn.superdesk.uniorder.dto.vo._statistics.ReceivableBalanceDetail;
import cn.superdesk.uniorder.dto.vo._statistics.ReceivableBalanceVO;
import cn.superdesk.uniorder.dto.vo._statistics.payable.PayableBalanceDetail;
import cn.superdesk.uniorder.dto.vo._statistics.payable.PayableBalanceVO;
import cn.superdesk.uniorder.notify.NotifyUtils;
import cn.superdesk.uniorder.service.sys.SysUserService;
import cn.superdesk.uniorder.service.thirdpart.qchjc.statistics.BalanceStatistcsService;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.builder.ExcelWriterBuilder;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.metadata.fill.FillConfig;
import com.alibaba.excel.write.metadata.fill.FillWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.File;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author xc
 * @description 应收/应付 余额报表service
 * @date 2022/6/22 09:32
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class BalanceStatistcsServiceImpl implements BalanceStatistcsService {
    private final ResourceConstant excelExportConstant;
    private final ReceivableStatisticsMapper receivableStatisticsMapper;
    private final PayableStatisticsMapper payableStatisticsMapper;
    @Autowired
    private FlowReceivableDetailMapper flowReceivableDetailMapper;
    @Autowired
    private FlowReceivableSummaryMapper flowReceivableSummaryMapper;
    @Autowired
    private FlowPayableDetailMapper flowPayableDetailMapper;
    @Autowired
    private FlowPayableSummaryMapper flowPayableSummaryMapper;
    //应收余额模板
    private static final String EXPORT_RECEIVABLE_BALANCE_TEMPLATE = "exportExcelReceivableBalanceTemplate.xls";
    //应付余额模板
    private static final String EXPORT_PAYABLE_BALANCE_TEMPLATE = "exportExcelPayableBalanceTemplate.xls";
    @Autowired
    private SysUserService sysUserService;
    /**
     * 应收余额列表查询
     *
     * @param param
     * @return
     */
    @Override
    public R<ReceivableBalanceVO> getReceivableBalanceVOList(ReceivableBalanceQueryParam param) {

        PageHelper.startPage(param.getPageNumber(), param.getPageSize());
        //获得应收报表分页后的settlementId
        List<String> settlementIdList = receivableStatisticsMapper.getSettlementIdsPage(param);
        PageInfo<String> page = new PageInfo<>(settlementIdList);
        PageHelper.clearPage();

        param.setSettlementIds(settlementIdList);

        //列表查询
        List<ReceivableBalanceDetail> res = receivableStatisticsMapper.getReceivableBalanceVOList(param);
        //应收合计
        BigDecimal totalReceivableAmount = res.stream().map(ReceivableBalanceDetail::getReceivableAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
        //实收合计
        BigDecimal totalActualReceipt = res.stream().map(ReceivableBalanceDetail::getActualReceipt).reduce(BigDecimal.ZERO, BigDecimal::add);
        //期末余额合计
        BigDecimal totalEndingBalance = res.stream().map(ReceivableBalanceDetail::getEndingBalance).reduce(BigDecimal.ZERO, BigDecimal::add);
        int seq = 0;
        for (ReceivableBalanceDetail re : res) {
            re.setSeq(String.valueOf(++seq));
        }
        ReceivableBalanceVO receivableBalanceVO = new ReceivableBalanceVO();
        receivableBalanceVO.setReceivableBalanceList(res);
        receivableBalanceVO.setTotalReceivableAmount(totalReceivableAmount);
        receivableBalanceVO.setTotalActualReceipt(totalActualReceipt);
        receivableBalanceVO.setTotalEndingBalance(totalEndingBalance);
        return new R<>(param.getPageNumber(), param.getPageSize(), (int) page.getTotal(), receivableBalanceVO);

    }

    /**
     * 应收余额报表导出
     * @param param
     * @return
     */
    @Override
    public SysNotify exportExcelReceivableBalance(ReceivableBalanceQueryParam param) {
        //应收余额sheet，汇总数据
        List<ReceivableBalanceDetail> sheet0Data = getReceivableBalanceTotalExcelData(param);

        List<String> settlementIdList = param.getSettlementIds();

        List<FlowReceivableSummary> receivableSummaryList = Lists.newArrayList();
        List<FlowReceivableSummary> receivedSummaryList = Lists.newArrayList();
        if (CollectionUtils.isNotEmpty(sheet0Data)) {
            Map<String,String> settlementNameMap = sheet0Data.stream()
            .filter(data -> StringUtils.isNotBlank(data.getSettlementId()) && StringUtils.isNotBlank(data.getSettlementName()) && !"小计".equals(data.getSettlementName()))
                    .collect(Collectors.toMap(data -> data.getSettlementId(), data -> data.getSettlementName(),(v1, v2) -> v2));
            Set<Long> userIdList = Sets.newHashSet();
            //应收流水汇总
            FlowReceivableSummary receivableSummaryParam = new FlowReceivableSummary();
            receivableSummaryParam.setSettlementIdList(settlementIdList);
            receivableSummaryParam.setType(0);
            receivableSummaryParam.setInAccountTypeList(Lists.newArrayList(1, 2));
            receivableSummaryParam.setRmAccountState(2);
            receivableSummaryParam.setStartTime(param.getStartTime());
            receivableSummaryParam.setEndTime(param.getEndTime());
            receivableSummaryParam.setBizPlatformId(param.getServiceNo());
            receivableSummaryParam.setCusBizCode(param.getSettlementId());
            receivableSummaryList = flowReceivableSummaryMapper.selectFlowReceivableSummaryList(receivableSummaryParam);
            if (CollectionUtils.isNotEmpty(receivableSummaryList)) {
                receivableSummaryList.forEach(data -> {
                    data.setRmAccountStateName(RmAccountStateEnum.getDesc(data.getRmAccountState()));
                    data.setInAccountTypeName(InAccountTypeEnum.getDesc(data.getInAccountType()));
                    data.setInAccountWayName(InAccountWayEnum.getDesc(data.getInAccountWay()));
                    if(StringUtils.isBlank(data.getCreateBy())){
                        userIdList.add(data.getCreateId());
                    }
                    if(StringUtils.isBlank(data.getUpdateBy())){
                        userIdList.add(data.getUpdateId());
                    }
                    data.setCusBizName(settlementNameMap.get(data.getCusBizCode()));
                });
            }

            //实收流水汇总
            receivableSummaryParam.setType(1);
            receivableSummaryParam.setInAccountTypeList(null);
            receivedSummaryList = flowReceivableSummaryMapper.selectFlowReceivableSummaryList(receivableSummaryParam);
            if (CollectionUtils.isNotEmpty(receivedSummaryList)) {
                receivedSummaryList.forEach(data -> {
                    data.setRmAccountStateName(RmAccountStateEnum.getDesc(data.getRmAccountState()));
                    data.setInAccountTypeName(InAccountTypeEnum.getDesc(data.getInAccountType()));
                    data.setInAccountWayName(InAccountWayEnum.getDesc(data.getInAccountWay()));
                    if(StringUtils.isBlank(data.getCreateBy())){
                        userIdList.add(data.getCreateId());
                    }
                    if(org.apache.commons.lang3.StringUtils.isBlank(data.getUpdateBy())){
                        userIdList.add(data.getUpdateId());
                    }
                    data.setCusBizName(settlementNameMap.get(data.getCusBizCode()));
                });
            }

            if(CollectionUtils.isNotEmpty(userIdList)){
                List<SysUser> sysUserList = sysUserService.findBySysUserIds(Lists.newArrayList(userIdList));
                Map<Long,String> sysUserMap = sysUserList.stream().collect(Collectors.toMap(data -> data.getId(), data -> data.getFullname(),(v1, v2) -> v2));
                if(CollectionUtils.isNotEmpty(receivableSummaryList)){
                    receivableSummaryList.forEach(data ->{
                        if(StringUtils.isBlank(data.getCreateBy())){
                            data.setCreateBy(sysUserMap.get(data.getCreateId()));
                        }
                        if(StringUtils.isBlank(data.getUpdateBy())){
                            data.setCreateBy(sysUserMap.get(data.getUpdateId()));
                        }
                    });
                }
                if(CollectionUtils.isNotEmpty(receivedSummaryList)){
                    receivedSummaryList.forEach(data ->{
                        if(StringUtils.isBlank(data.getCreateBy())){
                            data.setCreateBy(sysUserMap.get(data.getCreateId()));
                        }
                        if(StringUtils.isBlank(data.getUpdateBy())){
                            data.setCreateBy(sysUserMap.get(data.getUpdateId()));
                        }
                    });
                }
            }
        }

        //sheet0 日期
        Map<String, Object> exportDate = handleDate(param.getStartTime(), param.getEndTime());

        String excelExportPathRoot = excelExportConstant.getPathRoot();
        String excelExportRelativePathTemplate = excelExportConstant.getPathTemplate();
        String excelExportRelativePathStorage = excelExportConstant.getPathStorage();

        String relativePath = File.separator + "excel" + File.separator + "statistic" + File.separator;
        String templateRelativePath = excelExportPathRoot + excelExportRelativePathTemplate + relativePath;
        String templateFullPath = templateRelativePath + EXPORT_RECEIVABLE_BALANCE_TEMPLATE;

        String targetFullPath = excelExportPathRoot + excelExportRelativePathStorage;
        String targetFileName = "应收余额表_财务报表" + ExcelResponse.addTail() + ".xlsx";
        String targetWriteFileFullPath = targetFullPath + relativePath + targetFileName;

        FileUtil.touch(targetWriteFileFullPath);

        ExcelWriterBuilder excelWriterBuilder = EasyExcel.write(targetWriteFileFullPath);
        ExcelWriter excelWriter = null;
        WriteSheet writeSheet0 = EasyExcel.writerSheet(0, "应收余额表").build();
        WriteSheet writeSheet1 = EasyExcel.writerSheet(1, "应收帐款").build();
        WriteSheet writeSheet2 = EasyExcel.writerSheet(2, "实收账款").build();

        excelWriter = excelWriterBuilder.withTemplate(templateFullPath).build();

        FillConfig fillConfig = FillConfig.builder().forceNewRow(Boolean.FALSE).build();
        excelWriter.fill(new FillWrapper("detail", sheet0Data), fillConfig, writeSheet0).fill(exportDate, writeSheet0);
        excelWriter.fill(receivableSummaryList, fillConfig, writeSheet1);
        excelWriter.fill(receivedSummaryList, fillConfig, writeSheet2);

        excelWriter.finish();

        return NotifyUtils.createSysNotify(targetFileName, relativePath + targetFileName, SysQuartzScheduleTaskStatementOfAccountRunStateEnum.成功);
    }

    /**
     * 应付余额报表导出
     * @param param
     * @return
     */
    @Override
    public SysNotify exportExcelPayableBalance(PayableBalanceQueryParam param) {
        //应付余额sheet，汇总数据
        List<PayableBalanceDetail> sheet0Data = getPayableBalanceTotalExcelData(param);
        List<String> settlementIdList = param.getSettlementIds();

        List<FlowPayableSummary> payableSummaryList = Lists.newArrayList();
        List<FlowPayableSummary> actualSummaryList = Lists.newArrayList();
        if(CollectionUtils.isNotEmpty(sheet0Data)) {
            Map<String,String> settlementNameMap = sheet0Data.stream()
                    .filter(data -> StringUtils.isNotBlank(data.getSettlementId()) && StringUtils.isNotBlank(data.getSettlementName()) && !"小计".equals(data.getSettlementName()))
                    .collect(Collectors.toMap(data -> data.getSettlementId(),data -> data.getSettlementName(),(v1,v2) -> v2));
            Set<Long> userIdList = Sets.newHashSet();
            //应付账款
            FlowPayableSummary payableSummaryParam = new FlowPayableSummary();
            payableSummaryParam.setSettlementIdList(settlementIdList);
            payableSummaryParam.setType(0);
            payableSummaryParam.setInAccountTypeList(Lists.newArrayList(1, 2));
            payableSummaryParam.setRmAccountState(2);
            payableSummaryParam.setStartTime(param.getStartTime());
            payableSummaryParam.setEndTime(param.getEndTime());
            payableSummaryParam.setBizPlatformId(param.getServiceNo());
            payableSummaryParam.setCusBizCode(param.getSettlementId());
            payableSummaryList = flowPayableSummaryMapper.selectFlowPayableSummaryList(payableSummaryParam);
            if (CollectionUtils.isNotEmpty(payableSummaryList)) {
                payableSummaryList.forEach(data -> {
                    data.setRmAccountStateName(RmAccountStateEnum.getDesc(data.getRmAccountState()));
                    data.setInAccountTypeName(InAccountTypeEnum.getDesc(data.getInAccountType()));
                    data.setInAccountWayName(InAccountWayEnum.getDesc(data.getInAccountWay()));
                    if(StringUtils.isBlank(data.getCreateBy())){
                        userIdList.add(data.getCreateId());
                    }
                    if(StringUtils.isBlank(data.getUpdateBy())){
                        userIdList.add(data.getUpdateId());
                    }
                    data.setCusBizName(settlementNameMap.get(data.getCusBizCode()));
                });
            }
            //实付账款
            payableSummaryParam.setType(1);
            payableSummaryParam.setInAccountTypeList(null);
            actualSummaryList = flowPayableSummaryMapper.selectFlowPayableSummaryList(payableSummaryParam);
            if (CollectionUtils.isNotEmpty(actualSummaryList)) {
                actualSummaryList.forEach(data -> {
                    data.setRmAccountStateName(RmAccountStateEnum.getDesc(data.getRmAccountState()));
                    data.setInAccountTypeName(InAccountTypeEnum.getDesc(data.getInAccountType()));
                    data.setInAccountWayName(InAccountWayEnum.getDesc(data.getInAccountWay()));
                    if(StringUtils.isBlank(data.getCreateBy())){
                        userIdList.add(data.getCreateId());
                    }
                    if(StringUtils.isBlank(data.getUpdateBy())){
                        userIdList.add(data.getUpdateId());
                    }
                    data.setCusBizName(settlementNameMap.get(data.getCusBizCode()));
                });
            }


            if(CollectionUtils.isNotEmpty(userIdList)){
                List<SysUser> sysUserList = sysUserService.findBySysUserIds(Lists.newArrayList(userIdList));
                Map<Long,String> sysUserMap = sysUserList.stream().collect(Collectors.toMap(data -> data.getId(), data -> data.getFullname(),(v1, v2) -> v2));
                if(CollectionUtils.isNotEmpty(payableSummaryList)){
                    payableSummaryList.forEach(data ->{
                        if(StringUtils.isBlank(data.getCreateBy())){
                            data.setCreateBy(sysUserMap.get(data.getCreateId()));
                        }
                        if(StringUtils.isBlank(data.getUpdateBy())){
                            data.setCreateBy(sysUserMap.get(data.getUpdateId()));
                        }
                    });
                }
                if(CollectionUtils.isNotEmpty(actualSummaryList)){
                    actualSummaryList.forEach(data ->{
                        if(StringUtils.isBlank(data.getCreateBy())){
                            data.setCreateBy(sysUserMap.get(data.getCreateId()));
                        }
                        if(StringUtils.isBlank(data.getUpdateBy())){
                            data.setCreateBy(sysUserMap.get(data.getUpdateId()));
                        }
                    });
                }
            }
        }

        //sheet0 日期
        Map<String, Object> exportDate = handleDate(param.getStartTime(), param.getEndTime());

        String excelExportPathRoot = excelExportConstant.getPathRoot();
        String excelExportRelativePathTemplate = excelExportConstant.getPathTemplate();
        String excelExportRelativePathStorage = excelExportConstant.getPathStorage();

        String relativePath = File.separator + "excel" + File.separator + "statistic" + File.separator;
        String templateRelativePath = excelExportPathRoot + excelExportRelativePathTemplate + relativePath;
        String templateFullPath = templateRelativePath + EXPORT_PAYABLE_BALANCE_TEMPLATE;
        String targetFullPath = excelExportPathRoot + excelExportRelativePathStorage;
        String targetFileName = "应付余额表_财务报表" + ExcelResponse.addTail() + ".xlsx";
        String targetWriteFileFullPath = targetFullPath + relativePath + targetFileName;

        FileUtil.touch(targetWriteFileFullPath);
        ExcelWriterBuilder excelWriterBuilder = EasyExcel.write(targetWriteFileFullPath);
        ExcelWriter excelWriter = null;
        WriteSheet writeSheet0 = EasyExcel.writerSheet(0, "应付余额表").build();
        WriteSheet writeSheet1 = EasyExcel.writerSheet(1, "应收帐款").build();
        WriteSheet writeSheet2 = EasyExcel.writerSheet(2, "实收账款").build();
        excelWriter = excelWriterBuilder.withTemplate(templateFullPath).build();

        FillConfig fillConfig = FillConfig.builder().forceNewRow(Boolean.FALSE).build();
        excelWriter.fill(new FillWrapper("detail", sheet0Data), fillConfig, writeSheet0).fill(exportDate, writeSheet0);
        excelWriter.fill(payableSummaryList, fillConfig, writeSheet1);
        excelWriter.fill(actualSummaryList, fillConfig, writeSheet2);
        excelWriter.finish();

        return NotifyUtils.createSysNotify(targetFileName, relativePath + targetFileName, SysQuartzScheduleTaskStatementOfAccountRunStateEnum.成功);
    }

    private List<PayableBalanceDetail> getPayableBalanceTotalExcelData(PayableBalanceQueryParam param) {
        PageHelper.startPage(param.getPageNumber(), param.getPageSize());
        List<String> settlementIds = payableStatisticsMapper.getSettlementIdsPage(param);
        PageHelper.clearPage();

        param.setSettlementIds(settlementIds);
        List<PayableBalanceDetail> res = payableStatisticsMapper.getPayableBalanceVOList(param);
        int seq = 0;
        for (PayableBalanceDetail p : res) {
            p.setSeq(String.valueOf(++seq));
        }
        if (CollectionUtils.isNotEmpty(res)) {
            //应付合计
            BigDecimal totalPayableAmount = res.stream().map(PayableBalanceDetail::getPayableAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
            //实付合计
            BigDecimal totalActualPay = res.stream().map(PayableBalanceDetail::getActualPay).reduce(BigDecimal.ZERO, BigDecimal::add);
            //期末余额合计
            BigDecimal totalEndingBalance = res.stream().map(PayableBalanceDetail::getEndingBalance).reduce(BigDecimal.ZERO, BigDecimal::add);
            PayableBalanceDetail total = PayableBalanceDetail.builder()
                    .seq("合计")
                    .settlementName("")
                    .serviceNoName("")
                    .payableAmount(totalPayableAmount)
                    .actualPay(totalActualPay)
                    .endingBalance(totalEndingBalance).build();
            res.add(total);
        }
        return res;
    }


    private List<ReceivableBalanceDetail> getReceivableBalanceTotalExcelData(ReceivableBalanceQueryParam param) {
        PageHelper.startPage(param.getPageNumber(), param.getPageSize());
        //获得应收报表分页后的settlementId
        List<String> settlementIdList = receivableStatisticsMapper.getSettlementIdsPage(param);
        PageInfo<String> page = new PageInfo<>(settlementIdList);
        settlementIdList.add("1000000");//零星客户id
        PageHelper.clearPage();
        param.setSettlementIds(settlementIdList);
        //列表查询
        List<ReceivableBalanceDetail> res = receivableStatisticsMapper.getReceivableBalanceVOList(param);
        int seq = 0;
        for (ReceivableBalanceDetail re : res) {
            re.setSeq(String.valueOf(++seq));
        }
        if (CollectionUtils.isNotEmpty(res)) {
            //应收合计
            BigDecimal totalReceivableAmount = res.stream().map(ReceivableBalanceDetail::getReceivableAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
            //实收合计
            BigDecimal totalActualReceipt = res.stream().map(ReceivableBalanceDetail::getActualReceipt).reduce(BigDecimal.ZERO, BigDecimal::add);
            //期末余额合计
            BigDecimal totalEndingBalance = res.stream().map(ReceivableBalanceDetail::getEndingBalance).reduce(BigDecimal.ZERO, BigDecimal::add);

            //合计
            ReceivableBalanceDetail total = ReceivableBalanceDetail.builder()
                    .seq("合计")
                    .settlementName("")
                    .serviceNoName("")
                    .receivableAmount(totalReceivableAmount)
                    .actualReceipt(totalActualReceipt)
                    .endingBalance(totalEndingBalance).build();
            res.add(total);
        }
        return res;
    }

    /**
     * sheet0中的日期
     * @param
     * @return
     */
    private Map<String, Object> handleDate(String start, String end) {
        Map<String, Object> map = new HashMap<>();
        String startDate = "";
        String endDate = "";
        String exportDate = "";
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy年MM月dd日");
        if (StringUtils.isNotBlank(start)) {
            Date date = AnDateUtils.parseDate(start);
            startDate = simpleDateFormat.format(date);
        }
        if (StringUtils.isNotBlank(end)) {
            Date date = AnDateUtils.parseDate(end);
            endDate = simpleDateFormat.format(date);
        }

        if (StringUtils.isNotBlank(startDate) && StringUtils.isNotBlank(endDate)) {
            exportDate = startDate + "至" + endDate;
        }else if (StringUtils.isNotBlank(startDate) && StringUtils.isBlank(endDate)){
            endDate = simpleDateFormat.format(new Date());
            exportDate = startDate + "至" + endDate;
        }else if (StringUtils.isBlank(startDate)){
            endDate = simpleDateFormat.format(new Date());
            exportDate = "截止" + endDate;
        }

        map.put("exportDate", exportDate);
        return map;
    }
    /**
     * 应付余额列表查询
     *
     * @param param
     * @return
     */
    @Override
    public R<PayableBalanceVO> getPayableBalanceVOList(PayableBalanceQueryParam param) {
        PageHelper.startPage(param.getPageNumber(), param.getPageSize());
        List<String> settlementIds = payableStatisticsMapper.getSettlementIdsPage(param);
        PageInfo<String> page = new PageInfo<>(settlementIds);
        PageHelper.clearPage();
        param.setSettlementIds(settlementIds);
        List<PayableBalanceDetail> res = payableStatisticsMapper.getPayableBalanceVOList(param);
        //应付合计
        BigDecimal totalPayableAmount = res.stream().map(PayableBalanceDetail::getPayableAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
        //实付合计
        BigDecimal totalActualPay = res.stream().map(PayableBalanceDetail::getActualPay).reduce(BigDecimal.ZERO, BigDecimal::add);
        //期末余额合计
        BigDecimal totalEndingBalance = res.stream().map(PayableBalanceDetail::getEndingBalance).reduce(BigDecimal.ZERO, BigDecimal::add);
        int seq = 0;
        for (PayableBalanceDetail r : res) {
            r.setSeq(String.valueOf(++seq));
        }
        PayableBalanceVO payableBalanceVO = new PayableBalanceVO();
        payableBalanceVO.setTotalPayableAmount(totalPayableAmount);
        payableBalanceVO.setTotalActualPay(totalActualPay);
        payableBalanceVO.setTotalEndingBalance(totalEndingBalance);
        payableBalanceVO.setPayableBalanceList(res);

        return new R<>(param.getPageNumber(), param.getPageSize(), (int) page.getTotal(), payableBalanceVO);
    }
}
