package cn.superdesk.uniorder.controller._statistics;


import cn.superdesk.uniorder.common.exception.utils.ExceptionUtils;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dto.param.accounting.statistics.ReceivableStatisticsPageQueryParam;
import cn.superdesk.uniorder.dto.vo._statistics.ReceivableStatisticsPageVo;
import cn.superdesk.uniorder.dubbo.consumer.statistics.ReceivableStatisticsDubboService;
import cn.superdesk.uniorder.interceptor.logger.annotation.ResponseExcel;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.util.List;


/**
 * @Description: Java类作用描述
 * @Author: qzy
 * @CreateDate: 2021/12/2 下午3:49
 * @UpdateUser: qzy
 * @UpdateDate: 2021/12/2 下午3:49
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Api(value = "应收报表", tags = {"应收报表"})
@Slf4j
@RestController
@RequestMapping("/statistics/receivable")
public class ReceivableStatisticsController extends BaseController {
	@Autowired
	private ReceivableStatisticsDubboService receivableStatisticsDubboService;

	@ApiOperation(value = "应收报表", notes = "应收报表")
	@PostMapping("/getReceivableStatisticsPageVoList")
	@ResponseBody
	public R<List<ReceivableStatisticsPageVo>> getReceivableStatisticsPageVoList(ReceivableStatisticsPageQueryParam queryParam) {
		R<List<ReceivableStatisticsPageVo>> response = new R<>();
		try {
			response = receivableStatisticsDubboService.getReceivableStatisticsPageVoList(queryParam);
		} catch (Exception e) {
			response.setCode(String.valueOf(e.hashCode()));
			response.setSuccess(false);
			response.setMessage(e.getMessage());

			log.error(ExceptionUtils.getMessage(e));
		}

		return response;
	}

	/**
	 * @throws IOException
	 */
	@ResponseExcel
	@ApiOperation(value = "导出应收汇总报表", tags = "导出应收汇总报表")
	@RequestMapping(value = "/getReceivableStatisExcel", method = RequestMethod.POST)
	public R<String> getReceivableStatisExcel(ReceivableStatisticsPageQueryParam queryParam) throws IOException {
		return new R<>(receivableStatisticsDubboService.getReceivableStatisExcel(queryParam,getToken()));
	}
}
