package cn.superdesk.uniorder.controller._thirdpart.jd;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONUtil;
import cn.superdesk.libs.common.json.JsonUtils;
import cn.superdesk.uniorder.common.enums.thirdpart.jd.SysQuartzScheduleTaskStatementOfAccountIsPauseEnum;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.common.rest.response.ResponseResultOuterUtils;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dao.entity.thirdpart.jd.SysQuartzScheduleTaskStatementOfAccountJd;
import cn.superdesk.uniorder.dto.param.thirdpart.jd.*;
import cn.superdesk.uniorder.dto.vo.thirdpart.jd.*;
import cn.superdesk.uniorder.dubbo.consumer.thirdpart.jd.AccountingUnifiedSupplierOrderJdCheckUpReportDubboConsumerService;
import cn.superdesk.uniorder.dubbo.consumer.thirdpart.jd.SysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;


/**
 * @Description: 定时任务-时刻任务-京东对账单
 * @Author: linjinyu
 * @Email linjinyu@avic-s.com
 * @CreateDate: 2021-10-12 11:29:24
 * @UpdateUser: linjinyu
 * @UpdateDate: 2021-10-12 11:29:24
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Api(value = "供应商结算-京东对账", tags = {"供应商结算-京东对账"})
@Slf4j
@RestController
@RequestMapping("/sysQuartzScheduleTaskStatementOfAccountJd")
public class SysQuartzScheduleTaskStatementOfAccountJdController extends BaseController {

    @Autowired
    private SysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService;

    @Autowired
    private AccountingUnifiedSupplierOrderJdCheckUpReportDubboConsumerService accountingUnifiedSupplierOrderJdCheckUpReportDubboConsumerService;

    /**
     * http://192.168.6.223:8089/web/sysQuartzScheduleTaskStatementOfAccountJd/list
     *
     * @param queryCriteria
     * @return
     */
    @ApiOperation(value = "供应商结算-京东对账-op列表")
    @RequestMapping(value = "list", method = RequestMethod.POST)
    @ResponseBody
    public R<List<SysQuartzScheduleTaskStatementOfAccountJdListItemVo>> pageQueryGenericParamSysQuartzScheduleTaskStatementOfAccountJd(
            @RequestBody SysQuartzScheduleTaskStatementOfAccountJdQueryCriteria queryCriteria) {

        log.info("pageQuerySysQuartzScheduleTaskStatementOfAccountJd:\n{}", JsonUtils.toJson(queryCriteria));

        R<List<SysQuartzScheduleTaskStatementOfAccountJdListItemVo>> result = new R<>();

        result = sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService.pageQueryGenericParam(queryCriteria, getToken());

        return result;
    }

    @ApiOperation(value = "供应商结算-京东对账-op页头")
    @RequestMapping(value = "head", method = RequestMethod.POST)
    public R<SysQuartzScheduleTaskStatementOfAccountJdHeadVo> pageQueryHeadGenericParamSysQuartzScheduleTaskStatementOfAccountJd(
            @RequestBody SysQuartzScheduleTaskStatementOfAccountJdQueryCriteria queryCriteria) {

        return ResponseResultOuterUtils.createSuccesResult("",
                sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService
                        .queryHeadGenericParam(
                                queryCriteria,
                                getToken()
                        )
        );
    }


    @ApiOperation(value = "供应商结算-京东对账-op列表-op查看对账(对账详情)-带查询过滤")
    @RequestMapping(value = "/opDetail", method = RequestMethod.POST)
    public R<SysQuartzScheduleTaskStatementOfAccountJdDetailVo> opDetail(
            @RequestBody SysQuartzScheduleTaskStatementOfAccountJdOpDetailForm form) {

        log.info("供应商结算-京东对账-op列表-op查看对账 params : \n{}", JSONUtil.toJsonStr(form));


        R<SysQuartzScheduleTaskStatementOfAccountJdDetailVo> response = new R<>();

        if (ObjectUtil.isNull(form.getBatchId())) {
            return ResponseResultOuterUtils.createErrorResult("500", "[参数异常]对账单号");
        }

        // TODO 参数合法性判定

        /**************************************** head start ****************************************/

        List<SysQuartzScheduleTaskStatementOfAccountJdListItemVo> resultHead = sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService.queryGenericParam(
                SysQuartzScheduleTaskStatementOfAccountJdQueryCriteria
                        .builder()

                        .sysQuartzScheduleTaskStatementOfAccountJdId(form.getSysQuartzScheduleTaskStatementOfAccountJdId())

                        .batchId(form.getBatchId())

                        .build(),
                getToken()
        );

        if (CollUtil.isEmpty(resultHead)) {
            return ResponseResultOuterUtils.createErrorResult("500", "未查找到相关的信息");
        }

        SysQuartzScheduleTaskStatementOfAccountJdListItemVo head = resultHead.get(0);

        /**************************************** head   end ****************************************/

        /**************************************** statistics start ****************************************/

        AccountingUnifiedSupplierOrderJdCheckUpReportDetailStatisticsVo statistics = accountingUnifiedSupplierOrderJdCheckUpReportDubboConsumerService.pageQueryHeadGenericParam(
                AccountingUnifiedSupplierOrderJdCheckUpReportQueryCriteria
                        .builder()

                        .batchId(form.getBatchId())

                        // 要不要跟随条件呢? 先是跟随吧
                        /**************************************** 供应商结算-京东对账-op列表-op查看对账(对账详情) 页内查询过滤 start ****************************************/
                        .jdOrderSn(form.getJdOrderSn())
                        .jdOriginalOrderSn(form.getJdOriginalOrderSn())
                        .orderSn(form.getOrderSn())
                        .processResult(form.getProcessResult())
                        /**************************************** 供应商结算-京东对账-op列表-op查看对账(对账详情) 页内查询过滤 start ****************************************/

                        .build(),
                getToken()
        );

        statistics.setStatisticsAmountDimOrderDiff(
                statistics.getStatisticsAmountDimOrderOutsider()
                        .subtract(statistics.getStatisticsAmountDimOrderOwnsider())
        );


//        /**************************************** 修正 start ****************************************/
//
//        sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService.updateByPrimaryKeySelective(
//                SysQuartzScheduleTaskStatementOfAccountJd
//                        .builder()
//
//                        .id(head.getId())
//
//                        .statisticsAmountDimOrderOutsider(statistics.getStatisticsAmountDimOrderOutsider())
//                        .statisticsAmountDimOrderOwnsider(statistics.getStatisticsAmountDimOrderOwnsider())
//
//                        .build()
//        );
//
//        /**************************************** 修正 start ****************************************/


        /**************************************** statistics   end ****************************************/

        /**************************************** list start ****************************************/

        R<List<AccountingUnifiedSupplierOrderJdCheckUpReportListItemVo>> resultList = accountingUnifiedSupplierOrderJdCheckUpReportDubboConsumerService
                .pageQueryGenericParam(
                        AccountingUnifiedSupplierOrderJdCheckUpReportQueryCriteria
                                .builder()

                                .batchId(form.getBatchId())

                                /**************************************** 供应商结算-京东对账-op列表-op查看对账(对账详情) 页内查询过滤 start ****************************************/
                                .jdOrderSn(form.getJdOrderSn())
                                .jdOriginalOrderSn(form.getJdOriginalOrderSn())
                                .orderSn(form.getOrderSn())
                                .processResult(form.getProcessResult())
                                /**************************************** 供应商结算-京东对账-op列表-op查看对账(对账详情) 页内查询过滤 start ****************************************/

                                .pageNumber(form.getPageNumber())
                                .pageSize(form.getPageSize())

                                .build(),
                        getToken()
                );

        /**************************************** list   end ****************************************/


        response.setCode("200");
        response.setSuccess(true);
        response.setData(SysQuartzScheduleTaskStatementOfAccountJdDetailVo
                .builder()
                .head(head)
                .statistics(statistics)
                .list(resultList.getData())
                .build());
        response.setCount(resultList.getCount());
        response.setCurrentPage(resultList.getCurrentPage());
        response.setPageSize(resultList.getPageSize());

        return response;
    }

    @ApiOperation(value = "供应商结算-京东对账-op列表-op查看对账(对账详情)-带查询过滤-带狗")
    @RequestMapping(value = "/opDetail/woof", method = RequestMethod.POST)
    public R<SysQuartzScheduleTaskStatementOfAccountJdDetailVoWoof> opDetailWoof(
            @RequestBody SysQuartzScheduleTaskStatementOfAccountJdOpDetailForm form) {

        log.info("供应商结算-京东对账-op列表-op查看对账 params : \n{}", JSONUtil.toJsonStr(form));


        R<SysQuartzScheduleTaskStatementOfAccountJdDetailVoWoof> response = new R<>();

        if (ObjectUtil.isNull(form.getBatchId())) {
            return ResponseResultOuterUtils.createErrorResult("500", "[参数异常]对账单号");
        }

        // TODO 参数合法性判定

        /**************************************** head start ****************************************/

        List<SysQuartzScheduleTaskStatementOfAccountJdListItemVo> resultHead = sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService
                .queryGenericParam(
                        SysQuartzScheduleTaskStatementOfAccountJdQueryCriteria
                                .builder()

                                .sysQuartzScheduleTaskStatementOfAccountJdId(form.getSysQuartzScheduleTaskStatementOfAccountJdId())

                                .batchId(form.getBatchId())

                                .build(),
                        getToken()
                );

        if (CollUtil.isEmpty(resultHead)) {
            return ResponseResultOuterUtils.createErrorResult("500", "未查找到相关的信息");
        }

        SysQuartzScheduleTaskStatementOfAccountJdListItemVo head = resultHead.get(0);

        /**************************************** head   end ****************************************/

        /**************************************** statistics start ****************************************/

        AccountingUnifiedSupplierOrderJdCheckUpReportDetailStatisticsVo statistics = accountingUnifiedSupplierOrderJdCheckUpReportDubboConsumerService
                .pageQueryHeadGenericParam(
                        AccountingUnifiedSupplierOrderJdCheckUpReportQueryCriteria
                                .builder()

                                .batchId(form.getBatchId())

                                // 要不要跟随条件呢? 先是跟随吧
                                /**************************************** 供应商结算-京东对账-op列表-op查看对账(对账详情) 页内查询过滤 start ****************************************/
                                .jdOrderSn(form.getJdOrderSn())
                                .jdOriginalOrderSn(form.getJdOriginalOrderSn())
                                .orderSn(form.getOrderSn())
                                .processResult(form.getProcessResult())
                                /**************************************** 供应商结算-京东对账-op列表-op查看对账(对账详情) 页内查询过滤 start ****************************************/

                                .build(),
                        getToken()
                );

        statistics.setStatisticsAmountDimOrderDiff(
                statistics.getStatisticsAmountDimOrderOutsider()
                        .subtract(statistics.getStatisticsAmountDimOrderOwnsider())
        );


//        /**************************************** 修正 start ****************************************/
//
//        sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService.updateByPrimaryKeySelective(
//                SysQuartzScheduleTaskStatementOfAccountJd
//                        .builder()
//
//                        .id(head.getId())
//
//                        .statisticsAmountDimOrderOutsider(statistics.getStatisticsAmountDimOrderOutsider())
//                        .statisticsAmountDimOrderOwnsider(statistics.getStatisticsAmountDimOrderOwnsider())
//
//                        .build()
//        );
//
//        /**************************************** 修正 start ****************************************/


        /**************************************** statistics   end ****************************************/

        /**************************************** list start ****************************************/

        R<List<AccountingUnifiedSupplierOrderJdCheckUpReportListItemVoWoof>> resultList = accountingUnifiedSupplierOrderJdCheckUpReportDubboConsumerService
                .pageQueryGenericWoof(
                        AccountingUnifiedSupplierOrderJdCheckUpReportQueryCriteria
                                .builder()

                                .batchId(form.getBatchId())

                                /**************************************** 供应商结算-京东对账-op列表-op查看对账(对账详情) 页内查询过滤 start ****************************************/
                                .jdOrderSn(form.getJdOrderSn())
                                .jdOriginalOrderSn(form.getJdOriginalOrderSn())
                                .orderSn(form.getOrderSn())
                                .processResult(form.getProcessResult())
                                /**************************************** 供应商结算-京东对账-op列表-op查看对账(对账详情) 页内查询过滤 start ****************************************/

                                .pageNumber(form.getPageNumber())
                                .pageSize(form.getPageSize())

                                .build(),
                        getToken()
                );

        /**************************************** list   end ****************************************/


        response.setCode("200");
        response.setSuccess(true);
        response.setData(SysQuartzScheduleTaskStatementOfAccountJdDetailVoWoof
                .builder()
                .head(head)
                .statistics(statistics)
                .list(resultList.getData())
                .build());
        response.setCount(resultList.getCount());
        response.setCurrentPage(resultList.getCurrentPage());
        response.setPageSize(resultList.getPageSize());

        return response;
    }

    @ApiOperation(value = "供应商结算-京东对账-op列表-op修正统计")
    @RequestMapping(value = "/opFixStatistics", method = RequestMethod.POST)
    public R<String> opFixStatistics(
            @RequestBody SysQuartzScheduleTaskStatementOfAccountJdOpDetailForm form) {

        log.info("供应商结算-京东对账-op列表-op查看对账 params : \n{}", JSONUtil.toJsonStr(form));


        R<SysQuartzScheduleTaskStatementOfAccountJdDetailVo> response = new R<>();

        if (ObjectUtil.isNull(form.getBatchId())) {
            return ResponseResultOuterUtils.createErrorResult("500", "[参数异常]对账单号");
        }

        // TODO 参数合法性判定

        /**************************************** head start ****************************************/

        List<SysQuartzScheduleTaskStatementOfAccountJdListItemVo> resultHead = sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService.queryGenericParam(
                SysQuartzScheduleTaskStatementOfAccountJdQueryCriteria
                        .builder()

                        .sysQuartzScheduleTaskStatementOfAccountJdId(form.getSysQuartzScheduleTaskStatementOfAccountJdId())

                        .batchId(form.getBatchId())

                        .build(),
                getToken()
        );

        if (CollUtil.isEmpty(resultHead)) {
            return ResponseResultOuterUtils.createErrorResult("500", "未查找到相关的信息");
        }

        SysQuartzScheduleTaskStatementOfAccountJdListItemVo head = resultHead.get(0);

        /**************************************** head   end ****************************************/

        /**************************************** statistics start ****************************************/

        AccountingUnifiedSupplierOrderJdCheckUpReportDetailStatisticsVo statistics = accountingUnifiedSupplierOrderJdCheckUpReportDubboConsumerService.pageQueryHeadGenericParam(
                AccountingUnifiedSupplierOrderJdCheckUpReportQueryCriteria
                        .builder()

                        .batchId(form.getBatchId())

                        .build(),
                getToken()
        );

        statistics.setStatisticsAmountDimOrderDiff(
                statistics.getStatisticsAmountDimOrderOutsider()
                        .subtract(statistics.getStatisticsAmountDimOrderOwnsider())
        );


        /**************************************** 修正 start ****************************************/

        sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService.updateByPrimaryKeySelective(
                SysQuartzScheduleTaskStatementOfAccountJd
                        .builder()

                        .id(head.getId())

                        .statisticsAmountDimOrderOutsider(statistics.getStatisticsAmountDimOrderOutsider())
                        .statisticsAmountDimOrderOwnsider(statistics.getStatisticsAmountDimOrderOwnsider())

                        .build()
        );

        /**************************************** 修正 start ****************************************/


        /**************************************** statistics   end ****************************************/


        return ResponseResultOuterUtils.createSuccesResult();
    }

    @ApiOperation(value = "供应商结算-京东对账-op列表-op备注")
    @RequestMapping(value = "/opSetRemark", method = RequestMethod.POST)
    @ResponseBody
    public R<String> opSetRemark(@RequestBody SysQuartzScheduleTaskStatementOfAccountJdOpSetRemarkForm form) {

        log.info("供应商结算-京东对账-op列表-op备注 params : \n{}", JSONUtil.toJsonStr(form));

        int ret = sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService.opSetRemark(form, getToken());

        if (ret == 0) {
            return ResponseResultOuterUtils.createErrorResult("500", "失败");
        } else {
            return ResponseResultOuterUtils.createSuccesResult("成功");
        }
    }


    @ApiOperation(value = "供应商结算-京东对账-op列表-op设置导入任务为-自动模式")
    @RequestMapping(value = "/opSetImportTask/automatic", method = RequestMethod.POST)
    @ResponseBody
    public R<String> opSetImportTaskAutomatic(@RequestBody SysQuartzScheduleTaskStatementOfAccountJdOpSetIsPauseForm form) {

        form.setIsPauseEnum(SysQuartzScheduleTaskStatementOfAccountIsPauseEnum.启用);

        log.info("供应商结算-京东对账-op列表-op设置导入任务为自动模式 params : \n{}", JSONUtil.toJsonStr(form));

        int ret = sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService.opSetIsPause(form, getToken());

        if (ret == 0) {
            return ResponseResultOuterUtils.createErrorResult("500", "失败");
        } else {
            return ResponseResultOuterUtils.createSuccesResult("成功");
        }
    }

    @ApiOperation(value = "供应商结算-京东对账-op列表-op设置导入任务为-手动模式")
    @RequestMapping(value = "/opSetImportTask/manual", method = RequestMethod.POST)
    @ResponseBody
    public R<String> opSetImportTaskManual(@RequestBody SysQuartzScheduleTaskStatementOfAccountJdOpSetIsPauseForm form) {

        form.setIsPauseEnum(SysQuartzScheduleTaskStatementOfAccountIsPauseEnum.暂停);

        log.info("供应商结算-京东对账-op列表-op设置导入任务为手动模式 params : \n{}", JSONUtil.toJsonStr(form));

        int ret = sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService.opSetIsPause(form, getToken());

        if (ret == 0) {
            return ResponseResultOuterUtils.createErrorResult("500", "失败");
        } else {
            return ResponseResultOuterUtils.createSuccesResult("成功");
        }
    }


    @Deprecated
    @ApiOperation(value = "按id查询")
    @RequestMapping(value = "{id}", method = RequestMethod.GET)
    @ResponseBody
    public R<SysQuartzScheduleTaskStatementOfAccountJdListItemVo> getById(@PathVariable("id") long id) {

        // TODO 参数合法性判定

        return ResponseResultOuterUtils.createSuccesResult("",
                sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService
                        .findListItemVoById(
                                id,
                                getToken()
                        )
        );

    }

    @Deprecated
    @ApiOperation(value = "新增")
    @RequestMapping(value = "add", method = RequestMethod.POST)
    @ResponseBody
    public R<String> addSysQuartzScheduleTaskStatementOfAccountJd(@RequestBody SysQuartzScheduleTaskStatementOfAccountJdParam param) {

        log.info("addSysQuartzScheduleTaskStatementOfAccountJd:\n{}", JsonUtils.toJson(param));

        // TODO 参数合法性判定

        int ret = sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService.addSysQuartzScheduleTaskStatementOfAccountJd(param, getToken());

        if (ret == 0) {
            return ResponseResultOuterUtils.createErrorResult("500", "操作失败");
        } else {
            return ResponseResultOuterUtils.createSuccesResult("操作成功");
        }
    }

    @Deprecated
    @ApiOperation(value = "更新")
    @RequestMapping(value = "update", method = RequestMethod.POST)
    @ResponseBody
    public R<String> updateSysQuartzScheduleTaskStatementOfAccountJd(@RequestBody SysQuartzScheduleTaskStatementOfAccountJdParam param) {

        log.info("updateSysQuartzScheduleTaskStatementOfAccountJd:\n{}", JsonUtils.toJson(param));

        int ret = sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService.updateSysQuartzScheduleTaskStatementOfAccountJd(param, getToken());

        if (ret == 0) {
            return ResponseResultOuterUtils.createErrorResult("500", "操作失败");
        } else {
            return ResponseResultOuterUtils.createSuccesResult("操作成功");
        }
    }


    @Deprecated
    @ApiOperation(value = "供应商结算-京东对账-op列表-op批量删除任务")
    @RequestMapping(value = "delete_ids", method = RequestMethod.POST)
    @ResponseBody
    public R<String> deleteIdsSysQuartzScheduleTaskStatementOfAccountJd(@RequestParam(value = "ids") long[] ids) {

        log.info(JsonUtils.toJson(ids));

        int ret = 0;

        for (long id : ids) {
            ret = sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService.deleteSysQuartzScheduleTaskStatementOfAccountJd(id, getToken());
        }

        if (ret == 0) {
            return ResponseResultOuterUtils.createErrorResult("500", "操作失败");
        } else {
            return ResponseResultOuterUtils.createSuccesResult("操作成功");
        }

    }


}
