package cn.superdesk.uniorder.controller._thirdpart.jd;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import cn.superdesk.uniorder.common.enums.accounting.supplier.AccountingOrderSupplierAccountingStatusEnum;
import cn.superdesk.uniorder.common.enums.scheduler.jd.ScheduleTaskAccountingStatusEnum;
import cn.superdesk.uniorder.common.enums.scheduler.jd.ScheduleTaskCheckStateEnum;
import cn.superdesk.uniorder.common.enums.scheduler.jd.ScheduleTaskProcessStateEnum;
import cn.superdesk.uniorder.common.enums.scheduler.jd.ScheduleTaskRunStatusEnum;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.common.rest.response.ResponseResultOuterUtils;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dto.param.thirdpart.jd.SysQuartzScheduleTaskStatementOfAccountJdOpCancelForm;
import cn.superdesk.uniorder.dto.param.thirdpart.jd.SysQuartzScheduleTaskStatementOfAccountJdOpConfirmForm;
import cn.superdesk.uniorder.dto.param.thirdpart.jd.SysQuartzScheduleTaskStatementOfAccountJdOpDeleteForm;
import cn.superdesk.uniorder.dto.param.thirdpart.jd.SysQuartzScheduleTaskStatementOfAccountJdQueryCriteria;
import cn.superdesk.uniorder.dto.vo.thirdpart.jd.SysQuartzScheduleTaskStatementOfAccountJdListItemVo;
import cn.superdesk.uniorder.dubbo.consumer.thirdpart.jd.SysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService;
import cn.superdesk.uniorder.dubbo.consumer.thirdpart.jd.SysQuartzScheduleTaskStatementOfAccountJdExOpPartDubboConsumerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;


/**
 * @Description: 供应商结算-京东对账-op列表-操作部分
 * mark by linjinyu at 2022-06-167 11:54:53 - 目前这些操作 可以想象为外壳
 * 供应商结算-京东对账-op列表-op删除任务{@link #opDelete(SysQuartzScheduleTaskStatementOfAccountJdOpDeleteForm)} -> 京东对账层面逻辑 ~ 清理销定 ~ 删除(子数据(导入数据)子数据(比对数据)面数据(京东对账任务本身))
 * 供应商结算-京东对账-op列表-op确认账单{@link #opConfirm(SysQuartzScheduleTaskStatementOfAccountJdOpConfirmForm)} -> 京东对账层面逻辑 -> 供应商结算-op创建对账单 -> 京东对账层面逻辑
 * 供应商结算-京东对账-op列表-op取消账单{@link #opCancel(SysQuartzScheduleTaskStatementOfAccountJdOpCancelForm)} -> 京东对账层面逻辑 ~
 * @Author: linjinyu
 * @CreateDate: 10/12/20 4:28 PM
 * @UpdateUser: linjinyu
 * @UpdateDate: 10/12/20 4:28 PM
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Api(value = "供应商结算-京东对账-op列表-操作部分", tags = {"供应商结算-京东对账-op列表-操作部分"})
@Slf4j
@RestController
@RequestMapping("/sysQuartzScheduleTaskStatementOfAccountJd/opPart")
public class SysQuartzScheduleTaskStatementOfAccountJdExOpPartController extends BaseController {

    @Autowired
    private SysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService         sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService;
    @Autowired
    private SysQuartzScheduleTaskStatementOfAccountJdExOpPartDubboConsumerService sysQuartzScheduleTaskStatementOfAccountJdExOpPartDubboConsumerService;


    /**************************************** 供应商结算-京东对账-op列表-操作部分 start ****************************************/


    @ApiOperation(value = "供应商结算-京东对账-op列表-op删除任务")
    @RequestMapping(value = "opDelete", method = RequestMethod.POST)
    @ResponseBody
    public R<String> opDelete(@RequestBody SysQuartzScheduleTaskStatementOfAccountJdOpDeleteForm form) {


        // TODO 参数合法性判定

        // 该对账任务存在，若不存在，1.5s提示`该对账任务不存在，请返回检查`； ----

        /**************************************** 逻辑判定1 start ****************************************/


        List<SysQuartzScheduleTaskStatementOfAccountJdListItemVo> resultList = sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService
                .queryGenericParam(
                        SysQuartzScheduleTaskStatementOfAccountJdQueryCriteria
                                .builder()

                                .sysQuartzScheduleTaskStatementOfAccountJdId(form.getSysQuartzScheduleTaskStatementOfAccountJdId())

                                .build(),
                        getToken()
                );

        if (CollUtil.isEmpty(resultList)) {
            return ResponseResultOuterUtils.createErrorResult("500", "该对账任务不存在，请返回检查");
        }

        SysQuartzScheduleTaskStatementOfAccountJdListItemVo head = resultList.get(0);

        form.setBatchId(head.getBatchId());

        log.info("供应商结算-京东对账-op列表-op删除任务 opDelete\n{}\n{}\n运行步骤 -> {}\n导入状态 -> {}\n对账状态 -> {}\n对账结果 -> {}",
                JSONUtil.toJsonStr(form),
                JSONUtil.toJsonStr(head),
                ScheduleTaskRunStatusEnum.getDesc(head.getRunState()),
                ScheduleTaskProcessStateEnum.getDesc(head.getProcessState()),
                ScheduleTaskAccountingStatusEnum.getDesc(head.getAccountingStatus()),
                ScheduleTaskCheckStateEnum.getDesc(head.getCheckState())
        );

        /**************************************** 逻辑判定1   end ****************************************/

        /**************************************** 逻辑判定-插入-运行中判定 start ****************************************/

        Integer[] scheduleTaskRunStatusEnumContains = {
                ScheduleTaskRunStatusEnum.导入中.getKey(), // ---- 时间较长
                ScheduleTaskRunStatusEnum.对账中.getKey(), // ---- 时间很长++++ 按业务需求推，光从订单角度是不行的，得细到商品
                ScheduleTaskRunStatusEnum.确认中.getKey()  // ---- 时间较短
        };
        if (ArrayUtil.contains(
                scheduleTaskRunStatusEnumContains,
                head.getRunState())
        ) {
            return ResponseResultOuterUtils.createErrorResult("500", "该对账任务正在执行中，请稍后再试");
        }

        /**************************************** 逻辑判定-插入-运行中判定   end ****************************************/

        // 该对账任务的导入状态为`导入失败`或对账结果为`异常`或对账单状态为`已取消`时，满足其中一个即允许确认删除，否则1.5s提示`只允许删除`导入失败/异常/已取消的对账任务`，请返回检查`；
        // 该对账任务的
        // 导入状态为`导入失败`或
        // 对账结果为`异常`或
        // 对账状态为`已取消`时，满足其中一个即允许确认删除，否则1.5s提示`只允许删除`导入失败/异常/已取消的对账任务`，请返回检查`；

        /**************************************** 逻辑判定2 start ****************************************/

        int countCheckPoint = 0;

        // 导入状态 start ****************************************
        Integer[] scheduleTaskProcessStateEnumContains = {
                ScheduleTaskProcessStateEnum.导入失败.getKey(),
        };
        if (ArrayUtil.contains(
                scheduleTaskProcessStateEnumContains,
                head.getProcessState())
        ) {
            log.info("导入状态 | 导入失败 ++ ");
            countCheckPoint++;
        }
        // 导入状态   end ****************************************

        // 对账状态 start ****************************************
        Integer[] scheduleTaskAccountingStatusEnumContains = {
                ScheduleTaskAccountingStatusEnum.已取消.getKey(),
        };
        if (ArrayUtil.contains(
                scheduleTaskAccountingStatusEnumContains,
                head.getAccountingStatus())
        ) {
            log.info("对账状态 | 已取消;对账失败; ++ ");
            countCheckPoint++;
        }
        // 对账状态   end ****************************************

        // 对账结果 start ****************************************
        Integer[] scheduleTaskCheckStateEnumContains = {
                ScheduleTaskCheckStateEnum.待对账.getKey(),
                ScheduleTaskCheckStateEnum.异常.getKey()
        };
        if (ArrayUtil.contains(
                scheduleTaskCheckStateEnumContains,
                head.getCheckState())
        ) {
            log.info("对账结果 | 异常 ++ ");
            countCheckPoint++;
        }
        // 对账结果   end ****************************************

        // 对账单状态 start ****************************************
        Integer[] accountingOrderSupplierAccountingStatusEnumContains = {
                AccountingOrderSupplierAccountingStatusEnum.已取消.getKey(),
        };
        if (ArrayUtil.contains(
                accountingOrderSupplierAccountingStatusEnumContains,
                head.getAusuAccountingStatus())
        ) {
            countCheckPoint++;
        }
        // 对账单状态   end ****************************************

        if (countCheckPoint == 0) {
            return ResponseResultOuterUtils.createErrorResult("500", "只允许删除`导入失败/已取消/异常`的对账任务，请返回检查");
        }

        /**************************************** 逻辑判定2   end ****************************************/


        int ret = sysQuartzScheduleTaskStatementOfAccountJdExOpPartDubboConsumerService.opDelete(form, getToken());

        if (ret == 0) {
            return ResponseResultOuterUtils.createErrorResult("500", "删除任务失败");
        } else {
            return ResponseResultOuterUtils.createSuccesResult("删除任务成功");
        }

    }

    @ApiOperation(value = "供应商结算-京东对账-op列表-op确认账单")
    @RequestMapping(value = "/opConfirm", method = RequestMethod.POST)
    @ResponseBody
    public R<String> opConfirm(@RequestBody SysQuartzScheduleTaskStatementOfAccountJdOpConfirmForm form) {

        // TODO 参数合法性判定

        // 该对账任务存在，若不存在，1.5s提示`该对账任务不存在，请返回检查`； ----

        /**************************************** 逻辑判定1 start ****************************************/


        List<SysQuartzScheduleTaskStatementOfAccountJdListItemVo> resultList = sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService
                .queryGenericParam(
                        SysQuartzScheduleTaskStatementOfAccountJdQueryCriteria
                                .builder()

                                .sysQuartzScheduleTaskStatementOfAccountJdId(form.getSysQuartzScheduleTaskStatementOfAccountJdId())

                                .build(),
                        getToken()
                );

        if (CollUtil.isEmpty(resultList)) {
            return ResponseResultOuterUtils.createErrorResult("500", "该对账任务不存在，请返回检查");
        }

        SysQuartzScheduleTaskStatementOfAccountJdListItemVo head = resultList.get(0);

        form.setBatchId(head.getBatchId());

        log.info("供应商结算-京东对账-op列表-op删除任务 opDelete\n{}\n{}\n运行步骤 -> {}\n导入状态 -> {}\n对账状态 -> {}\n对账结果 -> {}",
                JSONUtil.toJsonStr(form),
                JSONUtil.toJsonStr(head),
                ScheduleTaskRunStatusEnum.getDesc(head.getRunState()),
                ScheduleTaskProcessStateEnum.getDesc(head.getProcessState()),
                ScheduleTaskAccountingStatusEnum.getDesc(head.getAccountingStatus()),
                ScheduleTaskCheckStateEnum.getDesc(head.getCheckState())
        );

        /**************************************** 逻辑判定1   end ****************************************/

        /**************************************** 逻辑判定-插入-运行中判定 start ****************************************/

        Integer[] scheduleTaskRunStatusEnumContains = {
                ScheduleTaskRunStatusEnum.导入中.getKey(), // ---- 时间较长
                ScheduleTaskRunStatusEnum.对账中.getKey(), // ---- 时间很长++++ 按业务需求推，光从订单角度是不行的，得细到商品
                ScheduleTaskRunStatusEnum.确认中.getKey(), // ---- 时间较短
        };
        if (ArrayUtil.contains(
                scheduleTaskRunStatusEnumContains,
                head.getRunState())
        ) {
            return ResponseResultOuterUtils.createErrorResult("500", "该对账任务正在执行中，请稍后再试");
        }

        /**************************************** 逻辑判定-插入-运行中判定   end ****************************************/


        /**************************************** 逻辑判定2 start ****************************************/

        // 该对账任务的对账状态为`对账完成`，若非`对账完成`，1.5s提示`该对账任务未对账完成，请返回检查`；
        // 对账状态 start ****************************************
        Integer[] scheduleTaskAccountingStatusEnumContains = {
                ScheduleTaskAccountingStatusEnum.对账完成.getKey(),
        };
        if (!ArrayUtil.contains(
                scheduleTaskAccountingStatusEnumContains,
                head.getAccountingStatus())
        ) {
            return ResponseResultOuterUtils.createErrorResult("500", "该对账任务未对账完成，请返回检查");
        }
        // 对账状态   end ****************************************

        // 该对账任务的对账结果为`正常`，若非`正常`，1.5s提示`该对账任务对账结果非正常，请返回检查`；
        // 对账结果 start ****************************************
        Integer[] scheduleTaskCheckStateEnumContains = {
                ScheduleTaskCheckStateEnum.正常.getKey(),
        };
        if (!ArrayUtil.contains(
                scheduleTaskCheckStateEnumContains,
                head.getCheckState())
        ) {
            return ResponseResultOuterUtils.createErrorResult("500", "该对账任务对账结果非正常，请返回检查");
        }
        // 对账结果   end ****************************************

        // 该对账任务的对账单状态为`未确认`，若非`未确认`，1.5s提示`该对账任务已创建对账单，请返回检查`；
        // 对账单状态 start ****************************************
        Integer[] accountingOrderSupplierAccountingStatusEnumContains = {
                AccountingOrderSupplierAccountingStatusEnum.未确认.getKey(),
        };
        if (!ArrayUtil.contains(
                accountingOrderSupplierAccountingStatusEnumContains,
                head.getAusuAccountingStatus())
        ) {
            return ResponseResultOuterUtils.createErrorResult("500", "该对账任务已创建对账单，请返回检查");
        }
        // 对账单状态   end ****************************************

        // 该对账任务的结算中心账单编号为空，若非空，1.5s提示`该对账任务已创建对账单，请返回检查`；
        if (StrUtil.isNotBlank(head.getAccountingOrderSn())) {
            return ResponseResultOuterUtils.createErrorResult("500", "该对账任务已创建对账单，请返回检查");
        }


        /**************************************** 逻辑判定2   end ****************************************/


        int ret = sysQuartzScheduleTaskStatementOfAccountJdExOpPartDubboConsumerService.opConfirm(form, getToken());

        if (ret == 0) {
            return ResponseResultOuterUtils.createErrorResult("500", "确认账单失败");
        } else {
            return ResponseResultOuterUtils.createSuccesResult("确认账单成功");
        }

    }

    @ApiOperation(value = "供应商结算-京东对账-op列表-op取消账单")
    @RequestMapping(value = "/opCancel", method = RequestMethod.POST)
    @ResponseBody
    public R<String> opCancel(@RequestBody SysQuartzScheduleTaskStatementOfAccountJdOpCancelForm form) {

        // TODO 参数合法性判定

        // 该对账任务存在，若不存在，1.5s提示`该对账任务不存在，请返回检查`； ----

        /**************************************** 逻辑判定1 start ****************************************/

        List<SysQuartzScheduleTaskStatementOfAccountJdListItemVo> resultList = sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService
                .queryGenericParam(
                        SysQuartzScheduleTaskStatementOfAccountJdQueryCriteria
                                .builder()

                                .sysQuartzScheduleTaskStatementOfAccountJdId(form.getSysQuartzScheduleTaskStatementOfAccountJdId())

                                .build(),
                        getToken()
                );

        if (CollUtil.isEmpty(resultList)) {
            return ResponseResultOuterUtils.createErrorResult("500", "该对账任务不存在，请返回检查");
        }

        SysQuartzScheduleTaskStatementOfAccountJdListItemVo head = resultList.get(0);

        form.setBatchId(head.getBatchId());
        log.info("供应商结算-京东对账-op列表-op删除任务 opDelete\n{}\n{}\n运行步骤 -> {}\n导入状态 -> {}\n对账状态 -> {}\n对账结果 -> {}",
                JSONUtil.toJsonStr(form),
                JSONUtil.toJsonStr(head),
                ScheduleTaskRunStatusEnum.getDesc(head.getRunState()),
                ScheduleTaskProcessStateEnum.getDesc(head.getProcessState()),
                ScheduleTaskAccountingStatusEnum.getDesc(head.getAccountingStatus()),
                ScheduleTaskCheckStateEnum.getDesc(head.getCheckState())
        );

        /**************************************** 逻辑判定1   end ****************************************/

        /**************************************** 逻辑判定-插入-运行中判定 start ****************************************/

        Integer[] scheduleTaskRunStatusEnumContains = {
                ScheduleTaskRunStatusEnum.导入中.getKey(), // ---- 时间较长
                ScheduleTaskRunStatusEnum.对账中.getKey(), // ---- 时间很长++++ 按业务需求推，光从订单角度是不行的，得细到商品
                ScheduleTaskRunStatusEnum.确认中.getKey(), // ---- 时间较短
        };
        if (ArrayUtil.contains(
                scheduleTaskRunStatusEnumContains,
                head.getRunState())
        ) {
            return ResponseResultOuterUtils.createErrorResult("500", "该对账任务正在执行中，请稍后再试");
        }

        /**************************************** 逻辑判定-插入-运行中判定   end ****************************************/


        /**************************************** 逻辑判定2 start ****************************************/

        // 该对账任务的对账状态为`对账完成`，若非`对账完成`，1.5s提示`该对账任务未对账完成，请返回检查`；
        // 对账结果 start ****************************************
        Integer[] scheduleTaskAccountingStatusEnumContains = {
                ScheduleTaskAccountingStatusEnum.对账完成.getKey(),
        };
        if (!ArrayUtil.contains(
                scheduleTaskAccountingStatusEnumContains,
                head.getAccountingStatus())
        ) {
            return ResponseResultOuterUtils.createErrorResult("500", "该对账任务未对账完成，请返回检查");
        }
        // 对账结果   end ****************************************

        // 该对账任务的对账结果为`正常`，若非`正常`，1.5s提示`该对账任务对账结果非正常，请返回检查`；
        // 对账结果 start ****************************************
        Integer[] scheduleTaskCheckStateEnumContains = {
                ScheduleTaskCheckStateEnum.正常.getKey(),
        };
        if (!ArrayUtil.contains(
                scheduleTaskCheckStateEnumContains,
                head.getCheckState())
        ) {
            return ResponseResultOuterUtils.createErrorResult("500", "该对账任务对账结果非正常，请返回检查");
        }
        // 对账结果   end ****************************************


        // 该对账任务的结算中心账单编号为空，若非空，1.5s提示`该对账任务已创建对账单，请返回检查`；
        if (StrUtil.isNotBlank(head.getAccountingOrderSn())) {
            return ResponseResultOuterUtils.createErrorResult("500", "该对账任务已创建对账单，请返回检查");
        }

        // 该对账任务的对账单状态为`未确认`，若非`未确认`，1.5s提示`该对账任务已创建对账单，请返回检查`；
        // 对账单状态 start ****************************************
        Integer[] accountingOrderSupplierAccountingStatusEnumContains = {
                AccountingOrderSupplierAccountingStatusEnum.未确认.getKey(),
                AccountingOrderSupplierAccountingStatusEnum.已取消.getKey(),
        };
        if (!ArrayUtil.contains(
                accountingOrderSupplierAccountingStatusEnumContains,
                head.getAusuAccountingStatus())
        ) {
            return ResponseResultOuterUtils.createErrorResult("500", "该对账任务未对账完成，请返回检查");
        }
        // 对账单状态   end ****************************************


        /**************************************** 逻辑判定2   end ****************************************/


        int ret = sysQuartzScheduleTaskStatementOfAccountJdExOpPartDubboConsumerService.opCancel(form, getToken());

        if (ret == 0) {
            return ResponseResultOuterUtils.createErrorResult("500", "取消账单失败");
        } else {
            return ResponseResultOuterUtils.createSuccesResult("取消账单成功");
        }
    }

    /**************************************** 供应商结算-京东对账-op列表-操作部分   end ****************************************/


}
