package cn.superdesk.uniorder.controller._thirdpart.jd;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.lang.UUID;
import cn.superdesk.uniorder.common.enums.accounting.AccountingOrderDimCreateByEnum;
import cn.superdesk.uniorder.common.enums.scheduler.jd.ScheduleTaskAccountingStatusEnum;
import cn.superdesk.uniorder.common.enums.scheduler.jd.ScheduleTaskCheckStateEnum;
import cn.superdesk.uniorder.common.enums.scheduler.jd.ScheduleTaskProcessStateEnum;
import cn.superdesk.uniorder.common.enums.scheduler.jd.ScheduleTaskRunStatusEnum;
import cn.superdesk.uniorder.common.enums.thirdpart.jd.SysQuartzScheduleTaskStatementOfAccountIsPauseEnum;
import cn.superdesk.uniorder.common.exception.UnifiedOrderRuntimeException;
import cn.superdesk.uniorder.common.exception.constants.ExceptionMsg;
import cn.superdesk.uniorder.common.json.AnJsonUtils;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.common.rest.response.ResponseResultOuterUtils;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dto._constant.ResourceConstant;
import cn.superdesk.uniorder.dto.param.thirdpart.jd.SysQuartzScheduleTaskStatementOfAccountJdParam;
import cn.superdesk.uniorder.dto.param.thirdpart.jd.SysQuartzScheduleTaskStatementOfAccountJdQueryCriteria;
import cn.superdesk.uniorder.dto.param.thirdpart.jd.SysQuartzScheduleTaskStatementOfAccountJdRunExcelImportForm;
import cn.superdesk.uniorder.dto.vo.thirdpart.jd.AccountingUnifiedSupplierOrderJdCheckUpReportSnapshotStruct;
import cn.superdesk.uniorder.dto.vo.thirdpart.jd.SysQuartzScheduleTaskStatementOfAccountJdListItemVo;
import cn.superdesk.uniorder.dubbo.consumer._common.ProviderEnvConstantDubboCustomerService;
import cn.superdesk.uniorder.dubbo.consumer.thirdpart.jd.AccountingUnifiedSupplierOrderJdCheckUpReportDubboConsumerService;
import cn.superdesk.uniorder.dubbo.consumer.thirdpart.jd.SysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.File;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.List;
import java.util.Objects;


/**
 * @Description: 第三方-szhtxx-深圳航天信息有限公司-e6软件
 * @Author: linjinyu
 * @CreateDate: 2021/3/17 上午9:45
 * @UpdateUser: linjinyu
 * @UpdateDate: 2021/3/17 上午9:45
 * @UpdateRemark: 修改内容
 * @Version: 1.0
 */
@Api(value = "供应商结算-京东对账-步骤", tags = {"供应商结算-京东对账-步骤"})
@Slf4j
@RestController
@RequestMapping("/thirdpart/jd/reconciliation_schedule/")
public class ThirdPartJdReconciliationScheduleController extends BaseController {

    @Autowired
    private SysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService;

    @Autowired
    private AccountingUnifiedSupplierOrderJdCheckUpReportDubboConsumerService accountingUnifiedSupplierOrderJdCheckUpReportDubboConsumerService;

    @Resource
    private ProviderEnvConstantDubboCustomerService providerEnvConstantDubboCustomerService;

    @Resource
    private Snowflake snowflake;

    @ApiOperation(value = "供应商结算-京东对账-op列表-(能否导入)op导入账单(上传excel与表单)-检查队列是否有任务")
    @RequestMapping(value = "checkIn/task", method = RequestMethod.POST)
    @ResponseBody
    public R<SysQuartzScheduleTaskStatementOfAccountJdListItemVo> checkIn() {
        return ResponseResultOuterUtils.createSuccesResult(
                "检查队列是否有任务-data为``代表没有",
                sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService
                        .deQueueOneImportExcel(
                                getToken()
                        )
        );
    }

    /**
     * http://192.168.6.223:8089/web/thirdpart/jd/reconciliation_schedule/store/excel/and/create/task
     *
     * @param multipartFile
     * @return
     * @throws Exception
     */
    @ApiOperation(value = "供应商结算-京东对账-op列表-op导入账单(上传excel与表单)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "statementOfAccountType", value = "账单类型:京东 对账单类型 1-客户结算;2-供应商结算;21-供应商结算-京东; 所以传21", paramType = "form", required = true, dataType = "Integer"),

            @ApiImplicitParam(name = "saleId", value = "供应商Id(外部)", paramType = "form", required = true, dataType = "Long"),

            @ApiImplicitParam(name = "excelFile", value = "导入账单", paramType = "form", required = true, dataType = "MultipartFile"),

            @ApiImplicitParam(name = "cycleBeginTime", value = "账单周期 日期范围 开始日期 eg 2021-10-01", paramType = "form", required = true, dataType = "String"),
            @ApiImplicitParam(name = "cycleEndTime", value = "账单周期 日期范围 开始日期 eg 2021-10-31", paramType = "form", required = true, dataType = "String"),

            @ApiImplicitParam(name = "thirdPartAccountingOrderSn", value = "账单编号(第三方)", paramType = "form", required = true, dataType = "String"),
            @ApiImplicitParam(name = "remark", value = "备注", paramType = "form", required = false, dataType = "String")
    })
    @RequestMapping(value = "store/excel/and/create/task", method = RequestMethod.POST)
    @ResponseBody
    public R<SysQuartzScheduleTaskStatementOfAccountJdListItemVo> excelImportAndStore(
            @RequestParam(value = "statementOfAccountType") Integer statementOfAccountType,

            @RequestParam(value = "saleId") Long saleId,

            @RequestParam(value = "excelFile") MultipartFile multipartFile,

            @RequestParam(value = "cycleBeginTime") String cycleBeginTimeStr,
            @RequestParam(value = "cycleEndTime") String cycleEndTimeStr,

            @RequestParam(value = "thirdPartAccountingOrderSn") String thirdPartAccountingOrderSn,
            @RequestParam(value = "remark") String remark,

            @RequestParam(value = "dateFormat", defaultValue = "yyyy-MM-dd HH:mm:ss") String dateFormat
    ) throws Exception {
//		yyyy-MM-dd
//		yyyy-MM-dd HH:mm:ss
//		2021-08-01 00:00:00
//		2021-08-31 23:59:59
        Date cycleBeginTime = DateUtil.parse(cycleBeginTimeStr + " " + "00:00:00", dateFormat);
        Date cycleEndTime   = DateUtil.parse(cycleEndTimeStr + " " + "23:59:59", dateFormat);

//		cycleEndTime = DateUtil.endOfDay(cycleEndTime);

        log.info("\n{}\n{}\n{}\n{}", cycleBeginTimeStr, cycleEndTimeStr, cycleBeginTime, cycleEndTime);


        ResourceConstant excelExportConstant = providerEnvConstantDubboCustomerService.getResourceConstant();

        String excelExportPathRoot             = excelExportConstant.getPathRoot();
        String excelExportRelativePathTemplate = excelExportConstant.getPathTemplate();
        String excelExportRelativePathStorage  = excelExportConstant.getPathStorage();

        String relativePath = File.separator + "upload" +
                File.separator + "thirdpart" +
                File.separator + "jd" +
                File.separator + "import" +
                File.separator + "reconciliation_schedule" +
                File.separator;

        String currentDate = LocalDate.now().format(DateTimeFormatter.ofPattern("yyyyMMdd"));

        String targetFullPath = excelExportPathRoot + relativePath + currentDate + File.separator;

        if (!FileUtil.exist(targetFullPath)) {
            FileUtil.mkdir(targetFullPath);
        }

        String originalFilename = multipartFile.getOriginalFilename();
        String fileName         = UUID.randomUUID().toString() + "@" + multipartFile.getOriginalFilename();
        String suffix           = Objects.requireNonNull(multipartFile.getOriginalFilename()).substring(multipartFile.getOriginalFilename().lastIndexOf(".") + 1);

        String targetRelativePathAndFileName = relativePath + currentDate + File.separator + fileName;

        File file1 = FileUtil.writeBytes(multipartFile.getBytes(), targetFullPath + fileName);

        log.info("\n{}\n{}\n{}\n{}", originalFilename, fileName, suffix, targetRelativePathAndFileName);

        if (!fileName.matches("^.+\\.(?i)(xls)$") && !fileName.matches("^.+\\.(?i)(xlsx)$")) {
            throw new UnifiedOrderRuntimeException(ExceptionMsg.SYS_ERROR.getCode(), "上传文件格式不正确");
        }

        boolean isExcel2003 = true;
        if (fileName.matches("^.+\\.(?i)(xlsx)$")) {
            isExcel2003 = false;
        }

        // TODO 添加任务


        String batchId = snowflake.nextIdStr();

        SysQuartzScheduleTaskStatementOfAccountJdParam form = SysQuartzScheduleTaskStatementOfAccountJdParam
                .builder()
                .batchId(batchId)

                // 此不是说来源是商品维度,而是说用商品维度去产生对账单 订单维度只有老版本才用
                .dimCreateBy(AccountingOrderDimCreateByEnum.商品.getKey())

                .statementOfAccountName(originalFilename)
                // 对账单类型 1-客户结算;2-供应商结算;21-供应商结算-京东; 引用 enum SysQuartzScheduleTaskStatementOfAccountStatementOfAccountTypeEnum
                .statementOfAccountType(statementOfAccountType)
                .thirdPartAccountingOrderSn(thirdPartAccountingOrderSn)
                .remark(remark)

                // .serviceNo() // 里边的 供应商处理过程会填充

                .sysOrgId(0L)
                .sysVirtId(0L)
                .outOrgId(0L)
                .outVirtId(0L)

                .sysMerchantId(0L)
                .outMerchantId(saleId)

                .cycleBeginTime(cycleBeginTime)
                .cycleEndTime(cycleEndTime)

                .isPause(SysQuartzScheduleTaskStatementOfAccountIsPauseEnum.暂停.getKey())
                .jobId(0L)

                .runState(ScheduleTaskRunStatusEnum.待导入.getKey())
                .runTime(DateUtil.date())
                .processState(ScheduleTaskProcessStateEnum.导入中.getKey())
                // .processTime()
                .accountingStatus(ScheduleTaskAccountingStatusEnum.待对账.getKey())
                // .accountingTime()
                .checkState(ScheduleTaskCheckStateEnum.待对账.getKey())
                // .checkTime()

                // .finishTime()
                // .accountingOrderSn()
                .params(AnJsonUtils.toJson(SysQuartzScheduleTaskStatementOfAccountJdRunExcelImportForm
                        .builder()
                        .batchId(batchId)
                        .excelExportPathRoot(excelExportPathRoot)
                        .excelExportRelativePathTemplate(excelExportRelativePathTemplate)
                        .excelExportRelativePathStorage(excelExportRelativePathStorage)
                        .originalFilename(originalFilename)
                        .fileName(fileName)
                        .suffix(suffix)
                        .relativePath(relativePath)
                        .currentDate(currentDate)
                        .targetRelativePathAndFileName(targetRelativePathAndFileName)
                        .build()
                ))
                // .result()
                // .paramsSnapshot()
                .build();


        int ret = sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService.addSysQuartzScheduleTaskStatementOfAccountJd(form, getToken());


        if (ret == 0) {
            return ResponseResultOuterUtils.createErrorResult("500", "失败");
        } else {

            /**************************************** head start ****************************************/

            List<SysQuartzScheduleTaskStatementOfAccountJdListItemVo> resultHead = sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService.queryGenericParam(
                    SysQuartzScheduleTaskStatementOfAccountJdQueryCriteria
                            .builder()

                            .batchId(form.getBatchId())

                            .build(),
                    getToken()
            );

            if (CollUtil.isEmpty(resultHead)) {
                return ResponseResultOuterUtils.createErrorResult("500", "未查找到相关的信息");
            }

            SysQuartzScheduleTaskStatementOfAccountJdListItemVo responseData = resultHead.get(0);

            /**************************************** head   end ****************************************/


            return ResponseResultOuterUtils.createSuccesResult("创建成功", responseData);
        }
    }

    /**
     * 此功能定义为 从excel文件导入数据
     * TODO 如果此 batchId 已经导入过 就要先删除这个记录
     * http://192.168.6.223:8089/web/thirdpart/jd/reconciliation_schedule/import/excel/from/storage
     *
     * @param form
     * @return
     * @throws Exception
     */
    @ApiOperation(value = "供应商结算-京东对账-op列表-op导入账单(上传excel与表单)-op手动导入数据(完整版 Full version)")
    @RequestMapping(value = "import/excel/fullVersion/storage", method = RequestMethod.POST)
    @ResponseBody
    public R<String> excelImportFullVersion(@RequestBody SysQuartzScheduleTaskStatementOfAccountJdRunExcelImportForm form) throws Exception {

        return ResponseResultOuterUtils.createSuccesResult(
                sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService
                        .asyncExcelImportFullVersion(
                                form,
                                getToken()
                        )
        );
    }

    /**
     * 此功能定义为 从excel文件导入数据
     * TODO 如果此 batchId 已经导入过 就要先删除这个记录
     * http://192.168.6.2:8089/web/thirdpart/jd/reconciliation_schedule/import/excel/liteVersion/storage
     *
     * @param form
     * @return
     * @throws Exception
     */
    @ApiOperation(value = "供应商结算-京东对账-op列表-op导入账单(上传excel与表单)-op手动导入数据(精简版 Lite version)")
    @RequestMapping(value = "import/excel/liteVersion/storage", method = RequestMethod.POST)
    @ResponseBody
    public R<String> opExcelImportLiteVersion(@RequestBody SysQuartzScheduleTaskStatementOfAccountJdRunExcelImportForm form) throws Exception {

        return ResponseResultOuterUtils.createSuccesResult(
                sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService
                        .asyncExcelImportLiteVersion(
                                form,
                                getToken()
                        )
        );
    }


    /**
     * 步骤3:通过京东的订单数据-比对统一订单数据-生成报告
     * http://192.168.6.2:8089/web/thirdpart/jd/reconciliation_schedule/check/up
     *
     * @param form
     * @return
     * @throws Exception
     */
    @ApiOperation(value = "供应商结算-京东对账-op列表-op导入账单(上传excel与表单)-op手动导入数据-步骤3:通过京东的订单数据-比对统一订单数据-生成报告")
    @RequestMapping(value = "check/up", method = RequestMethod.POST)
    @ResponseBody
    public R<String> opCompareOrderAndCheckUp(@RequestBody SysQuartzScheduleTaskStatementOfAccountJdRunExcelImportForm form) throws Exception {

        return ResponseResultOuterUtils.createSuccesResult(
                sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService
                        .asyncCompareOrderAndCheckUpThenReport(
                                form,
                                getToken()
                        )
        );
    }

    @ApiOperation(value = "供应商结算-京东对账-op对账详情-Debug")
    @RequestMapping(value = "checkUp/{batchId}/{jdOrderSn}", method = RequestMethod.POST)
    @ResponseBody
    public R<AccountingUnifiedSupplierOrderJdCheckUpReportSnapshotStruct> opCheckUpDebug(
            @PathVariable String batchId,
            @PathVariable String jdOrderSn
    ) throws Exception {

        log.info("\nopCheckUpDebug start \n{}\n{}\nopCheckUpDebug   end", batchId, jdOrderSn);

        return ResponseResultOuterUtils.createSuccesResult(
                "debuging",
                accountingUnifiedSupplierOrderJdCheckUpReportDubboConsumerService
                        .readSnapshotJson(
                                batchId,
                                jdOrderSn
                        )
        );
    }


    @ApiOperation(value = "供应商结算-京东对账-op列表-op导入账单(上传excel与表单)-op手动导入数据-步骤3:通过报告的异常数据-重新比对统一订单数据-生成报告")
    @RequestMapping(value = "check/up/exception", method = RequestMethod.POST)
    @ResponseBody
    public R<String> opCompareOrderAndCheckUpOnlyException(@RequestBody SysQuartzScheduleTaskStatementOfAccountJdRunExcelImportForm form) throws Exception {

        return ResponseResultOuterUtils.createSuccesResult(
                sysQuartzScheduleTaskStatementOfAccountJdDubboConsumerService
                        .asyncCompareOrderAndCheckUpThenReportOnlyException(
                                form,
                                getToken()
                        )
        );
    }

}
