package cn.superdesk.uniorder.controller.accounting.customer;

import cn.hutool.core.util.ObjectUtil;
import cn.superdesk.uniorder.common.constants.ResponseMsgConstants;
import cn.superdesk.uniorder.common.enums.accounting.AccountingOrderBackAccountStatusEnum;
import cn.superdesk.uniorder.common.enums.accounting.AccountingOrderInvoiceApplyStatusEnum;
import cn.superdesk.uniorder.common.enums.accounting.customer.AccountingOrderCustomerAccountingStatusEnum;
import cn.superdesk.uniorder.common.exception.BadRequestException;
import cn.superdesk.uniorder.common.exception.utils.ExceptionUtils;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.common.rest.response.ResponseResultOuterUtils;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dao.entity.accounting.score.customer.AccountingScoreCustomer;
import cn.superdesk.uniorder.dao.entity.biz.AccountingReceivablePayable;
import cn.superdesk.uniorder.dto.param.accounting.unified.customer.AccountingReceivablePayableParam;
import cn.superdesk.uniorder.dto.param.accounting.unified.customer.PushAccountingParam;
import cn.superdesk.uniorder.dto.vo.accounting.score.customer.PushRmAccountingVo;
import cn.superdesk.uniorder.dubbo.consumer.accounting.customer.AccountingScoreCustomerDubboConsumerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

@Api(value = "积分结算-对账单-状态流转", tags = {"积分结算-对账单-状态流转"})
@Slf4j
@RestController
@RequestMapping("/accounting/score/customer/statement/state")
public class ScoreCustomerStatementStateFieldAccountingStatusController extends BaseController {

    @Autowired
    private AccountingScoreCustomerDubboConsumerService accountingScoreCustomerDubboConsumerService;

    /****************************************** 状态流转类 start ******************************************/

    /**
     * 确认结算
     * <p>
     * http://192.168.1.223:8089/web/dzmain/confirmDz
     */
    @ApiOperation(value = "积分结算-对账单-op确认对账")
    @RequestMapping(value = "/confirmAccounting", method = RequestMethod.POST)
    @ResponseBody
    public R<?> opConfirmAccounting(Long id) {

        R<?> result = new R<>();
        try {
            AccountingScoreCustomer main = accountingScoreCustomerDubboConsumerService.findById(id);
            if (main.getAccountingStatus() != 0) {
                result.setSuccess(false);
                result.setMessage("确认失败,该对账单已操作！");
                return result;
            }
            return accountingScoreCustomerDubboConsumerService.opConfirmAccounting(id, getToken());
        } catch (Exception e) {

            result.setSuccess(false);
            result.setMessage(ResponseMsgConstants.ERROR_UNKNOW);
            log.error(ExceptionUtils.getMessage(e));
            e.printStackTrace();
        }
        return result;
    }

    /**
     * 拒绝 结算
     */
    @ApiOperation(value = "积分结算-对账单-op拒绝对账")
    @RequestMapping(value = "/refuseAccounting", method = RequestMethod.POST)
    @ResponseBody
    public R<?> opRefuseAccounting(Long id) {

        R<?> result = new R<>();
        try {

            AccountingScoreCustomer main = accountingScoreCustomerDubboConsumerService.findById(id);

            if (main.getAccountingStatus() != 0) {
                result.setSuccess(false);
                result.setMessage("拒绝失败,该对账单已操作！");
                return result;
            }

            return accountingScoreCustomerDubboConsumerService.opRefuseAccounting(id, getToken());
        } catch (Exception e) {

            result.setSuccess(false);
            result.setMessage(ResponseMsgConstants.ERROR_UNKNOW);
            log.error(ExceptionUtils.getMessage(e));
            e.printStackTrace();
        }
        return result;

    }

    /**
     * /accounting/score/customer/statement/state/opAgainAccounting
     * @param id
     * @return
     */
    @ApiOperation(value = "积分结算-对账单-op重新对账")
    @RequestMapping(value = "/opAgainAccounting", method = RequestMethod.POST)
    @ResponseBody
    public R<?> opAgainAccounting(Long id) {

        R<?> result = new R<>();
        try {

            AccountingScoreCustomer accountingScoreCustomerEntity = accountingScoreCustomerDubboConsumerService.findById(id);

            // 要求是 未确认 -> 已确认 mark by linjinyu at 20220321
            if (AccountingOrderCustomerAccountingStatusEnum.已确认.getKey().compareTo(accountingScoreCustomerEntity.getAccountingStatus()) != 0) {
                result.setSuccess(false);
                result.setMessage("该对账单状态发生变更, 请检查");
                return result;
            }
            if (AccountingOrderCustomerAccountingStatusEnum.已取消.getKey().compareTo(accountingScoreCustomerEntity.getAccountingStatus()) == 0) {
                result.setSuccess(false);
                result.setMessage("该对账单已取消,不可重复操作！");
                return result;
            }
            if (AccountingOrderInvoiceApplyStatusEnum.未申请.getKey().compareTo(accountingScoreCustomerEntity.getInvoiceStatus()) != 0) {
                result.setSuccess(false);
                result.setMessage("该对账单已开票,不可重新对账！");
                return result;
            }
            if (AccountingOrderBackAccountStatusEnum.未回款.getKey().compareTo(accountingScoreCustomerEntity.getBackAccountStatus()) != 0) {
                result.setSuccess(false);
                result.setMessage("该对账单已回款,不可重新对账！");
                return result;
            }
            return accountingScoreCustomerDubboConsumerService.opCancelAccounting(id, getToken());
        } catch (Exception e) {

            result.setSuccess(false);
            result.setMessage(ResponseMsgConstants.ERROR_UNKNOW);
            log.error(ExceptionUtils.getMessage(e));
            e.printStackTrace();

        }

        return result;

    }

    @ApiOperation(value = "积分对账-对账单-op取消对账")
    @RequestMapping(value = "/opCancelAccounting", method = RequestMethod.POST)
    @ResponseBody
    public R<?> opCancelAccounting(Long id) {

        R<?> result = new R<>();
        try {

            AccountingScoreCustomer main = accountingScoreCustomerDubboConsumerService.findById(id);

            if (main.getAccountingStatus().compareTo(AccountingOrderCustomerAccountingStatusEnum.未确认.getKey()) != 0) {
                result.setSuccess(false);
                result.setMessage("该对账单状态发生变更, 请检查");
                return result;
            }
            if (main.getAccountingStatus().compareTo(AccountingOrderCustomerAccountingStatusEnum.已取消.getKey()) == 0) {
                result.setSuccess(false);
                result.setMessage("该对账单已取消,不可重复操作！");
                return result;
            }
            if (main.getInvoiceStatus().compareTo(AccountingOrderInvoiceApplyStatusEnum.未申请.getKey()) != 0) {
                result.setSuccess(false);
                result.setMessage("该对账单已开票,不可重新对账！");
                return result;
            }
            if (main.getBackAccountStatus().compareTo(AccountingOrderBackAccountStatusEnum.未回款.getKey()) != 0) {
                result.setSuccess(false);
                result.setMessage("该对账单已回款,不可重新对账！");
                return result;
            }

            return accountingScoreCustomerDubboConsumerService.opCancelAccounting(id, getToken());
        } catch (Exception e) {

            result.setSuccess(false);
            result.setMessage(ResponseMsgConstants.ERROR_UNKNOW);
            log.error(ExceptionUtils.getMessage(e));
            e.printStackTrace();

        }

        return result;

    }

    /**
     * 删除对账 只能删除已取消的
     */
    @ApiOperation(value = "积分结算-对账单-op删除对账")
    @RequestMapping(value = "/deleteAccounting", method = RequestMethod.POST)
    @ResponseBody
    public R<?> opDeleteAccounting(Long id) {

        R<?> result = new R<>();

        try {

            AccountingScoreCustomer main = accountingScoreCustomerDubboConsumerService.findById(id);

            if (main.getAccountingStatus() == -1) {

                accountingScoreCustomerDubboConsumerService.opDeleteAccounting(id, getToken());

                result.setSuccess(true);

            } else {
                result.setSuccess(false);
                result.setMessage("不能删除 未取消结算单");
                return result;
            }


        } catch (Exception e) {

            result.setSuccess(false);
            result.setMessage(ResponseMsgConstants.ERROR_UNKNOW);

            log.error(ExceptionUtils.getMessage(e));

            e.printStackTrace();

        }

        return result;

    }

    /**
     * 生成应收&应付数据
     */
    @ApiOperation(value = "积分结算-对账单-生成应收&应付数据")
    @RequestMapping(value = "/startFlowData", method = RequestMethod.POST)
    @ResponseBody
    public R<?> opStartFlowData(Long id) {
        R<?> result = new R<>();
        try {
            AccountingScoreCustomer scoreCustomer = accountingScoreCustomerDubboConsumerService.findById(id);
            if (scoreCustomer.getAccountingStatus() != 1) {
                result.setSuccess(false);
                result.setMessage("该功能仅支持对账单状态为“已确认”时使用");
                return result;
            }
            if ((scoreCustomer.getReceivableDataStatus().equals(1) || scoreCustomer.getReceivableDataStatus().equals(3))
                    && (scoreCustomer.getPayableDataStatus().equals(1) || scoreCustomer.getPayableDataStatus().equals(3))) {
                result.setSuccess(false);
                result.setMessage("该功能仅支持应收或应付数据状态为“未生成/生成失败”时使用，请检查状态");
                return result;
            }
            return accountingScoreCustomerDubboConsumerService.opStartFlowData(id, getToken());
        } catch (Exception e) {
            result.setSuccess(false);
            result.setMessage(ResponseMsgConstants.ERROR_UNKNOW);
            log.error(ExceptionUtils.getMessage(e));
            e.printStackTrace();
        }
        return result;
    }

    /**
     * 创建报账单
     */
    @ApiOperation(value = "积分结算-对账单-创建报账单")
    @RequestMapping(value = "/createRmAccounting", method = RequestMethod.POST)
    @ResponseBody
    public R<?> createRmAccounting(Long id) {
        R<?> result = new R<>();
        try {
            AccountingScoreCustomer main = accountingScoreCustomerDubboConsumerService.findById(id);
            if (main.getAccountingStatus() != 1) {
                result.setSuccess(false);
                result.setMessage("该功能仅支持对账单状态为“已确认”时使用");
                return result;
            }
            if (!(main.getReceivableAccountStatus() == null || main.getReceivableAccountStatus() == 0 || main.getReceivableAccountStatus() == 1 ||
                    main.getWxReceivableAccountStatus() == null  || main.getWxReceivableAccountStatus() == 0 || main.getWxReceivableAccountStatus() == 1 ||
                    main.getPayableAccountStatus() == null  || main.getPayableAccountStatus() == 0 || main.getPayableAccountStatus() == 1)) {
                result.setSuccess(false);
                result.setMessage("该功能仅支持应收或应付报账状态为“未报账/部分报账”时使用，请检查状态");
                return result;
            }
            return accountingScoreCustomerDubboConsumerService.createRmAccounting(id, getToken());
        } catch (BadRequestException e) {
            result.setSuccess(false);
            result.setMessage(e.getMessage());
            log.error(ExceptionUtils.getMessage(e));
            e.printStackTrace();
        } catch (Exception e) {
            result.setSuccess(false);
            result.setMessage(ResponseMsgConstants.ERROR_UNKNOW);
            log.error(ExceptionUtils.getMessage(e));
            e.printStackTrace();
        }
        return result;
    }

    /**
     * 推送报账单
     */
    @ApiOperation(value = "积分结算-对账单-推送报账单")
    @RequestMapping(value = "/pushRmAccounting", method = RequestMethod.POST)
    @ResponseBody
    public R<?> pushRmAccounting(@RequestBody PushAccountingParam pushAccountingParam) {
        R<?> result = new R<>();
        try {
            if (ObjectUtil.isNull(pushAccountingParam) || ObjectUtil.isEmpty(pushAccountingParam.getId())) {
                return ResponseResultOuterUtils.createErrorResult("500", "报账单Id不能为空");
            }

            return accountingScoreCustomerDubboConsumerService.pushRmAccounting(pushAccountingParam, getToken());
        } catch (BadRequestException e) {
            result.setSuccess(false);
            result.setMessage(e.getMessage());
            log.error(ExceptionUtils.getMessage(e));
            e.printStackTrace();
        } catch (Exception e) {
            result.setSuccess(false);
            result.setMessage(ResponseMsgConstants.ERROR_UNKNOW);
            log.error(ExceptionUtils.getMessage(e));
            e.printStackTrace();
        }
        return result;
    }

    /**
     * 查询推送报账单
     */
    @ApiOperation(value = "积分结算-对账单-查询推送报账单")
    @RequestMapping(value = "/queryPushRmAccounting", method = RequestMethod.POST)
    @ResponseBody
    public R<?> queryPushRmAccounting(Long id, String accountingOrderSn) {
        R<?> result = new R<>();
        try {
            if (id == null && StringUtils.isBlank(accountingOrderSn)) {
                return ResponseResultOuterUtils.createErrorResult("500", "对账单编码不能为空");
            }

            return accountingScoreCustomerDubboConsumerService.queryPushRmAccounting(id,accountingOrderSn, getToken());
        } catch (BadRequestException e) {
            result.setSuccess(false);
            result.setMessage(e.getMessage());
            log.error(ExceptionUtils.getMessage(e));
            e.printStackTrace();
        } catch (Exception e) {
            result.setSuccess(false);
            result.setMessage(ResponseMsgConstants.ERROR_UNKNOW);
            log.error(ExceptionUtils.getMessage(e));
            e.printStackTrace();
        }
        return result;
    }

    /**
     * 删除报账单
     */
    @ApiOperation(value = "积分结算-对账单-删除报账单")
    @RequestMapping(value = "/delRmAccounting", method = RequestMethod.POST)
    @ResponseBody
    public R<?> delRmAccounting(Long id) {
        R<?> result = new R<>();
        try {
            if (ObjectUtil.isNull(id)) {
                return ResponseResultOuterUtils.createErrorResult("500", "报账单Id不能为空");
            }

            return accountingScoreCustomerDubboConsumerService.delRmAccounting(id, getToken());
        } catch (BadRequestException e) {
            result.setSuccess(false);
            result.setMessage(e.getMessage());
            log.error(ExceptionUtils.getMessage(e));
            e.printStackTrace();
        } catch (Exception e) {
            result.setSuccess(false);
            result.setMessage(ResponseMsgConstants.ERROR_UNKNOW);
            log.error(ExceptionUtils.getMessage(e));
            e.printStackTrace();
        }
        return result;
    }

    /**
     * 设置备注-报账单
     */
    @ApiOperation(value = "积分结算-对账单-报账单设置备注")
    @RequestMapping(value = "/setRemarkRmAccounting", method = RequestMethod.POST)
    @ResponseBody
    public R<?> setRemarkRmAccounting(@RequestBody AccountingReceivablePayableParam accountingParam) {
        R<?> result = new R<>();
        try {
            if (ObjectUtil.isNull(accountingParam) || accountingParam.getId() == null) {
                return ResponseResultOuterUtils.createErrorResult("500", "对账单Id不能为空");
            }
            if (StringUtils.isNoneBlank(accountingParam.getRemark()) && accountingParam.getRemark().length() > 200) {
                return ResponseResultOuterUtils.createErrorResult("500", "请输入后台备注内容（限200字以内）");
            }

            return accountingScoreCustomerDubboConsumerService.setRemarkRmAccounting(accountingParam, getToken());
        } catch (BadRequestException e) {
            result.setSuccess(false);
            result.setMessage(e.getMessage());
            log.error(ExceptionUtils.getMessage(e));
            e.printStackTrace();
        } catch (Exception e) {
            result.setSuccess(false);
            result.setMessage(ResponseMsgConstants.ERROR_UNKNOW);
            log.error(ExceptionUtils.getMessage(e));
            e.printStackTrace();
        }
        return result;
    }

    /****************************************** 状态流转类   end ******************************************/
}
