package cn.superdesk.uniorder.controller.biz;

import cn.superdesk.uniorder.common.HttpUtil;
import cn.superdesk.uniorder.common.api.APIUtil;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dto.param.datalake.DataLakeMasterDataPushParam;
import cn.superdesk.uniorder.dto.vo.lakedata.DataLakeMasterDataPushVo;
import cn.superdesk.uniorder.dto.vo.lakedata.DataLakeMianDataVo;
import cn.superdesk.uniorder.dto.vo.lakedata.DataLakeUuidVo;
import cn.superdesk.uniorder.dto.vo.organisation.SupplierDetailVo;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.UnknownHostException;
import java.text.SimpleDateFormat;
import java.util.*;

@Slf4j
@RestController
@RequestMapping("apiSource/")
@Api(tags = "数据湖控制层")
public class ApiSourceController extends BaseController {

    @Value("${uniorder.appKey:1}")
    private String appKey;
    @Value("${uniorder.appSecret:1}")
    private String appSecret;
    @Value("${uniorder.supplierUrl:1}")
    private String supplierUrl;
    @Value("${uniorder.dataLakeUuid:1}")
    private String dataLakeUuid;
    @Value("${uniorder.dataLakeMainData:1}")
    private String dataLakeMainData;

    @Value("${uniorder.dataLakeMasterDataPush:1}")
    private String dataLakeMasterDataPush;
    /**
     * 查询供应商信息
     * @param supplierCode
     */
    @ApiOperation(value = "查询供应商信息")
    @RequestMapping(value = "getSupplierDetail", method = RequestMethod.GET)
    public R<SupplierDetailVo> getSupplierDetail(@RequestParam(value = "supplierCode") String supplierCode) {
        if(StringUtils.isBlank(supplierCode)){
            return R.fail();
        }
        Map<String,String> apiParamMap = new HashMap<>();
        apiParamMap.put("supplierCode",supplierCode);
        String apiParam = JSON.toJSONString(apiParamMap);
        String result = "";
        try {
            log.info("getSupplierDetail apiParam:{}，postUrl：{}", apiParam, supplierUrl);
            result = APIUtil.post(supplierUrl,apiParam,appKey,appSecret);

            log.info("getSupplierDetail supplierUrl={} supplierCode={} result={}}",supplierUrl,supplierCode,result);
            if(StringUtils.isBlank(result)) {
                return new R<>("500","暂无数据",false);
            }
            JSONObject contentJson = JSONObject.parseObject(result);
            if(contentJson != null && contentJson.getInteger("code") == 0){
                JSONArray jsonArray = contentJson.getJSONArray("content");
                if(jsonArray == null || jsonArray.size() == 0){
                    return new R<>("500","暂无数据",false);
                }
                JSONObject jsonObject = jsonArray.getJSONObject(0);
                SupplierDetailVo supplierDetailVo = new SupplierDetailVo();
                supplierDetailVo.setUuid(jsonObject.getString("uuid"));
                supplierDetailVo.setFullName(jsonObject.getString("fullName"));
                supplierDetailVo.setNameAbbreviation(jsonObject.getString("nameAbbreviation"));
                supplierDetailVo.setSupplierCode(jsonObject.getString("supplierCode"));
                String bankJsonStr = jsonObject.getString("bank_info_fw");
                if(StringUtils.isNotBlank(bankJsonStr)){
                    JSONArray bankJsonList = JSONObject.parseArray(bankJsonStr);
                    if(bankJsonList != null && bankJsonList.size() > 0){
                        List<SupplierDetailVo.BankInfoFw> bankInfoFws = new ArrayList<>();
                        for (int i = 0; i < bankJsonList.size(); i++) {
                            JSONObject bankJson = bankJsonList.getJSONObject(i);
                            if(bankJson != null){
                                SupplierDetailVo.BankInfoFw bankInfoFw = new SupplierDetailVo.BankInfoFw();
                                bankInfoFw.setBankOfDeposit(bankJson.getString("BANK_OF_DEPOSIT"));
                                bankInfoFw.setBankAccount(bankJson.getString("BANK_ACCOUNT"));
                                bankInfoFw.setAccountName(bankJson.getString("ACCOUNT_NAME"));
                                bankInfoFw.setNumber(bankJson.getString("number"));
                                bankInfoFw.setAccountType(bankJson.getString("account_type"));
                                bankInfoFws.add(bankInfoFw);
                            }
                        }
                        supplierDetailVo.setBankInfoFw(bankInfoFws);
                    }
                }
                return R.ok(supplierDetailVo);
            } else {
                return new R<>("500","暂无数据",false);
            }
        } catch (Exception e) {
            log.error("getSupplierDetail error supplierUrl={} supplierCode={}",supplierUrl,supplierCode,e);
            return new R<>("500","暂无数据",false);
        }
   }

    @ApiOperation(value = "查询数据湖-查询客户customerUuid")
    @RequestMapping(value = "getDataLakeUuid", method = RequestMethod.GET)
    @ResponseBody
    public R<DataLakeUuidVo> getDataLakeUuid(@RequestParam(value = "customerId") String customerId) {
        if (StringUtils.isBlank(customerId)) {
            log.error("request param is null.customerId is ", customerId);
            return R.fail();
        }
        Map<String, String> apiParamMap = new HashMap<>();
        apiParamMap.put("customerId", customerId);
        String apiParam = JSON.toJSONString(apiParamMap);
        String result = "";
        try {
            result = APIUtil.post(dataLakeUuid, apiParam, appKey,appSecret);
            log.info("getDataLakeUuid result is ", result);
            if (StringUtils.isNotBlank(result)) {
                DataLakeUuidVo dataLakeUuidVo = JSONObject.parseObject(result, DataLakeUuidVo.class);
                return R.ok(dataLakeUuidVo);
            }
        } catch (Exception e) {
            log.error("getDataLake error customerId={}", customerId, e);
            return R.fail("失败");
        }
        return R.fail("失败");
    }

    @ApiOperation(value = "查询数据湖-查询客户fullName客户名称")
    @RequestMapping(value = "getDataLakeContent", method = RequestMethod.GET)
    @ResponseBody
    public R<DataLakeMianDataVo> getDataLakeContent(@RequestParam(value = "customerUuid") String customerUuid,
                                                    @RequestParam(value = "fullName") String fullName,
                                                    @RequestParam(value = "clientCode") String clientCode) {
        if (StringUtils.isBlank(customerUuid)
                || StringUtils.isBlank(fullName)
                ||StringUtils.isBlank(clientCode)) {
            log.error("request param is null. customerUuid is {},customerUuid is {},customerUuid is {}."
                    , customerUuid, fullName, clientCode);
            return R.fail();
        }
        Map<String, String> apiParamMap = new HashMap<>();
        apiParamMap.put("customerUuid", customerUuid);
        apiParamMap.put("fullName", fullName);
        apiParamMap.put("clientCode", clientCode);
        String apiParam = JSON.toJSONString(apiParamMap);
        String result = "";
        try {
            result = APIUtil.post(dataLakeMainData, apiParam, appKey,appSecret);
            log.info("getDataLakeContent result is ", result);
            if (StringUtils.isNotBlank(result)) {
                DataLakeMianDataVo dataLakeMianDataVo = JSONObject.parseObject(result, DataLakeMianDataVo.class);
                return R.ok(dataLakeMianDataVo);
            }
        } catch (Exception e) {
            log.error("getDataLake error customerUuid is {},customerUuid is {},customerUuid is {}."
                    , customerUuid, fullName, clientCode);
            return R.fail("失败");
        }
        return R.fail("失败");
    }

    @ApiOperation(value = "数据湖-主数据推送")
    @RequestMapping(value = "dataLakeMasterDataPush", method = RequestMethod.POST)
    @ResponseBody
    public R<DataLakeMasterDataPushVo> dataLakeMasterDataPush
            (@RequestBody DataLakeMasterDataPushParam dataLakeMasterDataPushParam) {
        if (dataLakeMasterDataPushParam ==null || StringUtils.isBlank(dataLakeMasterDataPushParam.getCustomerId())
                || StringUtils.isBlank(dataLakeMasterDataPushParam.getFullName())
                || StringUtils.isBlank(dataLakeMasterDataPushParam.getPropertyCode()))
        {
            log.error("request param is invalid.dataLakeMasterDataPushParam is {}", dataLakeMasterDataPushParam);
            return new R<>("500","参数必填项校验失败",false);
        }
        R<DataLakeMasterDataPushVo> resp = null;
        try {
            if (dataLakeMasterDataPushParam.getUpdateTime() == null) { // 默认当前时间
                SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMdd HH:mm:ss");
                dataLakeMasterDataPushParam.setUpdateTime(dateFormat.format(new Date()));
            }
            if (dataLakeMasterDataPushParam.getStatus() == null) {
                dataLakeMasterDataPushParam.setStatus("0"); // 默认0-有效
            }
            // 其他或境外组织：⽆需传输
            if (!"3".equals(dataLakeMasterDataPushParam.getPropertyCode())) {
                dataLakeMasterDataPushParam.setClientCode(dataLakeMasterDataPushParam.getCustomerId());
            }
            String apiParam = JSONObject.toJSONString(dataLakeMasterDataPushParam);
            log.info("dataLakeMainDataPush apiParam:{}，postUrl：{}", apiParam, dataLakeMasterDataPush);
            String result = APIUtil.post(dataLakeMasterDataPush, apiParam, appKey,appSecret);
            log.info("dataLakeMainDataPush result is {}", result);
            DataLakeMasterDataPushVo dataLakeMasterDataPushVo = JSONObject.parseObject(result, DataLakeMasterDataPushVo.class);
            if (dataLakeMasterDataPushVo != null && "0".equals(dataLakeMasterDataPushVo.getCode())) {
                resp = new R("200", "更新成功", true);
            } else {
                resp = new R<>("500", "更新失败", false);
            }
            resp.setData(dataLakeMasterDataPushVo);
            return resp;
        } catch (Exception e) {
            log.error("dataLakeMasterDataPush error dataLakeMasterDataPushParam={}, {}", dataLakeMasterDataPushParam, e);
            return new R<>("500","更新失败",false);
        }
    }

    @ApiOperation(value = "模拟服务器中转调用API接口")
    @PostMapping(value = "testApi")
    @ResponseBody
    public R testApi(@RequestParam(value = "apiUrl") String apiUrl,@RequestParam(value = "apiParam") String apiParam) {
        if (StringUtils.isBlank(apiUrl)) {
            return R.fail();
        }
        String result = "";
        try {
            result = HttpUtil.post(apiParam,apiUrl);
            log.info("testApi result is {}", result);
        } catch (Exception e) {
            log.error("testApi error apiUrl is {}", apiUrl,e);
        }
        return R.ok(result);
    }

    @ApiOperation(value = "模拟服务器下载附件")
    @PostMapping(value = "testDownload")
    @ResponseBody
    public R testDownload(@RequestParam(value = "apiUrl") String apiUrl,@RequestParam(value = "destFile") String destFile) {
        if (StringUtils.isBlank(apiUrl)) {
            return R.fail();
        }
        String result = "";
        try {
            toBDFile(apiUrl, destFile);
        } catch (Exception e) {
            log.error("testDownload error apiUrl is {}", apiUrl,e);
        }
        return R.ok(result);
    }

    public void toBDFile(String urlStr, String bdUrl) throws Exception {
        URL url = new URL(urlStr);
        HttpURLConnection conn = (HttpURLConnection) url.openConnection();
        DataInputStream in = new DataInputStream(conn.getInputStream());
        byte[] data=toByteArray(in);
        in.close();
        File f = new File(bdUrl);
        if (!f.getParentFile().exists()) {
            f.getParentFile().mkdirs();
        }
        FileOutputStream out=new FileOutputStream(bdUrl);
        out.write(data);
        out.close();
    }

    public byte[] toByteArray(InputStream in) throws IOException {

        ByteArrayOutputStream out=new ByteArrayOutputStream();
        byte[] buffer=new byte[1024*4];
        int n=0;
        while ( (n=in.read(buffer)) !=-1) {
            out.write(buffer,0,n);
        }
        return out.toByteArray();
    }
}



