package cn.superdesk.uniorder.controller.biz;

import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.common.security.JwtUtil;
import cn.superdesk.uniorder.common.security.dto.LoginSysUser;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dao.entity.biz.BizContract;
import cn.superdesk.uniorder.dao.entity.organisation.UnifiedSupplier;
import cn.superdesk.uniorder.dto.param.biz.BizContractParam;
import cn.superdesk.uniorder.dubbo.consumer.biz.BizContractDubboConsumerService;
import cn.superdesk.uniorder.dubbo.consumer.organisation.SysSupplierDubboConsumerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

@Slf4j
@RestController
@RequestMapping("bizContract/")
@Api(tags = "合同控制层")
public class BizContractController extends BaseController {

    @Autowired
    private BizContractDubboConsumerService bizContractDubboConsumerService;
    @Autowired
    private SysSupplierDubboConsumerService unifiedSupplierService;

    /**
     * 分页显示
     */
    @ApiOperation(value = "分页列表")
    @RequestMapping(value = "/getListPage", method = RequestMethod.POST)
    @ResponseBody
    public R<List<BizContract>> getListPage(@RequestBody BizContractParam contractParam) {
        return bizContractDubboConsumerService.getListPage(contractParam);
    }

    /**
     * 供应商ID或者编码查询
     * @param contractParam 分页参数
     */
    @ApiOperation(value = "供应商ID或者编码查询")
    @RequestMapping(value = "/getList", method = RequestMethod.POST)
    @ResponseBody
    public R<List<BizContract>> getList(@RequestBody BizContractParam contractParam) {
        BizContract bizContract = new BizContract();
        bizContract.setSupplierId(contractParam.getSupplierId());
        bizContract.setSupplierCode(contractParam.getSupplierCode());
        return R.ok(bizContractDubboConsumerService.selectBizContractList(bizContract));
    }


    /**
     * 新增合同
     * @param contractParam 新增参数
     */
    @ApiOperation(value = "新增")
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    public R save(@RequestBody BizContractParam contractParam) {
        if(contractParam.getSupplierId() == null || contractParam.getSupplierId() < 1L){
            return R.fail("供应商ID不能为空");
        }
        UnifiedSupplier unifiedSupplier = unifiedSupplierService.findById(contractParam.getSupplierId());
        if(unifiedSupplier == null){
            return R.fail("供应商信息为空");
        }

        LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(getToken());
        BizContract bizContract = new BizContract();
        BeanUtils.copyProperties(contractParam,bizContract);
        bizContract.setSourceType(unifiedSupplier.getServiceNo());
        bizContract.setSupplierName(unifiedSupplier.getSaleName());
        bizContract.setCreateBy(loginSysUser.getFullName());
        bizContract.setCreateTime(new Date());
        bizContract.setUpdateBy(loginSysUser.getFullName());
        bizContract.setUpdateTime(new Date());
        Integer result = bizContractDubboConsumerService.insertBizContract(bizContract);
        return result > 0 ? R.ok() : R.fail() ;
    }

    /**
     * 修改合同
     * @param contractParam 修改参数
     */
    @ApiOperation(value = "修改")
    @RequestMapping(value = "/update", method = RequestMethod.POST)
    public R update(@RequestBody BizContractParam contractParam) {
        if(contractParam.getId() == null || contractParam.getId() < 1L){
            return R.fail("ID不能为空");
        }

        LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(getToken());
        BizContract bizContract = new BizContract();
        BeanUtils.copyProperties(contractParam,bizContract);
        bizContract.setId(contractParam.getId());
        bizContract.setContractCode(contractParam.getContractCode());
        bizContract.setContractName(contractParam.getContractName());
        bizContract.setRemark(contractParam.getRemark());
        bizContract.setUpdateBy(loginSysUser.getFullName());
        bizContract.setUpdateTime(new Date());
        bizContract.setId(bizContract.getId());
        Integer result = bizContractDubboConsumerService.updateBizContract(bizContract);
        return result > 0 ? R.ok() : R.fail() ;
    }

    /**
     * 设置默认
     * @param contractParam 修改参数
     */
    @ApiOperation(value = "设置默认")
    @RequestMapping(value = "/defaultYes", method = RequestMethod.POST)
    public R defaultYes(@RequestBody BizContractParam contractParam) {
        LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(getToken());
        BizContract bizContract = new BizContract();
        BeanUtils.copyProperties(contractParam,bizContract);
        bizContract.setUpdateBy(loginSysUser.getFullName());
        bizContract.setUpdateTime(new Date());
        Integer result = bizContractDubboConsumerService.defaultYes(bizContract);
        return result > 0 ? R.ok() : R.fail() ;
    }

    /**
     * 删除合同
     * @param contractParam
     */
    @ApiOperation(value = "删除")
    @RequestMapping(value = "/delete", method = RequestMethod.POST)
    public R delete(@RequestBody BizContractParam contractParam) {
        if(CollectionUtils.isEmpty(contractParam.getIds())){
            return R.fail("ID集合不能为空");
        }

        int result = bizContractDubboConsumerService.deleteBizContractByIds(contractParam.getIds().toArray(new Long[0]));
        return result > 0 ? R.ok() : R.fail() ;
    }

    /**
     * 查询合同信息
     * @param id Integer
     */
    @ApiOperation(value = "详情")
    @RequestMapping(value = "/getDetail", method = RequestMethod.GET)
    public R<BizContract> getDetail(@RequestParam(value = "id") Long id) {
        return R.ok(bizContractDubboConsumerService.selectBizContractById(id));
   }
}



