package cn.superdesk.uniorder.controller.invoice.reportaccounting;

import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.common.rest.response.ResponseResultOuterUtils;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dto.param.invoice.customer.reportaccounting.*;
import cn.superdesk.uniorder.dto.vo.invoice.reportaccounting.ActualReceivedReportAccountingDetailVO;
import cn.superdesk.uniorder.dto.vo.invoice.reportaccounting.ActualReceivedReportAccountingVO;
import cn.superdesk.uniorder.dto.vo.invoice.reportaccounting.ActualReportAccountingPreCheckVO;
import cn.superdesk.uniorder.dubbo.consumer.invoice.reportaccounting.ReportAccountingDubboConsumerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @Description: 报账单
 * @Author: linjinyu
 * @CreateDate: 7/29/20 7:31 PM
 */
@Api(value = "积分结算-发票管理", tags = {"积分结算-发票管理"})
@Slf4j
@RestController
@RequestMapping("/invoice/report/accounting")
public class InvoiceReportAccountingController extends BaseController {

    @Autowired
    private ReportAccountingDubboConsumerService reportAccountingDubboConsumerService;

    /**
     * opt 修复账单回款状态
     * /invoice/manager/customer/score/invoiceAccountingStatusRepair
     *
     */
    @ApiOperation(value = "积分结算-发票管理-修复账单回款状态-mark by zhufeng at 20221124")
    @PostMapping("/invoiceAccountingStatusRepair")
    public R<?> repairInvoiceStatus(@RequestBody InvoiceStatusRepair repair) {
        reportAccountingDubboConsumerService.repairInvoiceStatus(repair, getToken());
        return ResponseResultOuterUtils.createSuccesResult();
    }

    /**
     * opt 积分结算-发票管理-实收报账详情（包括对账单列表）
     * /invoice/manager/customer/score/actualReceivedAccounting
     *
     */
    @ApiOperation(value = "积分结算-发票管理-实收报账详情（包括对账单列表）-mark by zhufeng at 20221124")
    @PostMapping("/actualReceivedAccounting")
    public R<ActualReceivedReportAccountingVO> getActualReceivedAccountingVO(@RequestBody ReportAccountingQueryParam param) {
        ActualReceivedReportAccountingVO actualReceivedReportAccountingVO = reportAccountingDubboConsumerService.getActualReceivedAccountingVO(param, getToken());
        return new R<>(actualReceivedReportAccountingVO);
    }

    /**
     * opt 积分结算-发票管理-实收报账单详情（包括对账单列表详情）
     * /invoice/manager/customer/score/actualReceivedReportAccountingDetail
     *
     */
    @ApiOperation(value = "积分结算-发票管理-实收报账单详情-mark by zhufeng at 20221124")
    @PostMapping("/actualReceivedReportAccountingDetail")
    public R<ActualReceivedReportAccountingDetailVO> getActualReceivedReportAccountingDetail(@RequestBody ActualReportAccountingQueryParam param) {
        ActualReceivedReportAccountingDetailVO actualReceivedReportAccountingDetail = reportAccountingDubboConsumerService.getActualReceivedReportAccountingDetail(param, getToken());
        return new R<>(actualReceivedReportAccountingDetail);
    }

    /**
     * opt 积分结算-发票管理-实收报账详情（包括对账单列表详情）
     * /invoice/manager/customer/score/actualReceivedAccountingPreCheck
     *
     */
    @ApiOperation(value = "积分结算-发票管理-新增实收报账预检查-mark by zhufeng at 20221124")
    @PostMapping("/actualReceivedAccountingPreCheck")
    public R<ActualReportAccountingPreCheckVO> preCheckByCreateActualReceivedAccounting(@RequestBody ActualReportAccountingPreCheckParam param) {
            return reportAccountingDubboConsumerService.preCheckByCreateActualReceivedAccounting(param, getToken());
    }

    /**
     * opt 积分结算-发票管理-实收报账单新增
     * /invoice/manager/customer/score/actualReceivedAccountingCreate
     *
     */
    @ApiOperation(value = "积分结算-发票管理-实收报账单新增 -mark by zhufeng at 20221124")
    @PostMapping("/actualReceivedAccountingCreate")
    public R<?> createActualReceivedAccounting(@RequestBody ActualReportAccountingCreateParam param) {
        return reportAccountingDubboConsumerService.createActualReceivedAccounting(param, getToken());
    }

    /**
     * opt 积分结算-发票管理-实收报账单信息修改
     * /invoice/manager/customer/score/actualReceivedAccountingModify
     *
     */
    @ApiOperation(value = "积分结算-发票管理-实收报账单信息修改 -mark by zhufeng at 20221124")
    @PostMapping("/actualReceivedAccountingModify")
    public R<?> modifyActualReceivedAccounting(@RequestBody ActualReportAccountingModifyParam param) {
        reportAccountingDubboConsumerService.modifyActualReceivedAccounting(param, getToken());
        return ResponseResultOuterUtils.createSuccesResult();
    }

    /**
     * opt 积分结算-发票管理-报账状态查询(结果确认)
     * /invoice/manager/customer/score/invoiceActualReceivedAccountingModify
     *
     */
    @ApiOperation(value = "积分结算-发票管理-报账状态查询(结果确认) -mark by zhufeng at 20221124")
    @PostMapping("/actualReceivedAccountingConfirm")
    public R<?> confirmActualReceivedAccounting(@RequestBody ActualReportAccountingConfirmParam param) {
        return reportAccountingDubboConsumerService.confirmActualReceivedAccounting(param, getToken());
    }
}
