package cn.superdesk.uniorder.controller.invoice.supplier;

import cn.superdesk.uniorder.common.exception.utils.ExceptionUtils;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.common.rest.response.ResponseResultOuterUtils;
import cn.superdesk.uniorder.common.security.JwtUtil;
import cn.superdesk.uniorder.common.security.dto.LoginSysUser;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dto.param.invoice.customer.reportaccounting.ActualPayableReportAccountingCreateParam;
import cn.superdesk.uniorder.dto.param.invoice.customer.reportaccounting.ActualReportAccountingConfirmParam;
import cn.superdesk.uniorder.dto.param.invoice.customer.reportaccounting.ActualReportAccountingModifyParam;
import cn.superdesk.uniorder.dto.param.invoice.customer.reportaccounting.InvoiceStatusRepair;
import cn.superdesk.uniorder.dto.vo.invoice.reportaccounting.ActualPayableReportAccountingDetailVO;
import cn.superdesk.uniorder.dto.vo.invoice.supplier.paymentReport.AddPaymentReportEchoVo;
import cn.superdesk.uniorder.dto.vo.invoice.supplier.paymentReport.PaymentReportEchoVo;
import cn.superdesk.uniorder.dubbo.consumer.invoice.reportaccounting.ReportAccountingDubboConsumerService;
import cn.superdesk.uniorder.dubbo.consumer.invoice.supplier.AccountingReportPaymentDubboConsumerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

@Api(value = "供应商结算-货款管理 - 付款报账单", tags = {"供应商结算-货款管理"})
@Slf4j
@RestController
@RequestMapping("/invoice/manager/supplier/paymentReport")
public class AccountingReportPaymentController extends BaseController {
    @Autowired
    private AccountingReportPaymentDubboConsumerService accountingReportPaymentDubboConsumerService;
    @Resource
    private ReportAccountingDubboConsumerService reportAccountingDubboConsumerService;

    @ApiOperation(value = "供应商结算-货款管理-付款报账单页面回显信息", notes = "供应商结算2.0")
    @PostMapping("/getPaymentReportEcho")
    @ResponseBody
    public R<PaymentReportEchoVo> getPaymentReportEcho(String accountingOrderId) {
        R<PaymentReportEchoVo> response = new R<>();
        try {
            PaymentReportEchoVo paymentReportEcho = accountingReportPaymentDubboConsumerService.getPaymentReportEcho(accountingOrderId);
            response.setData(paymentReportEcho);
        } catch (Exception e) {
            response.setCode(String.valueOf(e.hashCode()));
            response.setSuccess(false);
            response.setMessage(e.getMessage());
            log.error(ExceptionUtils.getMessage(e));
            e.printStackTrace();
        }
        return response;
    }

    @ApiOperation(value = "供应商结算-货款管理-新增付款报账单页面回显信息", notes = "供应商结算2.0")
    @PostMapping("/getAddPaymentReportEcho")
    @ResponseBody
    public R<AddPaymentReportEchoVo> getAddPaymentReportEcho(Long accountingOrderId) {
        LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(getToken());

        return accountingReportPaymentDubboConsumerService.getAddPaymentReportEcho(accountingOrderId, loginSysUser);
    }

    @ApiOperation(value = "供应商结算-货款管理-账单付款状态确认修复", notes = "供应商结算2.0")
    @PostMapping("/billPaymentStatusRepair")
    @ResponseBody
    public R<?> billPaymentStatusRepair(InvoiceStatusRepair repair) {
        accountingReportPaymentDubboConsumerService.billPaymentStatusRepair(repair, getToken());
        return ResponseResultOuterUtils.createSuccesResult();
    }

    @ApiOperation(value = "供应商结算-货款管理-新增付款报账单", notes = "供应商结算2.0")
	@PostMapping("/addPaymentReport")
	@ResponseBody
	public R<?> addPaymentReport(@RequestBody ActualPayableReportAccountingCreateParam param) {

		return accountingReportPaymentDubboConsumerService.addPaymentReport(param, getToken());
	}

	@ApiOperation(value = "供应商结算-货款管理-报账状态查询", notes = "供应商结算2.0")
	@PostMapping("/confirmPaymentReportList")
	@ResponseBody
	public R<?> confirmPaymentReportList(@RequestBody ActualReportAccountingConfirmParam param) {
		return accountingReportPaymentDubboConsumerService.confirmPaymentReportList(param, getToken());
	}

	@ApiOperation(value = "供应商结算-货款管理-付款报账单详情", notes = "供应商结算2.0")
	@PostMapping("/getPaymentReportDetail")
	@ResponseBody
	public R<ActualPayableReportAccountingDetailVO> getPaymentReportDetail(Long id) {

		return new R<>(accountingReportPaymentDubboConsumerService.getPaymentReportDetail(id));
	}

    @ApiOperation(value = "供应商结算-货款管理-付款报账单更新", notes = "供应商结算2.0")
    @PostMapping("/updatePaymentInfo")
    @ResponseBody
    public R<?> modifyActualPaymentAccounting(@RequestBody ActualReportAccountingModifyParam param) {
        reportAccountingDubboConsumerService.modifyActualReceivedAccounting(param, getToken());
        return ResponseResultOuterUtils.createSuccesResult();
    }
}
