package cn.superdesk.uniorder.controller.organisation;

import cn.superdesk.uniorder.common.json.AnJsonUtils;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.common.security.JwtUtil;
import cn.superdesk.uniorder.common.security.dto.LoginSysUser;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dao.entity.sys.SysUser;
import cn.superdesk.uniorder.dto.param.organisation.SysBusinessUserParam;
import cn.superdesk.uniorder.dto.param.organisation.SysBusinessUserQueryCriteria;
import cn.superdesk.uniorder.dto.param.organisation.SysOperationUserQueryCriteria;
import cn.superdesk.uniorder.dto.vo.sys.SysBusinessUserVo;
import cn.superdesk.uniorder.dto.vo.sys.SysOperationUserVo;
import cn.superdesk.uniorder.dubbo.consumer.organisation.SysBusinessUserDubboConsumerService;
import cn.superdesk.uniorder.dubbo.consumer.sys.SysUserDubboConsumerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;


@Api(value = "数据管理-业务人员)", tags = {"数据管理-业务人员"})
@Slf4j
@RestController
@RequestMapping("/businessUser")
public class SysBusinessUserController extends BaseController {

	@Autowired
	private SysBusinessUserDubboConsumerService sysBusinessUserDubboConsumerService;

	@Autowired
	private SysUserDubboConsumerService sysUserDubboConsumerService;


	@ApiOperation(value = "分页查询-业务人员", notes = "业务人员查询")
	@RequestMapping(value = "list", method = RequestMethod.POST)
	@ResponseBody
	public R<List<SysBusinessUserVo>> pageQuerySysBusinessUser(@RequestBody SysBusinessUserQueryCriteria queryCriteria) {

		log.info("SysBusinessUserController -> pageQuerySysBusinessUser:\n{}", AnJsonUtils.toJson(queryCriteria));

		R<List<SysBusinessUserVo>> result = new R<>();

		// TODO 参数合法性判定

		result = sysBusinessUserDubboConsumerService.pageQuery(queryCriteria, getToken());

		return result;
	}

	@ApiOperation(value = "操作用户下拉列表)", notes = "来源于结算中心后台创建的用户")
	@RequestMapping(value = "operationUserList", method = RequestMethod.POST)
	@ResponseBody
	public R<List<SysOperationUserVo>> pageQueryOperationUser(@RequestBody SysOperationUserQueryCriteria queryCriteria) {
		log.info("SysBusinessUserController -> pageQueryOperationUser:\n{}", AnJsonUtils.toJson(queryCriteria));
		R<List<SysOperationUserVo>> result = new R<>();
		result = sysBusinessUserDubboConsumerService.pageQueryOperationUser(queryCriteria);
		return result;
	}

	@ApiOperation(value = "按id查询")
	@RequestMapping(value = "{id}", method = RequestMethod.GET)
	@ResponseBody
	public R<List<SysBusinessUserVo>> getById(@PathVariable("id") long id) {

		R<List<SysBusinessUserVo>> result = new R<>();

		// TODO 参数合法性判定

		SysBusinessUserVo entity = sysBusinessUserDubboConsumerService.findById(id);

		List<SysBusinessUserVo> data = new ArrayList<>();
		data.add(entity);

		result.setData(data);

		return result;
	}

	@ApiOperation(value = "新增")
	@RequestMapping(value = "add", method = RequestMethod.POST)
	@ResponseBody
	public R<String> add(@RequestBody SysBusinessUserParam param) {
		log.info("addSysBusinessUserParam:\n{}", AnJsonUtils.toJson(param));
		// 当前用户
		LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(getToken());
		if (loginSysUser != null) {
			param.setCreateUserId(loginSysUser.getSysUserId());
			param.setCreateUserName(loginSysUser.getFullName());
			param.setUpdateUserId(loginSysUser.getSysUserId());
			param.setUpdateUserName(loginSysUser.getFullName());
		}
		R<String> isExistResult = validateOperationUserIsExist(param);
		if (isExistResult != null) {
			return isExistResult;
		}
		R<String> isBindResult = validateOperationUserIsBind(param);
		if (isBindResult != null) {
			return isBindResult;
		}
		sysBusinessUserDubboConsumerService.insert(param);
		return new R<>("200", "业务人员保存成功", true);
	}

	private R<String> validateOperationUserIsBind(SysBusinessUserParam param) {
		SysOperationUserQueryCriteria queryCriteria = new SysOperationUserQueryCriteria();
		queryCriteria.setId(param.getBindOperationUserid());
		R<List<SysOperationUserVo>> result = sysBusinessUserDubboConsumerService.pageQueryOperationUser(queryCriteria);
		if (result != null && CollectionUtils.isEmpty(result.getData())) {
			return new R<>("500", "当前所绑定的操作用户已绑定其他业务人员，请重新选择", false);
		}
		return null;
	}

	@ApiOperation(value = "更新")
	@RequestMapping(value = "update", method = RequestMethod.POST)
	@ResponseBody
	public R<String> update(@RequestBody SysBusinessUserParam param) {
		log.info("updateSysBusinessUserParam:\n{}", AnJsonUtils.toJson(param));
		// 当前用户
		LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(getToken());
		if (loginSysUser != null) {
			param.setUpdateUserId(loginSysUser.getSysUserId());
			param.setUpdateUserName(loginSysUser.getFullName());
		}
		R<String> x = validateOperationUserIsExist(param);
		if (x != null) {
			return x;
		}
		sysBusinessUserDubboConsumerService.updateByPrimaryKeySelective(param);
		return new R<>("200", "保存成功", true);
	}

	private R<String> validateOperationUserIsExist(SysBusinessUserParam param) {
		SysUser queryUser = new SysUser();
		queryUser.setId(param.getBindOperationUserid());
		queryUser.setDeleted(0);
		// 查询用户是否存在
		List<SysUser> checkUserList = sysUserDubboConsumerService.findUserAll(queryUser);
		if (CollectionUtils.isEmpty(checkUserList)) {
			return new R<>("500", "当前所绑定的操作用户不存在或已删除，请重新选择", false);
		}
		return null;
	}

	@ApiOperation(value = "删除-支持批量")
	@RequestMapping(value = "delete", method = RequestMethod.POST)
	@ResponseBody
	public R<String> delete(@RequestBody SysBusinessUserParam param) {
		if (StringUtils.isEmpty(param.getIds())) {
			return new R<>("500", "ids is null.", false);
		}
		// TODO 参数合法性判定
		List<String> idList = Arrays.asList(param.getIds().split(","));
		for(String id : idList) {
			SysBusinessUserVo entity = sysBusinessUserDubboConsumerService.findById(Long.valueOf(id));
			if (entity.getBindOperationUserid() != null) {
				return new R<>("500", "该业务人员已绑定操作用户，请先解绑后在删除", false);
			}
			// 逻辑删除
			param.setId(Long.valueOf(id));
			param.setDelFlag(0L);
			sysBusinessUserDubboConsumerService.updateByPrimaryKeySelective(param);
		}

//		sysBusinessUserDubboConsumerService.deleteByIds(idList.stream().
//				map(Long::valueOf).collect(Collectors.toList()));

		return new R<>("200", "删除成功", true);
	}

}
