package cn.superdesk.uniorder.controller.sys;

import cn.superdesk.uniorder.common.json.AnJsonUtils;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dto.param.system.SysAccountConfigParam;
import cn.superdesk.uniorder.dto.param.system.SysAccountConfigQueryCriteria;
import cn.superdesk.uniorder.dto.vo.sys.SysAccountConfigVo;
import cn.superdesk.uniorder.dubbo.consumer.sys.SysAccountConfigDubboConsumerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
* 共享参数配置
*
* @author zhu 270897222@qq.com
* @since 1.0.0 2022-10-28
*/
@RestController
@RequestMapping("/accountConfig")
@Api(value = "系统管理-共享参数配置)", tags = {"系统管理-共享参数配置"})
@Slf4j
public class SysAccountConfigController extends BaseController {
    @Autowired
    private SysAccountConfigDubboConsumerService sysAccountConfigDubboConsumerService;

    @ApiOperation(value = "系统管理-共享参数配置)", notes = "共享参数配置查询")
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    @ResponseBody
    public R<List<SysAccountConfigVo>> pageQuery(@RequestBody  SysAccountConfigQueryCriteria query){

        R<List<SysAccountConfigVo>> result = new R<>();

        result = sysAccountConfigDubboConsumerService.pageQuery(query, getToken());

        return result;
    }

    @ApiOperation(value = "按id查询")
    @RequestMapping(value = "{id}", method = RequestMethod.GET)
    @ResponseBody
    public R<List<SysAccountConfigVo>> getById(@PathVariable("id") long id) {

        R<List<SysAccountConfigVo>> result = new R<>();

        // TODO 参数合法性判定

        SysAccountConfigVo entity = sysAccountConfigDubboConsumerService.findById(id);

        List<SysAccountConfigVo> data = new ArrayList<>();
        data.add(entity);

        result.setData(data);

        return result;
    }

    @ApiOperation(value = "新增")
    @RequestMapping(value = "add", method = RequestMethod.POST)
    @ResponseBody
    public R<String> add(@RequestBody SysAccountConfigParam param) {
        log.info("addSysAccountConfigParam:\n{}", AnJsonUtils.toJson(param));
        // TODO 参数合法性判定
        sysAccountConfigDubboConsumerService.insert(param);

        return new R<>("200", "操作成功", true);
    }

    @ApiOperation(value = "更新")
    @RequestMapping(value = "update", method = RequestMethod.POST)
    @ResponseBody
    public R<String> update(@RequestBody SysAccountConfigParam param) {

        log.info("updateSysEnterprise:\n{}", AnJsonUtils.toJson(param));
        // 当前用户
//		LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(getToken());
        sysAccountConfigDubboConsumerService.updateByPrimaryKeySelective(param);

        return new R<>("200", "操作成功", true);
    }

    @ApiOperation(value = "删除-支持批量")
    @RequestMapping(value = "delete/{ids}", method = RequestMethod.POST)
    @ResponseBody
    public R<String> delete(@PathVariable(value = "ids") String ids) {

        // TODO 参数合法性判定
        List<String> idList = Arrays.asList(ids.split(","));
        sysAccountConfigDubboConsumerService.deleteByIds(idList.stream().
                map(Long::valueOf).collect(Collectors.toList()));

        return new R<>("200", "操作成功", true);
    }

}
