package cn.superdesk.uniorder.controller.sys;

import cn.superdesk.uniorder.common.json.AnJsonUtils;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dto.param.system.SysAccountRealityPayableParam;
import cn.superdesk.uniorder.dto.param.system.SysAccountRealityPayableQueryCriteria;
import cn.superdesk.uniorder.dto.vo.sys.SysAccountRealityPayableVo;
import cn.superdesk.uniorder.dubbo.consumer.sys.SysAccountRealityPayableDubboConsumerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
* 实付款报账单
*
* @author 阿沐 270897222@qq.com
* @since 1.0.0 2022-10-28
*/

@RestController
@RequestMapping("/accountRealityPayable")
@Api(value = "系统管理-实付款报账单)", tags = {"系统管理-实付款报账单"})
@Slf4j
public class  SysAccountRealityPayableController extends BaseController  {
    @Autowired
    private SysAccountRealityPayableDubboConsumerService sysAccountRealityPayableDubboConsumerService;
    @ApiOperation(value = "系统管理-实付款报账单)", notes = "实付款报账单查询")
    @RequestMapping(value = "list", method = RequestMethod.POST)
    @ResponseBody
    public R<List<SysAccountRealityPayableVo>> pageQuery(@RequestBody SysAccountRealityPayableQueryCriteria query){

        R<List<SysAccountRealityPayableVo>> result = new R<>();

        result = sysAccountRealityPayableDubboConsumerService.pageQuery(query, getToken());

        return result;
    }

    @ApiOperation(value = "按id查询")
    @RequestMapping(value = "{id}", method = RequestMethod.GET)
    @ResponseBody
    public R<List<SysAccountRealityPayableVo>> getById(@PathVariable("id") long id) {

        R<List<SysAccountRealityPayableVo>> result = new R<>();

        // TODO 参数合法性判定

        SysAccountRealityPayableVo entity = sysAccountRealityPayableDubboConsumerService.findById(id);

        List<SysAccountRealityPayableVo> data = new ArrayList<>();
        data.add(entity);

        result.setData(data);

        return result;
    }

    @ApiOperation(value = "新增")
    @RequestMapping(value = "add", method = RequestMethod.POST)
    @ResponseBody
    public R<String> add(@RequestBody SysAccountRealityPayableParam param) {
        log.info("addSysAccountRealityPayableParam:\n{}", AnJsonUtils.toJson(param));
        // TODO 参数合法性判定
        sysAccountRealityPayableDubboConsumerService.insert(param);

        return new R<>("200", "操作成功", true);
    }

    @ApiOperation(value = "更新")
    @RequestMapping(value = "update", method = RequestMethod.POST)
    @ResponseBody
    public R<String> update(@RequestBody SysAccountRealityPayableParam param) {

        log.info("updateSysEnterprise:\n{}", AnJsonUtils.toJson(param));
        // 当前用户
        param.setDataIsComplete(1);
        sysAccountRealityPayableDubboConsumerService.updateByPrimaryKeySelective(param,getToken());

        return new R<>("200", "操作成功", true);
    }

    @ApiOperation(value = "删除-支持批量")
    @RequestMapping(value = "delete/{ids}", method = RequestMethod.POST)
    @ResponseBody
    public R<String> delete(@PathVariable(value = "ids") String ids) {

        // TODO 参数合法性判定
        List<String> idList = Arrays.asList(ids.split(","));
        sysAccountRealityPayableDubboConsumerService.deleteByIds(idList.stream().
                map(Long::valueOf).collect(Collectors.toList()));

        return new R<>("200", "操作成功", true);
    }
}



