package cn.superdesk.uniorder.controller.sys;

import cn.superdesk.uniorder.common.json.AnJsonUtils;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dto.param.system.SysAccountReceivableParam;
import cn.superdesk.uniorder.dto.param.system.SysAccountReceivableQueryCriteria;
import cn.superdesk.uniorder.dto.vo.sys.SysAccountReceivableVo;
import cn.superdesk.uniorder.dubbo.consumer.sys.SysAccountReceivableDubboConsumerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
* 应收报账单
*
* @author zhu 270897222@qq.com
* @since 1.0.0 2022-10-28
*/
@RestController
@RequestMapping("/accountReceivable")
@Api(value = "系统管理-应收报账单)", tags = {"系统管理-应收报账单"})
@Slf4j
public class SysAccountReceivableController  extends BaseController {
    @Autowired
    private SysAccountReceivableDubboConsumerService sysAccountReceivableDubboConsumerService;

    @ApiOperation(value = "系统管理-应收报账单)", notes = "应收报账单查询")
    @RequestMapping(value = "list", method = RequestMethod.POST)
    @ResponseBody
    public R<List<SysAccountReceivableVo>> pageQuery(@RequestBody SysAccountReceivableQueryCriteria query){

        R<List<SysAccountReceivableVo>> result = new R<>();

        result = sysAccountReceivableDubboConsumerService.pageQuery(query, getToken());

        return result;
    }

    @ApiOperation(value = "按id查询")
    @RequestMapping(value = "{id}", method = RequestMethod.GET)
    @ResponseBody
    public R<List<SysAccountReceivableVo>> getById(@PathVariable("id") long id) {

        R<List<SysAccountReceivableVo>> result = new R<>();

        // TODO 参数合法性判定

        SysAccountReceivableVo entity = sysAccountReceivableDubboConsumerService.findById(id);

        List<SysAccountReceivableVo> data = new ArrayList<>();
        data.add(entity);

        result.setData(data);

        return result;
    }

    @ApiOperation(value = "新增")
    @RequestMapping(value = "add", method = RequestMethod.POST)
    @ResponseBody
    public R<String> add(@RequestBody SysAccountReceivableParam param) {
        log.info("addSysAccountReceivableParam:\n{}", AnJsonUtils.toJson(param));
        // TODO 参数合法性判定
        sysAccountReceivableDubboConsumerService.insert(param);

        return new R<>("200", "操作成功", true);
    }

    @ApiOperation(value = "更新")
    @RequestMapping(value = "update", method = RequestMethod.POST)
    @ResponseBody
    public R<String> update(@RequestBody SysAccountReceivableParam param) {

        log.info("updateSysEnterprise:\n{}", AnJsonUtils.toJson(param));
        // 当前用户
        param.setDataIsComplete(1);
        sysAccountReceivableDubboConsumerService.updateByPrimaryKeySelective(param,getToken());

        return new R<>("200", "操作成功", true);
    }

    @ApiOperation(value = "删除-支持批量")
    @RequestMapping(value = "delete/{ids}", method = RequestMethod.POST)
    @ResponseBody
    public R<String> delete(@PathVariable(value = "ids") String ids) {

        // TODO 参数合法性判定
        List<String> idList = Arrays.asList(ids.split(","));
        sysAccountReceivableDubboConsumerService.deleteByIds(idList.stream().
                map(Long::valueOf).collect(Collectors.toList()));

        return new R<>("200", "操作成功", true);
    }
}
