package cn.superdesk.uniorder.controller.sys;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.superdesk.uniorder.common.HttpClientUtils;
import cn.superdesk.uniorder.common.enums.common.CommonSoftDeleteEnum;
import cn.superdesk.uniorder.common.enums.common.CommonSourceFromEnum;
import cn.superdesk.uniorder.common.exception.UnifiedOrderRuntimeException;
import cn.superdesk.uniorder.common.exception.utils.ExceptionUtils;
import cn.superdesk.uniorder.common.json.AnJsonUtils;
import cn.superdesk.uniorder.common.numerical.AnConvertBeanUtil;
import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.common.rest.response.ResponseResultOuterUtils;
import cn.superdesk.uniorder.common.security.crypt.RSAUtils;
import cn.superdesk.uniorder.controller.base.BaseController;
import cn.superdesk.uniorder.dao.entity.organisation.SysCompany;
import cn.superdesk.uniorder.dao.entity.sys.SysUser;
import cn.superdesk.uniorder.dao.entity.sys.SysUserOrderSourcetypeKey;
import cn.superdesk.uniorder.dao.entity.sys.SysUserResetPassword;
import cn.superdesk.uniorder.dto.param.system.SysUserQueryCriteria;
import cn.superdesk.uniorder.dto.vo._superdeskapp.jwt.SupperdeskAppResponse;
import cn.superdesk.uniorder.dto.vo._superdeskapp.jwt.SupperdeskAppTokenUser;
import cn.superdesk.uniorder.dto.vo.sys.SysUserModify;
import cn.superdesk.uniorder.dto.vo.sys.SysUserView;
import cn.superdesk.uniorder.dto.vo.sys.SysUserVo;
import cn.superdesk.uniorder.dubbo.consumer.sys.SysUserDubboConsumerService;
import com.fasterxml.jackson.core.type.TypeReference;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.DigestUtils;
import org.springframework.web.bind.annotation.*;

import javax.imageio.ImageIO;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.List;
import java.util.*;

@Api(value = "系统管理-系统用户", tags = {"系统管理-系统用户"})
@Slf4j
@RestController
@RequestMapping("/user")
public class SysUserController extends BaseController {


    @Autowired
    private SysUserDubboConsumerService sysUserDubboConsumerService;

    /**
     * 判断字符串是不是日期格式字符串
     *
     * @param date 日期字符串
     * @return 符合日期格式返回true, 否则返回false
     */
    public static boolean isDateFormat(String date) {
        String dateRegex = "^[0-9]{4}-[0-9]{2}-[0-9]{2}$";// 2016-09-02格式

        return date.matches(dateRegex);
    }

    // unified.out.tokenurl=http://superdesk.avic-s.com/super_cloud/api/organisation/wechat/front/user/getVirAndUserInfo
    // unified.out.saleurl=http://superdesk.avic-s.com/super_cloud/api/organisation/wechat/front/merchant/findMerchantById
    @Value("${unified.out.tokenurl}")
    private String TOKENURL;


    private R<SysUserVo> checkLoginStatus(String username, String password, String localIp) {
        SysUser queryUser = new SysUser();
        queryUser.setUserName(username);
        queryUser.setDeleted(0);
        // 查询用户是否存在
        List<SysUser> checkUserList = sysUserDubboConsumerService.findUserAll(queryUser);
        R<SysUserVo> response = new R<>();
        if (CollUtil.isEmpty(checkUserList)) {
            response.setSuccess(false);
            response.setMessage("账户或密码错误");
            return response;
        }
        // 判断账户是否禁用
        SysUser currentSysUser = checkUserList.get(0);
        if (currentSysUser.getEnabled() == 0) {
            response.setSuccess(false);
            response.setMessage("该账号已被禁用,请联系管理员");
            return response;
        }
        // 判断用户名密码登陆
        SysUserVo sysUserVo = sysUserDubboConsumerService.userLoginBySupperDeskUniOrderBackEndFacade(username, password);
        if (sysUserVo == null) {
            // 用户名或密码错误，登陆失败处理
            sysUserVo = new SysUserVo();
            BeanUtils.copyProperties(currentSysUser, sysUserVo);
            loginFailedHandle(response, currentSysUser);
        } else if (sysUserVo.getIsLock() != null
                && sysUserVo.getIsLock() == 0
                && sysUserVo.getLockEndTime().getTime() > new Date().getTime()) {
            // 锁定状态并且未解锁，登陆失败处理
            loginFailedHandle(response, currentSysUser);
        } else {
            // 登陆成功处理
            loginSuccessHandle(response, currentSysUser);
            response.setData(sysUserVo);
        }

        // 记录登陆日志
        sysUserDubboConsumerService.createLog(currentSysUser.getId(), request.getRemoteAddr(), localIp);
        return response;
    }

    /**
     * 处理登陆成功逻辑
     *
     * @param response
     * @param currentSysUser
     */
    private void loginSuccessHandle(R<SysUserVo> response, SysUser currentSysUser) {
        currentSysUser.setIsLock(1);
        currentSysUser.setLoginFailCount(0);
        currentSysUser.setLockStartTime(null);
        currentSysUser.setLockEndTime(null);
        response.setSuccess(true);
        response.setMessage("登录成功");
        // 更新用户相关信息
        sysUserDubboConsumerService.edit(currentSysUser, null, "");
    }

    /**
     * 处理登陆失败逻辑
     *
     * @param response
     * @param currentSysUser
     */
    private void loginFailedHandle(R<SysUserVo> response, SysUser currentSysUser) {
        response.setSuccess(false);
        // 失败次数加1
        int loginFailCount = (currentSysUser.getLoginFailCount() == null ? 0 : currentSysUser.getLoginFailCount()) + 1;
        currentSysUser.setLoginFailCount(loginFailCount);
        if (loginFailCount <= 2) {
            response.setMessage("账户或密码错误");
        } else if (loginFailCount <= 4) {
            response.setMessage("今日登录失败次数已达" + loginFailCount + "次，达5次账号将被系统锁定");
        } else if (loginFailCount == 5) {
            setUserLock(response, currentSysUser, 5);
        } else if (loginFailCount == 6) {
            setUserLock(response, currentSysUser, 10);
        } else if (loginFailCount == 7) {
            setUserLock(response, currentSysUser, 30);
        } else if (loginFailCount == 8) {
            setUserLock(response, currentSysUser, 45);
        } else if (loginFailCount == 9) {
            setUserLock(response, currentSysUser, 60);
        } else if (loginFailCount == 10) {
            setUserLock(response, currentSysUser, 120);
        } else if (loginFailCount > 10) {
            response.setSuccess(false);
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm");
            String lockEndTime = sdf.format(currentSysUser.getLockEndTime());
            response.setMessage("该账号已锁定，请于" + lockEndTime + "后重试");
            return;
        }
        log.info("登录密码错误{}次", loginFailCount);
        // 更新用户相关信息
        if(loginFailCount<=10){
            sysUserDubboConsumerService.edit(currentSysUser, null, "");
        }

    }

    /**
     * 密码错误次数超过5次设置锁定
     *
     * @param currentSysUser
     * @param response
     * @param lockMinute
     */
    private void setUserLock(R<SysUserVo> response, SysUser currentSysUser, long lockMinute) {
        if (currentSysUser.getLoginFailCount() >= 5) {
            // 初次锁定或者锁定时间小于当前时间，设置锁定开始时间
            if (currentSysUser.getIsLock() != 0 || currentSysUser.getLockEndTime().getTime() < new Date().getTime()) {
                currentSysUser.setLockStartTime(new Date());
            }
            currentSysUser.setIsLock(0);
            currentSysUser.setLockEndTime(new Date(currentSysUser.getLockStartTime().getTime() + lockMinute * 60 * 1000));
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm");
            String lockEndTime = sdf.format(currentSysUser.getLockEndTime());
            response.setMessage("账号锁定" + lockMinute + "分钟，请于" + lockEndTime + "后重试，以免叠加锁定");
        }
    }

    /**
     * 登陆
     *
     * @param username
     * @param password
     * @return
     * @throws ServletException
     * @throws UnifiedOrderRuntimeException
     */
    @SneakyThrows
    @PostMapping("/login")
    @ResponseBody
    public R<SysUserVo> login(String username, String password, String localIp, String verificationCode,
                              HttpServletRequest request,
                              HttpSession session) throws ServletException, UnifiedOrderRuntimeException {

        log.info("params : \nusername:{}\npassword:{}\nlocalIp:{}\nverificationCode:{}\nsessionId:{}\n",
                username, password, localIp, verificationCode, session.getId()
        );
        R<SysUserVo> response = new R<>();

        if (StringUtils.isEmpty(username) || StringUtils.isEmpty(password)) {
            response.setSuccess(false);
            response.setMessage("账号或密码为空。");
            return response;
        }
        if (StringUtils.isBlank(verificationCode)) {
            response.setSuccess(false);
            response.setMessage("请输入验证码");
            return response;
        }

        if (StringUtils.isNotEmpty(verificationCode)) {

            String sessionVerificationCode = "";

            Object verificationCodeObj = session.getAttribute("verificationCode");

            if (null != verificationCodeObj) {
                sessionVerificationCode = verificationCodeObj.toString();
            } else {
                response.setSuccess(false);
                response.setMessage("验证码已过期,请刷新后重试");
                // response.setMessage("服务器 获取 验证码 失败, 前端没有携带之前的cookie信息, 请求请加上 withCredentials:true | " + sessionVerificationCode);
                return response;
            }
            log.info("\nlogin\nsessionId:{}\nsessionVerificationCode:{}", session.getId(), sessionVerificationCode);

            // 验证码使用一次就失效
            session.setAttribute("verificationCode", "PQRSTUVWXYZ");


            if (StringUtils.isNotEmpty(sessionVerificationCode)) {

                if (!sessionVerificationCode.equalsIgnoreCase(verificationCode)) {

                    response.setSuccess(false);
                    response.setMessage("验证码输入错误");
                    return response;
                }
            }
        }

        username = RSAUtils.decryptBase64(username);
        password = RSAUtils.decryptBase64(password);

        /**密码MD5加密***/
        String passwordMd5 = DigestUtils.md5DigestAsHex(password.getBytes());
        /**新增登录判断***/
        response = checkLoginStatus(username, passwordMd5, localIp);
        return response;

    }

    //@ApiOperation(value = "获取公钥")
    @PostMapping("/getKey")
    @ResponseBody
    //@RequestMapping(value = "/getKey", method = RequestMethod.POST)
    public R getKey(HttpServletRequest request) {
        log.info("SysUserController.getKey");
        String publicKey = RSAUtils.generateBase64PublicKey();
        return new R<>(publicKey);
    }

    //dz add 20190901 token登陆
    @PostMapping("/tokenlogin")
    @ResponseBody
    public R<SysUserVo> loginByToken(String token) throws ServletException, UnifiedOrderRuntimeException, IOException {

        R<SysUserVo> response = new R<>();

        if (StringUtils.isEmpty(token)) {
            response.setSuccess(false);
            response.setMessage("token 令牌为空");
            return response;
        }

        Map<String, String> params = new HashMap<>();
        params.put("userToken", token);

        // 请求服务之窗
        String responseJsonStr = HttpClientUtils.doPost(TOKENURL, params);

//		http://192.168.6.223:8282/#/login?token=64a927ae243535900589848e173359c8
//		responseJsonStr = FileUtil.readUtf8String("/Users/linjinyu/JAVA_HOME/workspace_superdesk/superdesk-uniorder/superdesk-uniorder-web/src/main/java/cn/superdesk/uniorder/controller/sys/yyds.json");


        /****************************************** 重构 mark by linjinyu at 20210615 start ******************************************/
//		Map jsonMap = JSON.parseObject(jsonstr, Map.class);
//
//		log.info("\n\n\n\nloginByToken response \n{}\n\n\n\n", jsonstr);
//
//		if (!jsonMap.get("code").toString().equals("200")) {
//			response.setSuccess(false);
//			response.setMessage("token 验证失败" + jsonMap.get("msg").toString());
//			return response;
//		}
//
//		log.info("token 解析:" + jsonstr);
//
//		// 服务之窗返回 结果 转 bean
//		TokenUserBySuperdeskApp tokenUserByService = JSONObject.parseObject(jsonMap.get("data").toString(), TokenUserBySuperdeskApp.class);
        /****************************************** 重构 mark by linjinyu at 20210615 start ******************************************/


        SupperdeskAppResponse<SupperdeskAppTokenUser> supperdeskAppResponse = AnJsonUtils.getMapper().readValue(responseJsonStr, new TypeReference<SupperdeskAppResponse<SupperdeskAppTokenUser>>() {
        });

        log.info("\n返回\n{}\n转换\n{}\n", responseJsonStr, AnJsonUtils.toJson(supperdeskAppResponse));

        if (null == supperdeskAppResponse.getCode() || !supperdeskAppResponse.getCode().equals(200)) {
            response.setSuccess(false);
            response.setMessage("token 验证失败" + responseJsonStr);
            return response;
        }

        SupperdeskAppTokenUser supperdeskAppTokenUser = supperdeskAppResponse.getData();

        SysUserVo responseVo = sysUserDubboConsumerService.userLoginBySupperDeskUniServiceTokenFacade(supperdeskAppTokenUser);

        if (null == responseVo) {
            response.setSuccess(false);
            response.setMessage("您还未开通运营商项目和业务权限,请联系管理员配置");
            return response;
        }

        if (supperdeskAppTokenUser.getTokenType() == null) {
            response.setSuccess(false);
            response.setMessage("tokenType 不能为 null");
            return response;
        }

        if (supperdeskAppTokenUser.getTokenType().equals(2) && responseVo.getSysRoleExtDataId() == null) {
            response.setSuccess(false);
            response.setMessage("您没有权限,或系统中无关联【数据权限】角色。请联系管理员！");
            return response;
        }

        if (supperdeskAppTokenUser.getTokenType().equals(2) && responseVo.getRoleId() == null) {
            response.setSuccess(false);
            response.setMessage("您没有权限,或系统中无关联【菜单权限】角色。请联系管理员！");
            return response;
        }

        responseVo.setTokenUserByService(supperdeskAppTokenUser);
        response.setData(responseVo);

        return response;

    }

    @ApiOperation(value = "系统(后台)用户列表-分页")
    @PostMapping("/findAllByPage")
    @ResponseBody
    public R<List<SysUserView>> findAllByPage(@RequestBody SysUserQueryCriteria queryCriteria) {

        R<List<SysUserView>> result = new R<>();

        /**************************************** 默认 start ****************************************/
          // 页面需要禁用和启用的所有用户
//        if (ObjectUtil.isNull(queryCriteria.getEnabled())) {
//            queryCriteria.setEnabled(CommonEnabledEnum.启用.getKey());
//        }

        if (ObjectUtil.isNull(queryCriteria.getDeleted())) {
            queryCriteria.setDeleted(CommonSoftDeleteEnum.正常.getKey());
        }

        String[] sourceFromContains = {
                CommonSourceFromEnum.结算中心.getSourceFromCode()
        };

        if (CollUtil.isEmpty(queryCriteria.getSourceFromsSelected())) {
            queryCriteria.setSourceFromsSelected(Arrays.asList(sourceFromContains));
        }


        /**************************************** 默认   end ****************************************/

        try {


            result = sysUserDubboConsumerService.findAllPage(queryCriteria);

        } catch (Exception e) {

            result.setCode(String.valueOf(e.hashCode()));
            result.setSuccess(false);
            result.setMessage("系统查找失败！");

            log.error(ExceptionUtils.getMessage(e));
            e.printStackTrace();
        }

        return result;
    }

    @PostMapping("/refreshToken")
    @ResponseBody
    public R<SysUserVo> refreshToken(String refreshToken) throws ServletException {

        R<SysUserVo> result = new R<SysUserVo>();

//		if (StringUtils.isEmpty(refreshToken)) {
//			result.setSuccess(false);
//			result.setMessage("refreshToken为空。");
//			return result;
//		}
//
//		String userId = JwtUtil.checkToken(refreshToken);
//
//		if (StringUtils.isEmpty(userId)) {
//			result.setSuccess(false);
//			result.setMessage("token expired");
//		} else {
//			Object objRefreshToken = redisService.hgetKey(RedisKeyPrefix.TOKEN_CACHE.getKey() + ":" + userId,
//					"refreshToken");
//
//			if (objRefreshToken == null || !refreshToken.equals(objRefreshToken.toString())) {
//				result.setSuccess(false);
//				result.setMessage("token expired");
//			} else {
//				// refresh token
//				SysUserVO user = service.refreshToken(userId);
//				result.setData(user);
//			}
//		}

        return result;
    }

    /**
     * 系統管理 > 用户管理 > op添加
     * 新增或修改接口
     *
     * @param model
     * @return
     */
    @PostMapping("/saveOrUpdate")
    @ResponseBody
    public R<?> saveOrUpdate(SysUserModify model) {

        R<?> result = new R<>();

        try {


            // 为了不用改
            List<SysUserOrderSourcetypeKey> sysUserOrderSourcetypeKeys = new ArrayList<SysUserOrderSourcetypeKey>();

//			if (!StringUtils.isEmpty(model.getSourcetypes())) {
//
//				String[] types = model.getSourcetypes().split(",");
//
//				for (int i = 0; i < types.length; i++) {
//					SysUserOrderSourcetypeKey key = new SysUserOrderSourcetypeKey();
//					key.setSourcetypeId(Long.valueOf(types[i]));
//					ls.add(key);
//				}
//			}

            SysUser sysUser = AnConvertBeanUtil.copyBean(model, SysUser.class);
            //密码加密
            if (StringUtils.isNotEmpty(sysUser.getUserPassword())) {
                String userPassWordMd5 = DigestUtils.md5DigestAsHex(sysUser.getUserPassword().getBytes());
                sysUser.setUserPassword(userPassWordMd5);
            }

            //用户名查询用户
            List<SysUser> sysUserList = null;
            if (StringUtils.isNotEmpty(model.getUserName())) {
                SysUser userParam = new SysUser();
                userParam.setUserName(model.getUserName());
                sysUserList = sysUserDubboConsumerService.findUserAll(userParam);
            }

            if (StrUtil.isBlank(sysUser.getSourceFrom())) {
                sysUser.setSourceFrom(CommonSourceFromEnum.结算中心.getSourceFromCode());
            }

            if (model.getId() == null) {
                // 新增
                if (CollectionUtils.isNotEmpty(sysUserList)) {
                    result.setSuccess(false);
                    result.setMessage("用户名已存在，请重新输入！");
                    return result;
                }
                // 校验密码
                String str = CheckPasswordUtils.checkPasswordRule(model.getUserPassword(), model.getUserName());
                if (StringUtils.isNotEmpty(str)) {
                    return new R("100", str, false);
                }
                sysUser.setIsLock(1);
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(new Date());
                calendar.add(Calendar.DATE, 90);
                sysUser.setUpdatePasswordTime(new Date());
                //初始修改密码时间90天后
                sysUser.setNextUpdatePasswordTime(calendar.getTime());
                sysUserDubboConsumerService.add(sysUser, sysUserOrderSourcetypeKeys, getToken());
                result.setSuccess(true);
            } else {
                // 编辑用户需要校验：用户名全局唯一，不可重复，含已删除的用户，不含当前用户***/
                if (CollectionUtils.isNotEmpty(sysUserList)
                        && !Objects.equals(sysUserList.get(0).getId(),model.getId())) {
                    result.setSuccess(false);
                    result.setMessage("用户名已存在，请重新输入！");
                    return result;
                }
                sysUserDubboConsumerService.edit(sysUser, sysUserOrderSourcetypeKeys, getToken());
                result.setSuccess(true);
            }

        } catch (Exception e) {
            e.printStackTrace();
        }

        return result;
    }

    @PostMapping("/updateUserStatus")
    @ResponseBody
    public R<?> updateUserStatus(SysUserModify sysUserModify) {
        try {
            List<String> ids = new ArrayList<>();
            List<SysUserOrderSourcetypeKey> sysUserOrderSourcetypeKeys = new ArrayList<SysUserOrderSourcetypeKey>();
            if(StringUtils.isNotEmpty(sysUserModify.getIds())) {
                ids = Arrays.asList(sysUserModify.getIds().split(","));
            }
            ids.stream().forEach(id -> {
                SysUser sysUser = AnConvertBeanUtil.copyBean(sysUserModify, SysUser.class);
                sysUser.setId(Long.valueOf(id));
                // 解锁，设置失败次数为0
                if (sysUser.getIsLock() != null && sysUser.getIsLock() == 1) {
                    sysUser.setLoginFailCount(0);
                }
                sysUserDubboConsumerService.edit(sysUser, sysUserOrderSourcetypeKeys, getToken());
            });
        } catch (Exception e) {
            e.printStackTrace();
            return new R<>("500", "更新失败", false);
        }
        return new R<>("200", "更新成功", true);
    }

    @Deprecated
    @ApiModelProperty(value = "物理删除 改 逻辑删除 软删除 物理删除-弃用")
    @PostMapping("/delete")
    public R<String> delete() {

//        return ResponseResultOuterUtils.createErrorResult("500", "弃用!请使用新接口");

        try {

            Long sysUserId = getLong("id");

            int ret = sysUserDubboConsumerService.opLogicalDelete(sysUserId, getToken());
            if (ret > 0) {
                return ResponseResultOuterUtils.createSuccesResult();
            } else {
                return ResponseResultOuterUtils.createErrorResult("500", "操作失败");
            }
        } catch (Exception e) {

            log.error(ExceptionUtils.getMessage(e));
            e.printStackTrace();
            return ResponseResultOuterUtils.createErrorResult("500", StrUtil.format("{}:{}", e.hashCode(), e.getMessage()));
        }

    }

    @ApiOperation(value = "物理删除-系统管理-用户管理", notes = "积分结算2.1")
    @PostMapping("/opPhysicsDelete/{sysUserId}")
    @ResponseBody
    public R<String> opPhysicsDelete(@PathVariable Long sysUserId) {

        try {
            int ret = sysUserDubboConsumerService.opPhysicsDelete(sysUserId, getToken());
            if (ret > 0) {
                return ResponseResultOuterUtils.createSuccesResult();
            } else {
                return ResponseResultOuterUtils.createErrorResult("500", "操作失败");
            }

        } catch (Exception e) {
            log.error(ExceptionUtils.getMessage(e));
            e.printStackTrace();
            return ResponseResultOuterUtils.createErrorResult("500", StrUtil.format("{}:{}", e.hashCode(), e.getMessage()));
        }
    }

    @ApiOperation(value = "逻辑删除-系统管理-用户管理", notes = "积分结算2.1")
    @PostMapping("/opLogicalDelete/{sysUserId}")
    @ResponseBody
    public R<String> opLogicalDelete(@PathVariable Long sysUserId) {

        try {

            int ret = sysUserDubboConsumerService.opLogicalDelete(sysUserId, getToken());
            if (ret > 0) {
                return ResponseResultOuterUtils.createSuccesResult();
            } else {
                return ResponseResultOuterUtils.createErrorResult("500", "操作失败");
            }

        } catch (Exception e) {
            log.error(ExceptionUtils.getMessage(e));
            e.printStackTrace();
            return ResponseResultOuterUtils.createErrorResult("500", StrUtil.format("{}:{}", e.hashCode(), e.getMessage()));
        }

    }


//	@PostMapping("/loginByToken")
//	@ResponseBody
//	public ResponseResult<SysUserVO> loginByToken(String token) throws MyException {
//
//		return new ResponseResult<>(service.loginByToken(token));
//	}

//	@PostMapping("/listSysUser")
//	public ResponseResult<List<SysUserQueryVO>> listSysUser(@RequestBody SysUserParamsVO vo) throws MyException {
//		return service.listSysUser(vo);
//	}
//
//	@PostMapping("/getSysUser")
//	public ResponseResult<SysUserEditVO> getSysUser(@RequestBody SysUserParamsVO vo) throws MyException {
//		return service.getSysUser(vo);
//	}

    /***
     * 检查用户电话是否存在
     *
     * @return
     */
    @RequestMapping("/ckUserExist")
    @ResponseBody
    public R<SysUser> checkUserExistByContactNumber(String contactNumber) {

        R<SysUser> result = new R<>();

//		try {
//			SysUser user = service.findByContact(contactNumber);
//			if (user != null) {
//				result.setData(user);
//				result.setMessage("用户联系电话已存在,请重新输入！");
//				result.setSuccess(false);
//			} else {
//				result.setSuccess(true);
//			}
//
//		} catch (Exception e) {
//
//			result.setMessage("获取数据失败！");
//			result.setSuccess(false);
//		}

        return result;

    }

    /**
     * 返回平台方公司信息-目前写死
     *
     * @return
     */
    @PostMapping("/getSysCompany")
    @ResponseBody
    public R<SysCompany> getSysCompany() {

        R<SysCompany> result = new R<>();

        try {

            SysCompany company = sysUserDubboConsumerService.getSysCompany(1L);

            result.setSuccess(true);
            result.setData(company);

        } catch (Exception e) {

            result.setCode(String.valueOf(e.hashCode()));
            result.setSuccess(false);
            result.setMessage("系统查找失败！");

            log.error(ExceptionUtils.getMessage(e));
            e.printStackTrace();
        }
        return result;
    }


    private static final String CODE_RANGE = "23456789ABDEFHJKLMNPQRSTUVWXYZ";

    /**
     * 生成验证码
     *
     * @return
     * @throws IOException
     */
    @RequestMapping(value = "/imageCode", method = RequestMethod.POST)
    public String imageCode(HttpServletRequest request, HttpServletResponse response, HttpSession session) throws IOException {
        // 在内存中创建图象
        int width = 65, height = 20;
        BufferedImage image = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
        // 获取图形上下文
        Graphics g = image.getGraphics();
        // 生成随机类
        Random random = new Random();
        // 设定背景色
        g.setColor(getRandColor(230, 255));
        g.fillRect(0, 0, 100, 25);
        // 设定字体
        g.setFont(new Font("Arial", Font.CENTER_BASELINE | Font.ITALIC, 18));
        // 产生0条干扰线,
        g.drawLine(0, 0, 0, 0);
        // 取随机产生的认证码(4位数字)
        String sRand = "";
        for (int i = 0; i < 4; i++) {
            String rand = CODE_RANGE.charAt((int) (Math.random() * CODE_RANGE.length())) + "";
            sRand += rand;
            // 将认证码显示到图象中
            g.setColor(getRandColor(100, 150));// 调用函数出来的颜色相同,可能是因为种子太接近,所以只能直接生成
            g.drawString(rand, 15 * i + 6, 16);
        }
        for (int i = 0; i < (random.nextInt(5) + 5); i++) {
            g.setColor(new Color(random.nextInt(255) + 1, random.nextInt(255) + 1, random.nextInt(255) + 1));
            g.drawLine(random.nextInt(100), random.nextInt(30), random.nextInt(100), random.nextInt(30));
        }
        //String superId = request.getParameter("super");
        //String key = superId + "_checkCode";
        // 将验证码存入页面KEY值的SESSION里面
        session.setAttribute("verificationCode", sRand);

        log.info("\nimageCode\nsessionId:{}\nsessionVerificationCode:{}", session.getId(), session.getAttribute("verificationCode"));

//		log.info("session.setAttribute: verificationCode = {}", sRand);

        //session.setAttribute("keys", sRand);
        // 图象生效
        g.dispose();
        ServletOutputStream responseOutputStream = response.getOutputStream();
        // 输出图象到页面
        ImageIO.write(image, "JPEG", responseOutputStream);
        // 以下关闭输入流！
        responseOutputStream.flush();
        responseOutputStream.close();
        // 获得页面key值
        return null;
    }

    /**
     * 给定范围获得随机颜色
     *
     * @param fc
     * @param bc
     * @return
     */
    Color getRandColor(int fc, int bc) {
        Random random = new Random();
        if (fc > 255)
            fc = 255;
        if (bc > 255)
            bc = 255;
        int r = fc + random.nextInt(bc - fc);
        int g = fc + random.nextInt(bc - fc);
        int b = fc + random.nextInt(bc - fc);
        return new Color(r, g, b);
    }

    @ApiOperation(value = "修改密码")
    @RequestMapping(value = "/updatePassword", method = RequestMethod.POST)
    @ResponseBody
    public R<String> updatePwd(@RequestBody SysUserResetPassword user) {
//        try {
//         String   newPassWd = RSAUtils.decryptBase64(user.getUserPassword());
//         String   oldPassWd = RSAUtils.decryptBase64(user.getOldPassword());
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
        String newPassword = user.getUserPassword();
        String oldPassWord = user.getOldPassword();

        SysUser sUser = new SysUser();
        sUser.setId(user.getId());
        if (StringUtils.isNotEmpty(oldPassWord)) {
            String oldPwd = DigestUtils.md5DigestAsHex(oldPassWord.getBytes());
            sUser.setUserPassword(oldPwd);
        }

        List<SysUser> bySysUser = sysUserDubboConsumerService.findUserAll(sUser);
        SysUser currentSysUser;
        if (CollectionUtils.isEmpty(bySysUser)) {
            return new R("100","原密码不正确，请重新输入！",false);
        } else {
            currentSysUser = bySysUser.get(0);
        }
        String newPasswordMd5 = DigestUtils.md5DigestAsHex(newPassword.getBytes());
        if (newPasswordMd5.equals(currentSysUser.getUserPassword())) {
            return new R("100","新密码与原密码不能相同，请重新输入",false);
        }
        String str = CheckPasswordUtils.checkPasswordRule(newPassword, currentSysUser.getUserName());
        if (StringUtils.isNotEmpty(str)) {
            return new R("100", str, false);
        }
        user.setUserPassword(newPasswordMd5);
        SysUser sysUser = AnConvertBeanUtil.copyBean(user, SysUser.class);
        Calendar calendar = Calendar.getInstance();
        //设置当前时间
        calendar.setTime(new Date());
        //增加90天
        calendar.add(Calendar.DATE, 90);
        sysUser.setUpdatePasswordTime(new Date());
        sysUser.setNextUpdatePasswordTime(calendar.getTime());
        sysUser.setIsLock(1);
        List<SysUserOrderSourcetypeKey> sysUserOrderSourcetypeKeys = new ArrayList<SysUserOrderSourcetypeKey>();
        sysUserDubboConsumerService.edit(sysUser, sysUserOrderSourcetypeKeys, getToken());
        // 记录修改密码日志
        sysUserDubboConsumerService.updatePasswordLog(currentSysUser.getId(), request.getRemoteAddr(), "");
        return  new R("200","修改密码成功",true);
    }


}
