package cn.superdesk.uniorder.dubbo.consumer.biz;

import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.dao.entity.biz.BizContract;
import cn.superdesk.uniorder.dto.param.biz.BizContractParam;
import cn.superdesk.uniorder.dubbo.facade.biz.IBizContractService;
import com.alibaba.dubbo.config.annotation.Reference;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.List;

@Slf4j
@Component
public class BizContractDubboConsumerService {

    @Reference(version = "1.0.0", timeout = 30000, retries = -1)
    private IBizContractService bizContractService;

    /**
     * 查询合同信息
     *
     * @param id 合同信息主键
     * @return 合同信息
     */
    public BizContract selectBizContractById(Long id){
        return bizContractService.selectBizContractById(id);
    }

    /**
     * 合同列表
     * @param bizContract 分页参数
     * @return List
     */
    public R<List<BizContract>> getListPage(BizContractParam bizContract){
        return bizContractService.getListPage(bizContract);
    }

    /**
     * 查询合同信息列表
     *
     * @param bizContract 合同信息
     * @return 合同信息集合
     */
    public List<BizContract> selectBizContractList(BizContract bizContract){
        return bizContractService.selectBizContractList(bizContract);
    }

    /**
     * 新增合同信息
     *
     * @param bizContract 合同信息
     * @return 结果
     */
    public int insertBizContract(BizContract bizContract){
        return bizContractService.insertBizContract(bizContract);
    }

    /**
     * 修改合同信息
     *
     * @param bizContract 合同信息
     * @return 结果
     */
    public int updateBizContract(BizContract bizContract){
        return bizContractService.updateBizContract(bizContract);
    }

    /**
     * 修改合同设置默认
     * @param bizContractRequest 修改参数
     */
    public int defaultYes(BizContract bizContractRequest){
        if("Y".equals(bizContractRequest.getIsDefault())){
            bizContractService.updateAllDefaultNo();
        }
        return bizContractService.updateBizContract(bizContractRequest);
    }

    /**
     * 批量删除合同信息
     *
     * @param ids 需要删除的合同信息主键集合
     * @return 结果
     */
    public int deleteBizContractByIds(Long[] ids){
        return bizContractService.deleteBizContractByIds(ids);
    }

    /**
     * 删除合同信息信息
     *
     * @param id 合同信息主键
     * @return 结果
     */
    public int deleteBizContractById(Long id){
        return bizContractService.deleteBizContractById(id);
    }
}
