package cn.superdesk.uniorder.dubbo.consumer.invoice.reportaccounting;

import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.common.security.JwtUtil;
import cn.superdesk.uniorder.common.security.dto.LoginSysUser;
import cn.superdesk.uniorder.dto.param.invoice.customer.reportaccounting.*;
import cn.superdesk.uniorder.dto.vo.invoice.reportaccounting.ActualReceivedReportAccountingDetailVO;
import cn.superdesk.uniorder.dto.vo.invoice.reportaccounting.ActualReceivedReportAccountingVO;
import cn.superdesk.uniorder.dto.vo.invoice.reportaccounting.ActualReportAccountingPreCheckVO;
import cn.superdesk.uniorder.dubbo.facade.invoice.customer.DubboReportAccountingService;
import com.alibaba.dubbo.config.annotation.Reference;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

@Slf4j
@Component
public class ReportAccountingDubboConsumerService {

    @Reference(version = "1.0.0", timeout = 300000, retries = -1)
    private DubboReportAccountingService dubboReportAccountingService;

    /**
     * opt 修复账单回款状态
     *
     */
    public void repairInvoiceStatus(InvoiceStatusRepair repair, String token) {

        LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(token);
        dubboReportAccountingService.repairInvoiceStatus(repair, loginSysUser);
    }

    /**
     * opt 积分结算-发票管理-实收报账详情（包括对账单列表）
     *
     */
    public ActualReceivedReportAccountingVO getActualReceivedAccountingVO(ReportAccountingQueryParam param, String token) {

        LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(token);
        return dubboReportAccountingService.getActualReceivedAccountingVO(param, loginSysUser);
    }

    /**
     * 积分结算-发票管理-实收报账单详情（包括对账单列表详情）
     */
    public ActualReceivedReportAccountingDetailVO getActualReceivedReportAccountingDetail(ActualReportAccountingQueryParam param, String token) {
        LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(token);

        return dubboReportAccountingService.getActualReceivedReportAccountingDetail(param, loginSysUser);
    }

    /**
     * opt 积分结算-发票管理-实收报账详情（包括对账单列表详情）
     */
    public R<ActualReportAccountingPreCheckVO> preCheckByCreateActualReceivedAccounting(ActualReportAccountingPreCheckParam param, String token) {
        LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(token);
        return dubboReportAccountingService.preCheckByCreateActualReceivedAccounting(param, loginSysUser);
    }

    /**
     * opt 积分结算-发票管理-实收报账单新增
     *
     */
    public R<?> createActualReceivedAccounting(ActualReportAccountingCreateParam param, String token) {
        LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(token);
        return dubboReportAccountingService.createActualReceivedAccounting(param, loginSysUser);
    }

    /**
     * opt 积分结算-发票管理-实收报账单信息修改
     *
     */
    public void modifyActualReceivedAccounting(ActualReportAccountingModifyParam param, String token) {

        LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(token);
        dubboReportAccountingService.modifyActualReceivedAccounting(param, loginSysUser);
    }

    /**
     * opt 积分结算-发票管理-报账状态查询(结果确认)
     *
     */
    public R<?> confirmActualReceivedAccounting(ActualReportAccountingConfirmParam param, String token) {

        LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(token);
        return dubboReportAccountingService.confirmActualReceivedAccounting(param, loginSysUser);
    }
}
