package cn.superdesk.uniorder.dubbo.consumer.sys;

import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.common.security.JwtUtil;
import cn.superdesk.uniorder.common.security.dto.LoginSysUser;
import cn.superdesk.uniorder.dto.param.system.SysAccountConfigParam;
import cn.superdesk.uniorder.dto.param.system.SysAccountRealityPayableParam;
import cn.superdesk.uniorder.dto.param.system.SysAccountRealityPayableQueryCriteria;
import cn.superdesk.uniorder.dto.vo.sys.SysAccountRealityPayableVo;
import cn.superdesk.uniorder.dubbo.consumer._permission.PermissionDubboCustomerService;
import cn.superdesk.uniorder.dubbo.facade.sys.DubboSysAccountRealityPayableService;
import com.alibaba.dubbo.config.annotation.Reference;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * 实付款报账单
 *
 * @author 阿沐 270897222@qq.com
 * @since 1.0.0 2022-10-28
 */

@Slf4j
@Component
public class SysAccountRealityPayableDubboConsumerService  {

    @Reference(version = "1.0.0", timeout = 30000, retries = -1)
    private DubboSysAccountRealityPayableService dubboSysAccountRealityPayableService;
    @Resource
    private SysAccountConfigDubboConsumerService sysAccountConfigDubboConsumerService;

    @Resource
    private PermissionDubboCustomerService permissionDubboCustomerService;

    public SysAccountRealityPayableVo findById(Long id) {

        return dubboSysAccountRealityPayableService.findById(id);
    }

    public R<List<SysAccountRealityPayableVo>> pageQuery(SysAccountRealityPayableQueryCriteria queryCriteria, String token) {

        LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(token);
        return dubboSysAccountRealityPayableService.findByPaging(queryCriteria);
    }

    public void insert(SysAccountRealityPayableParam param) {

        dubboSysAccountRealityPayableService.insert(param);
    }
    @Transactional(rollbackFor = Exception.class)
    public void updateByPrimaryKeySelective(SysAccountRealityPayableParam param, String token) {

        dubboSysAccountRealityPayableService.updateByPrimaryKeySelective(param);
        LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(token);
        SysAccountConfigParam configParam = new SysAccountConfigParam();
        configParam.setId(param.getAccountConfigId());
        if(loginSysUser!=null){
            configParam.setUserId(String.valueOf(loginSysUser.getSysUserId()));
            configParam.setUserName(loginSysUser.getSysUserName());
        }
        configParam.setUpdateTime(new Date());
        sysAccountConfigDubboConsumerService.updateByPrimaryKeySelective(configParam);
    }

    public void deleteByIds(List<Long> ids) {
        dubboSysAccountRealityPayableService.deleteByIds(ids);
    }

}
