package cn.superdesk.uniorder.dubbo.consumer.sys;

import cn.superdesk.uniorder.common.rest.response.R;
import cn.superdesk.uniorder.common.security.JwtUtil;
import cn.superdesk.uniorder.common.security.dto.LoginSysUser;
import cn.superdesk.uniorder.dto.param.system.SysAccountConfigParam;
import cn.superdesk.uniorder.dto.param.system.SysAccountReceivableParam;
import cn.superdesk.uniorder.dto.param.system.SysAccountReceivableQueryCriteria;
import cn.superdesk.uniorder.dto.vo.sys.SysAccountReceivableVo;
import cn.superdesk.uniorder.dubbo.consumer._permission.PermissionDubboCustomerService;
import cn.superdesk.uniorder.dubbo.facade.sys.DubboSysAccountReceivableService;
import com.alibaba.dubbo.config.annotation.Reference;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;


/**
 * 应收报账单
 *
 * @author zhu 270897222@qq.com
 * @since 1.0.0 2022-10-28
 */
@Slf4j
@Component
public class SysAccountReceivableDubboConsumerService {

    @Reference(version = "1.0.0", timeout = 30000, retries = -1)
    private DubboSysAccountReceivableService dubboSysAccountReceivableService;
    @Resource
    private SysAccountConfigDubboConsumerService sysAccountConfigDubboConsumerService;

    @Resource
    private PermissionDubboCustomerService permissionDubboCustomerService;

    public SysAccountReceivableVo findById(Long id) {

        return dubboSysAccountReceivableService.findById(id);
    }

    public R<List<SysAccountReceivableVo>> pageQuery(SysAccountReceivableQueryCriteria queryCriteria, String token) {

        LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(token);
        return dubboSysAccountReceivableService.findByPaging(queryCriteria);
    }

    public void insert(SysAccountReceivableParam param) {

        dubboSysAccountReceivableService.insert(param);
    }
    @Transactional(rollbackFor = Exception.class)
    public void updateByPrimaryKeySelective(SysAccountReceivableParam param, String token) {

        dubboSysAccountReceivableService.updateByPrimaryKeySelective(param);
        LoginSysUser loginSysUser = JwtUtil.getLoginSysUser(token);
        SysAccountConfigParam configParam = new SysAccountConfigParam();
        configParam.setId(param.getAccountConfigId());
        if(loginSysUser!=null){
            configParam.setUserId(String.valueOf(loginSysUser.getSysUserId()));
            configParam.setUserName(loginSysUser.getSysUserName());
        }
        configParam.setUpdateTime(new Date());
        sysAccountConfigDubboConsumerService.updateByPrimaryKeySelective(configParam);
    }

    public void deleteByIds(List<Long> ids) {
        dubboSysAccountReceivableService.deleteByIds(ids);
    }

}
